package org.exoplatform.velocity.directive;

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.component.UICommand;
import javax.faces.event.ActionListener;

import org.apache.velocity.context.InternalContextAdapter;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.directive.Directive;
import org.apache.velocity.runtime.parser.node.Node;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.event.ActionInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.cms.scripts.CmsScript;
import org.exoplatform.services.cms.scripts.ScriptService;

public class InterceptorDirective extends Directive {

  public String getName() {
    return "interceptor";
  }

  public int getType() {
    return LINE;
  }

  public boolean render(InternalContextAdapter context, Writer writer, Node node)
      throws IOException, ResourceNotFoundException, ParseErrorException,
      MethodInvocationException {
    UISimpleForm uiForm = (UISimpleForm) context.get(UIExoComponent.UICOMPONENT);
    
    String preSave = (String) node.jjtGetChild(0).value(context);
    String postSave = (String) node.jjtGetChild(1).value(context);
    
    ActionListener[] actionListeners = uiForm.getActionListeners();
    for (ActionListener actionListener : actionListeners) {
      if(actionListener instanceof ExoActionListener) {
        ExoActionListener exoListener = (ExoActionListener)actionListener;
        exoListener.removeTransientInterceptors();
        String actionName = exoListener.getActionName();        
        if(!uiForm.getCancelAction().equals(actionName)) {
          ExoActionInterceptor jsfInterceptor = new ExoActionInterceptor();
          
          if(preSave != null && !"".equals(preSave))
            jsfInterceptor.setPreSaveInterceptor(preSave);
          if(postSave != null && !"".equals(postSave))
            jsfInterceptor.setPostSaveInterceptor(postSave);  
          exoListener.addInterceptor(jsfInterceptor);          
        }
      }
    }
    return false;
  }
  
  public class ExoActionInterceptor implements ActionInterceptor {
    private String preSaveInterceptor;
    private String postSaveInterceptor;        
    
    public void setPostSaveInterceptor(String postSaveInterceptor) {
      this.postSaveInterceptor = postSaveInterceptor;
    }

    public void setPreSaveInterceptor(String preSaveInterceptor) {
      this.preSaveInterceptor = preSaveInterceptor;
    }

    public void preExecute(ExoActionEvent event) throws Exception {
      if(preSaveInterceptor == null)
        return;
      UISimpleForm uiForm = (UISimpleForm) event.getSource();
      Map sortedInputs = CMSUtils.prepareMap(uiForm);
      executeScript(preSaveInterceptor, sortedInputs);
    }
    
    public void postExecute(ExoActionEvent event) throws Exception {
    	  UICommand source = (UICommand)event.getComponent();
      if(postSaveInterceptor == null) {
    	    cleanInterceptors(source);
        return;      
      }
      executeScript(postSaveInterceptor,event.getParameter("jcrPath"));
      cleanInterceptors(source);
    }    

    private void executeScript(String script, Object o) {
      PortalContainer pC = PortalContainer.getInstance();
      ScriptService scriptService = (ScriptService) pC
          .getComponentInstanceOfType(ScriptService.class);
      try {
        CmsScript interceptorScript = scriptService.getScript(script);
        interceptorScript.execute(o);
      } catch (Exception e) {
        // TODO use log instead
        e.printStackTrace();
      }
    }    
    
    private String extractNodeName(Set keys) {
      for (Iterator iter = keys.iterator(); iter.hasNext();) {
        String key = (String) iter.next();
        if (key.endsWith("node"))
          return key;
      }
      return null;
    }

    public boolean isTransient() {      
      return true;
    }    

    private void cleanInterceptors(UICommand component) {
      ActionListener[] actionListeners = component.getActionListeners();
      for (ActionListener actionListener : actionListeners) {
        if(actionListener instanceof ExoActionListener) {
          ExoActionListener exoListener = (ExoActionListener)actionListener;
          exoListener.removeTransientInterceptors();
        }
      }
    }
    
  }

}
