package org.exoplatform.portlets.content.admin.component;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.actions.component.UIActionTypeList;
import org.exoplatform.portlets.content.drives.component.UIDriveList;
import org.exoplatform.portlets.content.folksonomy.component.UIFolksonomyManager;
import org.exoplatform.portlets.content.metadata.component.UIMetadataManager;
import org.exoplatform.portlets.content.namespace.component.UINamespaceRegistry;
import org.exoplatform.portlets.content.nodetypes.component.UINodeTypeList;
import org.exoplatform.portlets.content.queries.component.UIQueriesManager;
import org.exoplatform.portlets.content.rules.component.UIRuleList;
import org.exoplatform.portlets.content.script.component.UIScriptsManager;
import org.exoplatform.portlets.content.taxonomy.component.UITaxonomyManager;
import org.exoplatform.portlets.content.templates.component.UITemplatesManager;
import org.exoplatform.portlets.content.views.component.UIManageView;

public class UIECMAdminSummary extends UIExoCommand{

  private static String MANAGE_TEMPLATES_ACTION = "manageTemplates".intern() ;
  private static String MANAGE_SCRIPTS_ACTION = "manageScripts".intern() ;
  private static String MANAGE_RULES_ACTION = "manageRules".intern() ;
  private static String CREATE_ACTIONS_TYPE_ACTION = "createActionType".intern() ;
  private static String ADD_EDIT_NODE_TYPE = "addEditNodeType".intern() ;
  private static String ADD_EDIT_VIEW = "addEditView".intern() ;
  private static String MANAGE_TAXONOMY_ACTION = "manageTaxonomy".intern() ;
  private static String REGISTER_NAMESPACE_ACTION = "registerNamespace".intern() ;
  private static String MANAGE_METADATA_ACTION = "manageMetadata".intern() ;
  private static String MANAGE_DRIVES_ACTION = "manageDrives".intern() ;
  private static String MANAGE_FOLKSONOMY_ACTION = "manageFolksonomy".intern() ;
  private static String MANAGE_QUERIES_ACTION = "managQueries".intern() ;

  static Parameter[] MANAGE_TEMPLATES_PARAMS = { new Parameter(ACTION , MANAGE_TEMPLATES_ACTION)} ;
  static Parameter[] MANAGE_SCRIPTS_PARAMS = { new Parameter(ACTION , MANAGE_SCRIPTS_ACTION)} ;
  static Parameter[] MANAGE_RULES_PARAMS = { new Parameter(ACTION , MANAGE_RULES_ACTION)} ;
  static Parameter[] CREATE_MOVE_TYPE_PARAMS = { new Parameter(ACTION , CREATE_ACTIONS_TYPE_ACTION)} ;  
  static Parameter[] ADD_EDIT_NODE_TYPE_PARAMS = { new Parameter(ACTION , ADD_EDIT_NODE_TYPE)} ;
  static Parameter[] ADD_EDIT_VIEW_PARAMS = { new Parameter(ACTION , ADD_EDIT_VIEW)} ;
  static Parameter[] MANAGE_TAXONOMY_PARAMS = {new Parameter(ACTION, MANAGE_TAXONOMY_ACTION)} ;
  static Parameter[] REGISTER_NAMESPACE_PARAMS = {new Parameter(ACTION,REGISTER_NAMESPACE_ACTION)} ;
  static Parameter[] MANAGE_METADATA_PARAMS = {new Parameter(ACTION,MANAGE_METADATA_ACTION)} ;
  static Parameter[] MANAGE_DRIVES_PARAMS = {new Parameter(ACTION,MANAGE_DRIVES_ACTION)} ;
  static Parameter[] MANAGE_FOLKSONOMY_PARAMS = {new Parameter(ACTION,MANAGE_FOLKSONOMY_ACTION)} ;
  static Parameter[] MANAGE_QUERIES_PARAMS = {new Parameter(ACTION,MANAGE_QUERIES_ACTION)} ;


  public UIECMAdminSummary() {
    setRendererType("VelocityRenderer") ;
    setId("UICMSAdminSummary");    

    addActionListener(ManageTemplatesActionListener.class, MANAGE_TEMPLATES_ACTION);
    addActionListener(ManageScriptsActionListener.class, MANAGE_SCRIPTS_ACTION);
    addActionListener(ManageRulesActionListener.class, MANAGE_RULES_ACTION);
    addActionListener(CreateActionTypeActionListener.class, CREATE_ACTIONS_TYPE_ACTION);
    addActionListener(ManageNodeTypeListener.class, ADD_EDIT_NODE_TYPE);
    addActionListener(ManageViewListener.class, ADD_EDIT_VIEW);
    addActionListener(ManageTaxonomyActionListener.class, MANAGE_TAXONOMY_ACTION) ;
    addActionListener(RegisterNamespaceActionListener.class,REGISTER_NAMESPACE_ACTION) ;
    addActionListener(ManageMetadataActionListener.class,MANAGE_METADATA_ACTION) ;
    addActionListener(ManageDrivesActionListener.class,MANAGE_DRIVES_ACTION) ;
    addActionListener(ManageFolksonomyActionListener.class,MANAGE_FOLKSONOMY_ACTION) ;    
    addActionListener(ManageQueriesActionListener.class,MANAGE_QUERIES_ACTION) ;
  }

  public Parameter[] getManageTemplatesParams() { return MANAGE_TEMPLATES_PARAMS ; }
  public Parameter[] getManageScriptsParams() { return MANAGE_SCRIPTS_PARAMS ; }
  public Parameter[] getManageRulesParams() { return MANAGE_RULES_PARAMS ; }
  public Parameter[] getCreateActionTypeParams() { return CREATE_MOVE_TYPE_PARAMS ; }  
  public Parameter[] getAddEditNodeTypeParams() { return ADD_EDIT_NODE_TYPE_PARAMS ; }
  public Parameter[] getAddEditViewParams() { return ADD_EDIT_VIEW_PARAMS ; }
  public Parameter[] getManageTaxonomyParams() { return MANAGE_TAXONOMY_PARAMS ; }
  public Parameter[] getRegisterNamespaceParams() { return REGISTER_NAMESPACE_PARAMS ;} 	
  public Parameter[] getMetadataManagerParams() { return MANAGE_METADATA_PARAMS ;}
  public Parameter[] getDrivesManagerParams() { return MANAGE_DRIVES_PARAMS ;}
  public Parameter[] getManageFolksonomyParams() { return MANAGE_FOLKSONOMY_PARAMS;}
  public Parameter[] getQueriesManagerParams() { return MANAGE_QUERIES_PARAMS ;}

  static public class ManageTemplatesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;      
      uicomp.setRenderedSibling(UITemplatesManager.class) ;
    }
  }    

  static public class ManageScriptsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;      
      uicomp.setRenderedSibling(UIScriptsManager.class) ;
    }
  }    

  static public class ManageRulesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;      
      uicomp.setRenderedSibling(UIRuleList.class) ;
    }
  }      

  static public class CreateActionTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;      
      uicomp.setRenderedSibling(UIActionTypeList.class) ;
    }
  }

  static public class ManageNodeTypeListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;      
      uicomp.setRenderedSibling(UINodeTypeList.class) ;
    }
  }

  static public class ManageViewListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;      
      uicomp.setRenderedSibling(UIManageView.class) ;
    }
  }

  static public class ManageTaxonomyActionListener extends ExoActionListener {		
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;
      uicomp.setRenderedSibling(UITaxonomyManager.class) ;
    }		
  }

  static public class RegisterNamespaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;
      uicomp.setRenderedSibling(UINamespaceRegistry.class) ;
    }		
  }

  static public class ManageMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;      
      ((UIMetadataManager)uicomp.getSibling(UIMetadataManager.class)).update() ;      
      uicomp.setRenderedSibling(UIMetadataManager.class) ;
    }    
  }

  static public class ManageDrivesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;
      uicomp.setRenderedSibling(UIDriveList.class) ;
    }    
  }

  static public class ManageFolksonomyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;
      uicomp.setRenderedSibling(UIFolksonomyManager.class) ;
    }    
  }

  static public class ManageQueriesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMAdminSummary uicomp = (UIECMAdminSummary) event.getComponent() ;
      uicomp.setRenderedSibling(UIQueriesManager.class) ;
    }    
  }
}
