/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.contentbrowser.component;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.StringWriter;
import java.security.AccessControlException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Stack;

import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.portlet.ActionResponse;
import javax.portlet.PortletPreferences;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletResponse;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.runtime.resource.ResourceManager;
import org.exoplatform.Constants;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDelegate;
import org.exoplatform.faces.cms.CMSViewComponent;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.JCRSessionManager;
import org.exoplatform.services.cms.transform.TransformationService;
import org.exoplatform.services.download.DownloadResource;
import org.exoplatform.services.download.DownloadService;
import org.exoplatform.services.download.InputStreamDownloadResource;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.velocity.utils.VelocityUtils;

/**
 * Created by The eXo Platform SARL
 * Author : HoaPham
 *          phamvuxuanhoa@yahoo.com
 * Jan 20, 2006
 */
public class UIDetailDocument extends UIExoCommand implements ChangeNodeListener, CMSViewComponent{     
  final public static String CLOSE_DOCUMENT_ACTION = "closeDocument".intern() ;  
  final public static String VIEW_RELATED_ACTION = "viewRelated".intern() ;
  final static public String CHANGE_LANGUAGE_ACTION = "changeLanguage" ;
  final static public String VOTE_ACTION  = "voteAction".intern() ;
  final static public String COMMENT_ACTION = "commentAction".intern() ;
  final static public String EXPORT_ACTION = "exportAction".intern() ;
  final static public String DEFAULT_LANGGUAGE = "default".intern() ;

  static Parameter[] CLOSE_DOCUMENT_PARAMS = { new Parameter(ACTION, CLOSE_DOCUMENT_ACTION), new Parameter(Constants.WINDOW_STATE_PARAMETER,WindowState.NORMAL.toString()) };
  static Parameter[] VIEW_RELATED_PARAMS = {new Parameter(ACTION,VIEW_RELATED_ACTION)} ;
  public static Parameter[] CHANGE_LANGUAGE_ACTION_PARAMS   = { new Parameter(ACTION,CHANGE_LANGUAGE_ACTION) } ;
  static Parameter[] VOTE_PARAMS = { new Parameter(ACTION,VOTE_ACTION)} ;
  static Parameter[] COMMENT_PARAMS = { new Parameter(ACTION,COMMENT_ACTION)} ;
  static Parameter[] EXPORT_PARAMS = { new Parameter(ACTION,EXPORT_ACTION)} ;

  private CMSDelegate delegate_;
  private String templatePath_ ;  
  private Class opener_ ;
  private String language_ = DEFAULT_LANGGUAGE ;  
  private Stack<Node> relationsHistory_ = new Stack<Node>() ;
  private JCRSessionManager sessionManager_ ;
  PortletPreferences prefs_ = FacesUtil.getPortletPreferences() ;  

  public UIDetailDocument (JCRSessionManager sessionManager) throws Exception {
    setId("DetailDocument") ;
    setRendererType("VelocityRenderer");    
    sessionManager_ = sessionManager ;

    addChild(UIVotingForm.class).setRendered(false);    
    addChild(UICommentsForm.class).setRendered(true) ;

    addActionListener(CloseDocumentActionListener.class, CLOSE_DOCUMENT_ACTION) ;
    addActionListener(ViewRelationActionListener.class, VIEW_RELATED_ACTION) ;
    addActionListener(ChangeLanguageActionListener.class,CHANGE_LANGUAGE_ACTION) ;
    addActionListener(VoteActionListener.class,VOTE_ACTION) ;
    addActionListener(ExportToPdfActionListener.class,EXPORT_ACTION) ;

  }

  public Parameter[] getCloseDocumentParams() { return CLOSE_DOCUMENT_PARAMS ; }	
  public Parameter[] getViewNodeParams() { return VIEW_RELATED_PARAMS; }
  public Parameter[] getChangeLanguageParams() {return CHANGE_LANGUAGE_ACTION_PARAMS;  }
  public Parameter[] getVoteParams() { return VOTE_PARAMS ; }
  public Parameter[] getExportParams() { return EXPORT_PARAMS ; }

  public Class getOpener() { return this.opener_ ; }

  public void setOpener(Class onpener) {
    this.opener_ = onpener ;    
  }

  public boolean isDocumentConfig() {    
    String usecase = prefs_.getValue("usecase", "") ;
    if(usecase.equals(CBUtils.USE_DOCUMENT))
      return true ;
    return false ;
  }

  public boolean allowVoting() throws Exception {
    Node document = delegate_.getNode() ;
    if(!document.isNodeType("mix:votable")) return false;    
    return Boolean.parseBoolean(prefs_.getValue(CBUtils.ALLOW_VOTING,"")) ;
  }

  public boolean allowComment() throws Exception {
    Node document = delegate_.getNode() ;
    if(!document.isNodeType("mix:commentable")) return false ;
    return Boolean.parseBoolean(prefs_.getValue(CBUtils.ALLOW_COMMENT,"")) ;
  }

  public boolean allowExporting() throws Exception {
    return Boolean.parseBoolean(prefs_.getValue(CBUtils.ALLOW_COMMENT,"")) ;
  }

  public void onChange(UIContentExplorer uiExplorer, Node node) throws Exception {
    setRendered(false) ;    
  }  

  public void setTemplatePath(String templatePath) {
    this.templatePath_ = templatePath ;
  }

  public String getTemplateToView() {
    return "jcr:"+templatePath_ ;     
  }

  public boolean isReadAuthorized(Node node) throws Exception {
    ExtendedNode extendedNode = (ExtendedNode)node ;
    try {
      extendedNode.checkPermission(PermissionType.READ) ;      
    }catch (AccessControlException e) {
      return false ;
    }
    return true ;
  }  

  public Node getNodeByUUID(String uuid) throws Exception{
    String applicationId = SessionContainer.getInstance().getSessionId() ;
    Session session = sessionManager_.openSession(getWorkspaceName(),applicationId);
    Node node = session.getNodeByUUID(uuid);
    return node;
  }

  public void setNodeByUri(String uri) throws Exception{
    String workspace = uri.substring(0, uri.indexOf(CBUtils.COLON));
    String applicationId = SessionContainer.getInstance().getSessionId() ;
    Session session = sessionManager_.openSession(workspace,applicationId) ;
    uri = uri.substring(uri.indexOf(CBUtils.COLON) + 1);
    Node node = (Node)session.getItem(uri);    
    setNode(node);    
    setTemplatePath(delegate_.getTemplatePath()) ;
  }

  public boolean isRssLink() { return delegate_.isRssLink() ; }
  public String getRssLink() { return delegate_.getRssLink() ; }

  public List<Node> getRelations() throws Exception {
    return delegate_.getRelations();
  }

  public List<Node> getAttachments() throws Exception {
    return delegate_.getAttachments();
  }

  public String getAttachmentSize(Node node){
    return delegate_.getAttachmentSize(node);
  }

  public Node getNode() throws Exception {
    Node node = delegate_.getNode() ;
    if(!node.hasProperty("exo:language")) return node ;
    String defaultLang = node.getProperty("exo:language").getString() ;
    if(!DEFAULT_LANGGUAGE.equals(language_) && !language_.equals(defaultLang)) {
      Node curNode = node.getNode("languages/" + language_) ;
      return curNode ;
    }
    return node;    
  }

  public List<Value> getMultiValues(Node node, String name) throws Exception {    
    List<Value> list = new ArrayList<Value>() ;   
    if(!node.hasProperty(name)) return list ;
    if(!node.getProperty(name).getDefinition().isMultiple()){
      list.add(node.getProperty(name).getValue()) ;
      return list ;
    }    
    Value[] values =node.getProperty(name).getValues() ;    
    for( Value value:values) { list.add(value) ;}            
    return list ;
  }

  public List getSupportedLocalise() throws Exception {
    List<String> locales = new ArrayList<String> () ;
    Node curNode = delegate_.getNode() ;
    String defaultLocale = curNode.getProperty("exo:language").getString() ;
    if(curNode.hasNode("languages")){
      Node languages = curNode.getNode("languages") ;       
      for(NodeIterator iter = languages.getNodes();iter.hasNext();) {
        locales.add(iter.nextNode().getName()) ;
      }                                     
    }    
    if(Collections.frequency(locales,defaultLocale)==0) {
      locales.add(defaultLocale) ;
    }

    return locales ;
  }

  public void setNode(Node node) {
    if(delegate_ == null)
      delegate_ = new CMSDelegate(node);
    else
      delegate_.setNode(node);

    UICommentsForm commentsForm = (UICommentsForm)getChildComponentOfType(UICommentsForm.class) ;
    commentsForm.setDocument2Comments(node) ;
  }

  public String getPortalName() {
    return delegate_.getPortalName();
  }

  public String getWorkspaceName() throws Exception {    
    return delegate_.getWorkspaceName();
  }

  public String getNodeUri(Node node) throws Exception {
    return getWorkspaceName() + CBUtils.COLON + node.getPath();
  }

  public boolean isNodeTypeSupported() {
    return delegate_.isNodeTypeSupported();
  }

  public Object getComponentInstanceOfType(String className) {
    return delegate_.getComponentInstanceOfType(className);
  }  

  public WikiContext getWikiContext() {
    return delegate_.getWikiContext();
  }

  public WikiEngineService getWikiService() {
    return delegate_.getWikiService();
  }

  public String getNodeType() throws Exception {
    return delegate_.getNodeType();
  }

  public String getDocumentTemplatePath() throws Exception {    
    return getTemplatePath() ;
  }      

  public String getLanguage() { return language_; }
  public void setLanguage(String language) { this.language_ = language; }

  public boolean hasPropertyContent(Node node, String property){
    return delegate_.hasPropertyContent(node, property);
  }

  public String getTemplatePath() throws Exception {
    return delegate_.getTemplatePath() ;
  }

  public String getLargeNodeTypeIcon(Node node) {   
    return delegate_.getLargeNodeTypeIcon(node);
  }

  public String getPropertyIconByName(String propertyName) {
    return delegate_.getPropertyIconByName(propertyName);
  }

  public String getPropertyTypeIcon(Property property) {
    return delegate_.getPropertyTypeIcon(property);
  }

  public String getSmallNodeTypeIcon(Node node) {
    return delegate_.getSmallNodeTypeIcon(node);
  }

  static public class ViewRelationActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDetailDocument uicomp = (UIDetailDocument)event.getComponent() ;
      String uri = event.getParameter(OBJECTID) ;
      Node node = uicomp.getNode() ;
      uicomp.relationsHistory_.push(node) ;
      uicomp.setNodeByUri(uri) ;
      uicomp.setRenderedSibling(UIDetailDocument.class);
    }    
  } 

  public static class CloseDocumentActionListener extends ExoActionListener{
    public void execute(ExoActionEvent event) throws Exception {
      UIDetailDocument uicomp= (UIDetailDocument)event.getComponent() ;            
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
      boolean usePathConfig = Boolean.parseBoolean(prefs.getValue(CBUtils.FORM_PATH,"")) ;            
      if(uicomp.getParent() instanceof UIContentExplorer) {
        UIContentExplorer uiExplorer = (UIContentExplorer)uicomp.getParent() ;               
        if(!uicomp.relationsHistory_.isEmpty()) {
          Node node = uicomp.relationsHistory_.pop() ;
          uicomp.setNode(node) ;
          uicomp.setRenderedSibling(UIDetailDocument.class) ;
          return ;
        }
        if(usePathConfig) {
          Information iprovider = findInformationProvider(uicomp);
          try {
            Node node = uiExplorer.getSelectNode() ;
            uiExplorer.setSelectNode(node) ;
          }catch (Exception e) {
            JCRExceptionManager.process(iprovider, e);
          }
        }            
        boolean isShowingTagsCloud = Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TAGS_CLOUD,"")) ;
        boolean isShowingToolbar= Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TOOLBAR,"")) ;
        uiExplorer.setShowingTagsCloud(isShowingTagsCloud) ;
        uiExplorer.setShowingToolbar(isShowingToolbar) ;
        Class opener = uicomp.getOpener() ;        
        uicomp.getSibling(opener).setRendered(true) ;
        //contentList.setRendered(true) ;
        uicomp.setRendered(false) ;
      } else if(uicomp.getParent() instanceof UISearch) {        
        uicomp.setRenderedSibling(UISearchController.class) ;        
      } else if(!uicomp.relationsHistory_.empty()) {
        Node node = uicomp.relationsHistory_.pop() ;
        uicomp.setNode(node) ;
        uicomp.setRenderedSibling(UIDetailDocument.class) ;
      }
      ExternalContext eContext = uicomp.getFacesContext().getExternalContext();
      ActionResponse response = (ActionResponse) eContext.getResponse();
      response.setWindowState(WindowState.NORMAL);
    }  
  } 

  static public class ChangeLanguageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDetailDocument uicomp = (UIDetailDocument) event.getComponent();
      uicomp.setLanguage(event.getParameter(OBJECTID));
    }   
  }

  static public class VoteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDetailDocument uicomp = (UIDetailDocument) event.getComponent();
      UIVotingForm votingForm = (UIVotingForm)uicomp.getChildComponentOfType(UIVotingForm.class) ;      
      votingForm.setDocument2Vote(uicomp.getNode()) ;
      votingForm.setRendered(!votingForm.isRendered()) ;
    }    
  }

  static public class ExportToPdfActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {
      UIDetailDocument uicomp = (UIDetailDocument) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;      
      Node currentNode = uicomp.getNode() ;
      String sourceMimeType = null ;
      InputStream inputStream = null ;             
      if(!currentNode.isNodeType("nt:file")) {    
        String docTemplatePath = "jcr:"+uicomp.getTemplatePath() ;
        String htmlContent = getDocumentContentAsHTML(docTemplatePath,res) ;
        if(htmlContent == null) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR , 
              res.getString("UIDocumentContent.msg.export-error"))) ;          
          return ;
        }
        inputStream = new ByteArrayInputStream(htmlContent.getBytes()) ;
        sourceMimeType = TransformationService.EXO_MIMETYPE ;        
      }else {
        Node content = currentNode.getNode("jcr:content") ;
        sourceMimeType = content.getProperty("jcr:mimeType").getString() ;      
        inputStream = content.getProperty("jcr:data").getStream() ; 
      }      
      TransformationService transformService = 
        (TransformationService)PortalContainer.getComponent(TransformationService.class) ;
      DownloadService downloadService = 
        (DownloadService)PortalContainer.getComponent(DownloadService.class) ;     
      ByteArrayOutputStream outputStream = new ByteArrayOutputStream() ;
      String pdfMimeType = "application/pdf" ;
      boolean success = true ;
      try{        
        success = transformService.transform(inputStream,sourceMimeType,outputStream,pdfMimeType) ;
      }catch (Exception e) {
        success = false ;
      }      
      if(!success) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR , 
            res.getString("UIDocumentContent.msg.export-error"))) ;
        return ;
      }        
      ByteArrayInputStream input = new ByteArrayInputStream(outputStream.toByteArray()) ;
      DownloadResource downloadResource = new InputStreamDownloadResource(input,pdfMimeType) ;
      downloadResource.setDownloadName(currentNode.getName().concat(".pdf")) ;
      String id = downloadService.addDownloadResource(downloadResource) ;
      String downloadLink = downloadService.getDownloadLink(id) ;
      FacesContext facesContext = FacesContext.getCurrentInstance() ;
      HttpServletResponse response =(HttpServletResponse)facesContext.getExternalContext().getResponse();           
      response.sendRedirect(downloadLink);
      facesContext.responseComplete();
    }

    private String getDocumentContentAsHTML(String jcrTemplatePath,ResourceBundle res) {
      ResourceManager manager = 
        (ResourceManager)PortalContainer.getInstance().getComponentInstanceOfType(ResourceManager.class) ;
      VelocityContext vContext = new VelocityContext() ;
      vContext.put("uicomponent",this) ;
      vContext.put("res",res) ;
      vContext.put("VelocityUtils",new VelocityUtils()) ;
      vContext.put("DateFormater",new SimpleDateFormat()) ;
      StringWriter writer = new StringWriter() ;
      try {
        Template template = (Template)manager.getResource(jcrTemplatePath,ResourceManager.RESOURCE_TEMPLATE,"UTF-8") ;      
        template.merge(vContext,writer) ;
        writer.flush() ;
        return writer.toString() ;
      }catch (Exception e) {
      }    
      return null ;
    }
  }

  public Locale getCurrentLocale() {
    return delegate_.getCurrentLocale();
  }
}
