package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.AccessDeniedException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.portlet.PortletPreferences;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.jcr.RepositoryService;

public class UIDocumentConfig extends UISimpleForm implements SelectorHandler{

  final public static String NEW_CONFIG_ACTION = "newConfig".intern() ;
  final public static String EDIT_CONFIG_ACTION = "editConfig".intern() ;
  final public static String SELECT_PATH_ACTION = "selectPath".intern() ;
  final public static String SELECT_DOCUMENT_ACTION = "selectDocument".intern() ;

  private UIStringInput pathInput_ ;  		  
  private UIStringInput nbPerPage_ ;	
  private UISelectBox boxTemplateSelectBox_ ; 
  private UIStringInput documentNameInput_ ;  
  protected UISelectBox workspaces_ ;
  private UICheckBox allowVoting_ ;
  private UICheckBox allowComment_ ;
  private UICheckBox allowExporting_ ;
  private boolean editable_ ;
  private boolean isChangedForm_ ; 
  private boolean isSelectPath_ = true ;

  public UIDocumentConfig() throws Exception {
    super("DocumentConfig","post",null) ;		
    setId("DocumentConfig") ;
    setRendererType("SimpleFormVelocityRenderer");		

    PortletPreferences prefs = FacesUtil.getPortletPreferences();            

    String[] workspaceNames = CBUtils.getWorkspaceNames() ;    
    List<SelectItem> workspaceOptions = new ArrayList<SelectItem>() ;    
    for(String workspace:workspaceNames) {
      workspaceOptions.add(new SelectItem(workspace,workspace)) ;
    }   
    workspaces_ = new  UISelectBox(CBUtils.WORKSPACE,prefs.getValue(CBUtils.WORKSPACE,""), workspaceOptions) ;
    add(workspaces_);

    pathInput_ = new UIStringInput(CBUtils.JCR_PATH, prefs.getValue(CBUtils.JCR_PATH,"")) ;
    add(pathInput_) ;    
    documentNameInput_ = new UIStringInput(CBUtils.DOCUMENT_NAME,prefs.getValue(CBUtils.DOCUMENT_NAME,"")) ;
    add(documentNameInput_) ;        

    nbPerPage_ = new UIStringInput(CBUtils.NB_PER_PAGE, prefs.getValue(CBUtils.NB_PER_PAGE,"")) ;
    add(nbPerPage_) ;           

    boxTemplateSelectBox_ = new UISelectBox(CBUtils.BOX_TEMPLATE,"",new ArrayList<SelectItem>()) ;
    add(boxTemplateSelectBox_) ; 

    allowVoting_ = new UICheckBox(CBUtils.ALLOW_VOTING,prefs.getValue(CBUtils.ALLOW_VOTING,"")) ;
    add(allowVoting_) ;

    allowComment_ = new UICheckBox(CBUtils.ALLOW_COMMENT,prefs.getValue(CBUtils.ALLOW_COMMENT,"")) ;
    add(allowComment_) ;
    
    allowExporting_ = new UICheckBox(CBUtils.ALLOW_EXPORTING,prefs.getValue(CBUtils.ALLOW_EXPORTING,"")) ;
    add(allowExporting_) ;
    
    updateSelectBoxes() ;

    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;       
    addActionListener(NewConfigActionListener.class,NEW_CONFIG_ACTION) ;
    addActionListener(EditConfigActionListener.class,EDIT_CONFIG_ACTION) ;    
    addActionListener(SelectPathActionListener.class,SELECT_PATH_ACTION) ;
    addActionListener(SelectDocumentActionListener.class,SELECT_DOCUMENT_ACTION) ;
  }

  public boolean isEditable() { return editable_ ;}  

  public String getEditConfigParams() { return EDIT_CONFIG_ACTION ;}
  public String getNewConfigParams() { return NEW_CONFIG_ACTION ; }  
  public String getSelectPathParams() { return SELECT_PATH_ACTION ; }
  public String getSelectDocumentParams() { return SELECT_DOCUMENT_ACTION ; }

  public void updateFormValues() throws Exception {
    PortletPreferences prefs = FacesUtil.getPortletPreferences();
    workspaces_.setValue(prefs.getValue(CBUtils.WORKSPACE,"")) ;
    pathInput_.setValue(prefs.getValue(CBUtils.JCR_PATH,"")) ;             
    nbPerPage_.setValue(prefs.getValue(CBUtils.NB_PER_PAGE,"")) ;  
    boxTemplateSelectBox_.setValue(prefs.getValue(CBUtils.BOX_TEMPLATE,"")) ;    
    documentNameInput_.setValue(prefs.getValue(CBUtils.DOCUMENT_NAME,"")) ;
    allowComment_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_COMMENT,""))) ;
    allowVoting_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_VOTING,""))) ;
  }

  public void updateSelectBoxes() throws Exception {
    ManageViewService viewService = 
      (ManageViewService)PortalContainer.getComponent(ManageViewService.class) ;
    List<SelectItem> docTempOptions = new ArrayList<SelectItem>() ;
    List<Node> docTemplates = viewService.getAllTemplates(BasePath.CB_DETAIL_VIEW_TEMPLATES) ;
    for(Node template: docTemplates) {
      docTempOptions.add(new SelectItem(template.getName(),template.getName())) ;
    }    
    boxTemplateSelectBox_.setOptions(docTempOptions) ;
  }

  public void setEditable(boolean editable) {  
    editable_ = editable ;
    workspaces_.setEditable(editable_) ;
    pathInput_.setEditable(editable_) ;          
    nbPerPage_.setEditable(editable_) ;  
    boxTemplateSelectBox_ .setEditable(editable_);   
    documentNameInput_.setEditable(editable_) ;
    allowComment_.setEditable(editable_) ;
    allowVoting_.setEditable(editable_) ;
  }    
  public void setChangedForm(boolean isChangedForm) {
    isChangedForm_ = isChangedForm ;
  }  

  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIDocumentConfig.class);
  }

  public void select(Selector selector) throws Exception {
    if(isSelectPath_) pathInput_.setValue(selector.getInputValue()) ;
    else {
      String path = pathInput_.getValue() ;
      String document = selector.getInputValue() ;
      if(path == null) {
        documentNameInput_.setValue(document.substring(1)) ;
      }else {
        documentNameInput_.setValue(selector.getInputValue().substring(path.length() + 1)) ;
      }
    }
    selector.setRenderedSibling(UIDocumentConfig.class);
  }   

  static public class SelectPathActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentConfig uicomp = (UIDocumentConfig) event.getComponent() ;
      uicomp.isSelectPath_ = true ;
      JcrBrowser jcrBrowser = (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setFilter(null) ;
      jcrBrowser.setHandler(uicomp) ;
      jcrBrowser.setParameters(new String[] {null, "/"});
      uicomp.setRenderedSibling(JcrBrowser.class);   
    }
  } 

  static public class SelectDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentConfig uicomp = (UIDocumentConfig) event.getComponent() ;
      PortalContainer pC = PortalContainer.getInstance();
      TemplateService tservice = (TemplateService)pC.getComponentInstanceOfType(TemplateService.class) ;
      uicomp.isSelectPath_ = false ;
      JcrBrowser jcrBrowser = 
        (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setHandler(uicomp) ;
      jcrBrowser.setFilter(tservice.getDocumentTemplates()) ;
      String path = uicomp.pathInput_.getValue() ;
      if(path != null) jcrBrowser.setParameters(new String[] {null, path});
      else jcrBrowser.setParameters(new String[] {null, "/"});
      uicomp.setRenderedSibling(JcrBrowser.class);   
    }
  }

  public static class NewConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentConfig documentConfig = (UIDocumentConfig)event.getSource() ;
      documentConfig.setRenderedSibling(UINewConfigForm.class) ;
    }    
  }      

  public static class EditConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentConfig documentConfig = (UIDocumentConfig)event.getSource() ;
      documentConfig.setEditable(true) ;
      documentConfig.updateSelectBoxes() ;
      documentConfig.setChangedForm(false) ;
    }    
  }        

  public static class CancelActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentConfig documentConfig = (UIDocumentConfig)event.getSource() ;
      if(documentConfig.isChangedForm_) {
        documentConfig.setRenderedSibling(UINewConfigForm.class) ;
      }else {
        documentConfig.setEditable(false) ;
      }
    }    
  }  

  public static class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentConfig documentConfig = (UIDocumentConfig)event.getSource() ;  
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;

      String workspace = documentConfig.getUIInput(CBUtils.WORKSPACE).getValue() ;
      String path = documentConfig.pathInput_.getValue() ;                            
      String documentName = documentConfig.getUIInput(CBUtils.DOCUMENT_NAME).getValue() ;

      String nbPerPage = documentConfig.getUIInput(CBUtils.NB_PER_PAGE).getValue() ;
      String boxTemplate = documentConfig.getUISelectBox(CBUtils.BOX_TEMPLATE).getValue() ;            
      String usecase = CBUtils.USE_DOCUMENT ;

      UICheckBox allowVotingBox = (UICheckBox)documentConfig.getUIInput(CBUtils.ALLOW_VOTING) ;
      UICheckBox allowCommentBox = (UICheckBox)documentConfig.getUIInput(CBUtils.ALLOW_COMMENT) ;      
      String allowVoting = Boolean.toString(allowVotingBox.getSelect()) ;
      String allowComment = Boolean.toString(allowCommentBox.getSelect());
      UICheckBox allowExportingBox = (UICheckBox)documentConfig.getUIInput(CBUtils.ALLOW_EXPORTING) ;
      String allowExporting = Boolean.toString(allowExportingBox.getSelect()) ;
      
      Information iprovider = findInformationProvider(documentConfig);			
      ResourceBundle res = getApplicationResourceBundle() ;
      Session session = null ;
      Node node = null ;
      if(documentName == null || documentName.length() == 0) {
        iprovider.addMessage(
            new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,res.getString("UIContentBrowserConfig.msg.document-name-is-null"))) ;
        return ;
      }
      if(path == null || path.length() == 0) 
        path = "/" ;
      try {
        RepositoryService repositoryService = 
          (RepositoryService)PortalContainer.getComponent(RepositoryService.class) ;
        if(path.startsWith("/jcr:system")) {
          session = repositoryService.getRepository().getSystemSession(workspace) ; 
        } else {
          session = repositoryService.getRepository().login(workspace) ;
        }
        node = (Node)session.getItem(path) ;		
        Node document = node.getNode(documentName) ;
        if(!CBUtils.isDocument(document.getPrimaryNodeType())) 
          throw new ItemNotFoundException(path+"/"+documentName) ;        
      } catch (AccessDeniedException ae) {     
        if(session!=null) { session.logout(); }
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
            res.getString("UIContentBrowserConfig.msg.unable-access"),new Object[]{path})) ;
        return ;
      }catch (Exception e) {
        if(session!=null) { session.logout(); }
        JCRExceptionManager.process(iprovider,e) ;
        return ;
      }
      session.logout() ;
      prefs.setValue(CBUtils.WORKSPACE,workspace) ;
      prefs.setValue(CBUtils.JCR_PATH,path) ;      
      prefs.setValue(CBUtils.NB_PER_PAGE,nbPerPage) ;      
      prefs.setValue(CBUtils.BOX_TEMPLATE,boxTemplate) ;              
      prefs.setValue(CBUtils.USECASE,usecase) ;
      prefs.setValue(CBUtils.DOCUMENT_NAME,documentName) ;      
      prefs.setValue(CBUtils.ALLOW_VOTING,allowVoting) ;
      prefs.setValue(CBUtils.ALLOW_COMMENT,allowComment) ;
      prefs.setValue(CBUtils.ALLOW_EXPORTING,allowExporting) ;      
      prefs.setValue(CBUtils.QUERY_LANGUAGE,getValue(prefs,CBUtils.QUERY_LANGUAGE)) ;
      prefs.setValue(CBUtils.QUERY_STATEMENT,getValue(prefs,CBUtils.QUERY_STATEMENT)) ;
      prefs.setValue(CBUtils.IS_NEW_QUERY,getValue(prefs,CBUtils.IS_NEW_QUERY)) ;
      prefs.setValue(CBUtils.IS_PERSONAL_QUERY,getValue(prefs,CBUtils.IS_PERSONAL_QUERY)) ;
      prefs.setValue(CBUtils.QUERY_STORE,getValue(prefs,CBUtils.QUERY_STORE)) ;      
      prefs.setValue(CBUtils.TEMPLATE,getValue(prefs,CBUtils.TEMPLATE)) ;         
      prefs.setValue(CBUtils.REF_DOCUMENT,getValue(prefs,CBUtils.REF_DOCUMENT)) ;
      prefs.setValue(CBUtils.CHILD_DOCUMENT,getValue(prefs,CBUtils.CHILD_DOCUMENT)) ;    
      prefs.setValue(CBUtils.VIEW_TOOLBAR,getValue(prefs,CBUtils.VIEW_TOOLBAR)) ;               
      prefs.setValue(CBUtils.SCRIPT_NAME, getValue(prefs,CBUtils.SCRIPT_NAME)) ;
      prefs.setValue(CBUtils.VIEW_TAGS_CLOUD,getValue(prefs,CBUtils.VIEW_TAGS_CLOUD)) ;      
      
      prefs.store() ;
      iprovider.addMessage(
          new ExoFacesMessage(FacesMessage.SEVERITY_INFO,res.getString("UIContentBrowserConfig.msg.save-config-successfully"))) ;
      documentConfig.updateFormValues() ;
      documentConfig.setEditable(false) ;
    }
    
    private String getValue(PortletPreferences pref,String name) {      
      String value = pref.getValue(name,"") ;
      if(value == null) value = "" ;
      return value ;
    }
  }
  
}
