package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.jcr.Node;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;


public class UIResults extends UIExoCommand{
  
  public final static int NB_PER_PAGE = 5 ;
  protected List<Node> results_  = new ArrayList<Node>();
  protected long duration_;  
  protected Node node_ ;
  protected String searchType_ ;
  private UIPageListIterator uiPageListIterator_ ;
  
  static Parameter[] CHANGE_NODE_PARAMS = { new Parameter(ACTION,UIContentExplorer.CHANGE_NODE_ACTION) };    
  
  public UIResults() throws Exception{
    setId("UIResults") ;
    setRendererType("VelocityRenderer") ;
    uiPageListIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageListIterator_) ;
    update() ;
    
    addActionListener(ChangeNodeActionListener.class, UIContentExplorer.CHANGE_NODE_ACTION);
  }
  
  public Parameter[] getChangeNodeParams() { return CHANGE_NODE_PARAMS; }    
  
  public void update() throws Exception {
    if(hasResults()) {
      List<Node> pageList = getResultNodes() ;
      ObjectPageList objectPageList = new ObjectPageList(pageList,NB_PER_PAGE) ;
      uiPageListIterator_.setPageList(objectPageList) ;
    }    
  }
  
  public Collection<Node> getPageListData() throws Exception {
   return uiPageListIterator_.getPageListDataHandler().getDatas() ; 
  }
  
  public boolean hasResults() throws Exception {
    if(results_.isEmpty()) return false;    
    for(Node node:results_) {    	
    	if(node.getPath().startsWith(node_.getPath())) return true ;
    }
    return false ;
  }
  
  public long searchTime() { return duration_; }        
  
  public List<Node> getResultNodes() throws Exception {
  	List<Node> categories = new ArrayList<Node>() ;
  	for(Node node:results_) {
  		if(node.getPath().startsWith(node_.getPath())) {
  			categories.add(node) ;
  		}
  	}
  	return categories ;
  }
  
  public void setQueryResults(long duration, List<Node> queryResult,Node node,String searchType) throws Exception{
    duration_ = duration;
    results_ = queryResult;
    node_ = node ;
    searchType_ = searchType ;
    update() ;
  }  
  
  static public class ChangeNodeActionListener extends ExoActionListener {
  	public void execute(ExoActionEvent event) throws Exception {
  		UIResults uicomp = (UIResults) event.getComponent();
  		UIContentExplorer uiExplorer = (UIContentExplorer) uicomp.getAncestorOfType(UIContentExplorer.class);
  		String uri = event.getParameter(OBJECTID);
  		Information iprovider = findInformationProvider(uicomp);
      UISearchController controller = (UISearchController)uicomp.getParent() ;
      if(uicomp.searchType_.equals(UISearchForm.CATEGORY_SEARCH)) {
        try {
          if ("../".equals(uri)) {
            if (!"/".equals(uiExplorer.getSelectNode().getPath())) {
              uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
            }
          }
          uiExplorer.setSelectNode(uri);
        } catch (Exception e) {
          JCRExceptionManager.process(iprovider, e);        
        }        
        UISearch uiSearch = (UISearch) controller.getParent() ;
        uiSearch.setRenderedSibling(UIContentList.class) ; 
      }else {
        UIDetailDocument uicontent = (UIDetailDocument)controller.getSibling(UIDetailDocument.class) ;
        String path = uicomp.node_.getSession().getWorkspace().getName()+":"+uri ;        
        uicontent.setNodeByUri(path) ; 
        controller.setRenderedSibling(UIDetailDocument.class) ;
      }
  	}  	  
  }      
}
