package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.portlet.PortletPreferences;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.scripts.ScriptService;
import org.exoplatform.services.cms.views.ManageViewService;

public class UIScriptConfig extends UISimpleForm {

  final public static String NEW_CONFIG_ACTION = "newConfig".intern() ;
  final public static String EDIT_CONFIG_ACTION = "editConfig".intern() ;  
  public static Parameter[] EDIT_CONFIG_PARAMS = { new Parameter(ACTION,EDIT_CONFIG_ACTION)} ;
  public static Parameter[] NEW_CONFIG_PARAMS = {new Parameter(ACTION,NEW_CONFIG_ACTION)} ;  

  private UISelectBox scriptList_ ;
  protected UISelectBox workspaces_ ;
  private UISelectBox scritpTemplatesBox_ ;
  private UISelectBox detailTemplatesBox_ ;
  private UICheckBox viewTagsCloud_ ;
  private UICheckBox allowVoting_ ;
  private UICheckBox allowComment_ ;
  private UICheckBox allowExporting_ ;
  private boolean editable_ ;
  private boolean isChangedForm_ ;

  public UIScriptConfig() throws Exception {
    super("ScriptConfigForm","post",null) ;		
    setId("ScriptConfigForm") ;
    setRendererType("SimpleFormVelocityRenderer");		
    PortletPreferences prefs = FacesUtil.getPortletPreferences();

    scriptList_ = new UISelectBox("scriptList",prefs.getValue(CBUtils.SCRIPT_NAME,""),new ArrayList<SelectItem>());
    add(scriptList_) ;

    String[] workspaceNames = CBUtils.getWorkspaceNames() ;    
    List<SelectItem> workspaceOptions = new ArrayList<SelectItem>() ;    
    for(String workspace:workspaceNames) {
      workspaceOptions.add(new SelectItem(workspace,workspace)) ;
    }        
    workspaces_ = new UISelectBox(CBUtils.WORKSPACE,prefs.getValue(CBUtils.WORKSPACE,""), workspaceOptions) ;
    add(workspaces_) ;

    scritpTemplatesBox_ = new UISelectBox(CBUtils.TEMPLATE,"", new ArrayList<SelectItem>()) ;
    add(scritpTemplatesBox_) ;

    detailTemplatesBox_ = new UISelectBox(CBUtils.BOX_TEMPLATE,"",new ArrayList<SelectItem>()) ;
    add(detailTemplatesBox_) ;

    allowVoting_ = new UICheckBox(CBUtils.ALLOW_VOTING,prefs.getValue(CBUtils.ALLOW_VOTING,"")) ;
    add(allowVoting_) ;

    allowComment_ = new UICheckBox(CBUtils.ALLOW_COMMENT,prefs.getValue(CBUtils.ALLOW_COMMENT,"")) ;
    add(allowComment_) ;

    viewTagsCloud_ = new UICheckBox(CBUtils.VIEW_TAGS_CLOUD,prefs.getValue(CBUtils.VIEW_TOOLBAR,"")) ;
    add(viewTagsCloud_) ;
    
    allowExporting_ = new UICheckBox(CBUtils.ALLOW_EXPORTING,prefs.getValue(CBUtils.ALLOW_EXPORTING,"")) ;
    add(allowExporting_) ;
    
    updateSelectBoxes() ;

    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
    addActionListener(EditConfigActionListener.class,EDIT_CONFIG_ACTION) ;
    addActionListener(NewConfigActionListener.class,NEW_CONFIG_ACTION) ;    
  }

  public boolean isEditable() {return editable_ ; }

  public String getEditConfigParams() { return EDIT_CONFIG_ACTION ;}
  public String getNewConfigParams() { return NEW_CONFIG_ACTION ;}  

  public void updateFormValues() throws Exception {      
    PortletPreferences prefs = FacesUtil.getPortletPreferences();    
    workspaces_.setValue(prefs.getValue(CBUtils.WORKSPACE,"")) ;
    scriptList_.setValue(prefs.getValue(CBUtils.SCRIPT_NAME,"")) ;
    scritpTemplatesBox_.setValue(prefs.getValue(CBUtils.TEMPLATE, "")) ;
    detailTemplatesBox_.setValue(prefs.getValue(CBUtils.BOX_TEMPLATE, "")) ;
    viewTagsCloud_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TAGS_CLOUD,""))) ;
    allowComment_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_COMMENT,""))) ;
    allowVoting_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_VOTING,""))) ;
    allowExporting_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_EXPORTING,""))) ;
  }

  public void updateSelectBoxes() throws Exception {
    ManageViewService viewService = 
      (ManageViewService)PortalContainer.getComponent(ManageViewService.class) ;
    List<SelectItem> scriptTempOptions = new ArrayList<SelectItem>() ;
    List<Node> scriptTemplates = viewService.getAllTemplates(BasePath.CB_SCRIPT_TEMPLATES) ;
    for(Node template:scriptTemplates) {
      scriptTempOptions.add(new SelectItem(template.getName(),template.getName())) ;
    }
    scritpTemplatesBox_.setOptions(scriptTempOptions) ;

    List<SelectItem> docTempOptions = new ArrayList<SelectItem>() ;
    List<Node> docTemplates = viewService.getAllTemplates(BasePath.CB_DETAIL_VIEW_TEMPLATES) ;
    for(Node template: docTemplates) {
      docTempOptions.add(new SelectItem(template.getName(),template.getName())) ;
    }
    detailTemplatesBox_.setOptions(docTempOptions) ;

    ScriptService scriptService = (ScriptService)PortalContainer.getComponent(ScriptService.class) ;
    List<SelectItem> groovyList = new ArrayList<SelectItem>() ;
    List<Node> scriptList = scriptService.getCBScripts() ;
    String baseScriptPath = scriptService.getBaseScriptPath() ;    
    for(Node script:scriptList) {      
      String cbScriptName = StringUtils.substringAfter(script.getPath(),baseScriptPath + CBUtils.SLASH) ; 
      String displayName = StringUtils.substringAfterLast(cbScriptName,"/") ; 
      groovyList.add(new SelectItem(displayName, cbScriptName)) ;
    }
    scriptList_.setOptions(groovyList) ;
  }

  public void setEditable(boolean editable) {
    editable_ = editable ;
    workspaces_.setEditable(editable_) ;
    scriptList_.setEditable(editable_) ;
    scritpTemplatesBox_.setEditable(editable_) ; 
    allowComment_.setEditable(editable_) ;
    viewTagsCloud_.setEditable(editable_) ;
    allowVoting_.setEditable(editable_) ;
    allowExporting_.setEditable(editable_) ;
  }

  public void setChangedForm(boolean isChangeForm) {
    isChangedForm_ = isChangeForm ;
  }

  public static class NewConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {         
      UIScriptConfig scriptConfig = (UIScriptConfig)event.getComponent() ;      
      scriptConfig.setRenderedSibling(UINewConfigForm.class) ;
    }    
  }      

  public static class EditConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptConfig scriptConfig = (UIScriptConfig)event.getComponent() ;      
      scriptConfig.setEditable(true) ;
      scriptConfig.updateSelectBoxes() ;
      scriptConfig.setChangedForm(false) ;
    }    
  }   

  public static class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIScriptConfig scriptConfig = (UIScriptConfig)event.getSource() ;
      if(scriptConfig.isChangedForm_) {
        scriptConfig.setRenderedSibling(UINewConfigForm.class) ;
      }else {
        scriptConfig.setEditable(false) ;
      }
    }    
  }
  public static class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptConfig uiForm = (UIScriptConfig)event.getSource() ;  
      Information iprovider = findInformationProvider(uiForm);     
      ResourceBundle res = getApplicationResourceBundle() ;
      String scriptName = uiForm.getUISelectBox("scriptList").getValue()  ;
      String ws = uiForm.getUISelectBox(CBUtils.WORKSPACE).getValue() ;
      String template = uiForm.getUISelectBox(CBUtils.TEMPLATE).getValue() ;
      String boxTemplate = uiForm.getUISelectBox(CBUtils.BOX_TEMPLATE).getValue() ;      

      PortletPreferences prefs = FacesUtil.getPortletPreferences();                                           
      UICheckBox allowVotingBox = (UICheckBox)uiForm.getUIInput(CBUtils.ALLOW_VOTING) ;
      UICheckBox allowCommentBox = (UICheckBox)uiForm.getUIInput(CBUtils.ALLOW_COMMENT) ;
      String allowVoting = Boolean.toString(allowVotingBox.getSelect()) ;
      String allowComment = Boolean.toString(allowCommentBox.getSelect());
      UICheckBox viewTagsCloudBox = (UICheckBox)uiForm.getUIInput(CBUtils.VIEW_TAGS_CLOUD);
      String viewTagsCloud = Boolean.toString(viewTagsCloudBox.getSelect()) ;
      UICheckBox allowExportingBox = (UICheckBox)uiForm.getUIInput(CBUtils.ALLOW_EXPORTING) ;
      String allowExporting = Boolean.toString(allowExportingBox.getSelect()) ;
      
      prefs.setValue(CBUtils.WORKSPACE, ws) ;                        
      prefs.setValue(CBUtils.TEMPLATE, template) ;
      prefs.setValue(CBUtils.BOX_TEMPLATE, boxTemplate) ;                  
      prefs.setValue(CBUtils.USECASE,CBUtils.USE_SCRIPT) ;      
      prefs.setValue(CBUtils.SCRIPT_NAME, scriptName) ;
      prefs.setValue(CBUtils.ALLOW_VOTING,allowVoting) ;
      prefs.setValue(CBUtils.ALLOW_COMMENT,allowComment) ;
      prefs.setValue(CBUtils.VIEW_TAGS_CLOUD,viewTagsCloud) ;
      prefs.setValue(CBUtils.ALLOW_EXPORTING,allowExporting) ;            
      prefs.setValue(CBUtils.JCR_PATH,getValue(prefs,CBUtils.JCR_PATH)) ;
      prefs.setValue(CBUtils.QUERY_LANGUAGE,getValue(prefs,CBUtils.QUERY_LANGUAGE)) ;
      prefs.setValue(CBUtils.QUERY_STATEMENT,getValue(prefs,CBUtils.QUERY_STATEMENT)) ;
      prefs.setValue(CBUtils.IS_NEW_QUERY,getValue(prefs,CBUtils.IS_NEW_QUERY)) ;
      prefs.setValue(CBUtils.IS_PERSONAL_QUERY,getValue(prefs,CBUtils.IS_PERSONAL_QUERY)) ;
      prefs.setValue(CBUtils.QUERY_STORE,getValue(prefs,CBUtils.QUERY_STORE)) ;
      prefs.setValue(CBUtils.NB_PER_PAGE,getValue(prefs,CBUtils.NB_PER_PAGE)) ;                
      prefs.setValue(CBUtils.REF_DOCUMENT,getValue(prefs,CBUtils.REF_DOCUMENT)) ;
      prefs.setValue(CBUtils.CHILD_DOCUMENT,getValue(prefs,CBUtils.CHILD_DOCUMENT)) ;    
      prefs.setValue(CBUtils.VIEW_TOOLBAR,getValue(prefs,CBUtils.VIEW_TOOLBAR)) ;          
      prefs.setValue(CBUtils.DOCUMENT_NAME,getValue(prefs,CBUtils.DOCUMENT_NAME)) ;
      
      prefs.store() ;            
      iprovider.addMessage( new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
             res.getString("UIContentBrowserConfig.msg.save-config-successfully"))) ;
      uiForm.updateFormValues() ;
      uiForm.setEditable(false) ;      
    }
    
    private String getValue(PortletPreferences pref,String name) {
      String value = pref.getValue(name,"") ;
      if(value == null) value = "" ;
      return value ;
    }
    
  }		
}
