/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.portlet.PortletPreferences;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.jcr.core.ExtendedNode;

/**
 * Created by The eXo Platform SAS
 * Author : Xuan Hoa Pham
 *          hoapham@exoplatform.com
 * 					phamvuxuanhoa@gmail.com
 * Dec 11, 2006  
 */
public class UITaggedDocuments extends UIExoCommand {

  final static private String VIEW_DOCUMENT_ACTION = "viewDocument".intern() ;
  final static private int NUMBER_OF_DOCUMENTS = 7 ;  
  static Parameter[] VIEW_DOCUMENT_PARAMS = {new Parameter(ACTION,VIEW_DOCUMENT_ACTION)} ;  
  static Parameter[] CANCEL_ACTION_PARAMS = {new Parameter(ACTION,CANCEL_ACTION)} ;
  
  
  private List<Node> documentList_ = new ArrayList<Node>() ;
  private String tagName_ ;
  private String htmlTagStyle_ ;
  
  private UIPageListIterator pageListIterator_ ;
  

  public UITaggedDocuments() {
    setId("UITaggedDocuments") ;
    setRendererType("VelocityRenderer") ;
    
    pageListIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(pageListIterator_) ;
    
    addActionListener(ViewDocumentActionListener.class,VIEW_DOCUMENT_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
  }
  
  public Parameter[] getViewDocumentParams() { return VIEW_DOCUMENT_PARAMS ; }
  public Parameter[] getCancelParams() { return CANCEL_ACTION_PARAMS ; }
  
  public String getTagName() { return this.tagName_ ; }
  public void setTagName(String tagName) {
    this.tagName_ = tagName ;
  }
  
  public String getHtmlTagStyle() { return this.htmlTagStyle_ ; }
  public void setHtmlTagStyle(String htmlStyle) {
    this.htmlTagStyle_ = htmlStyle ;
  }
  
  public void setDatas(List<Node> documentList, String tagName, String htmlStyle) {
    setDocumentList(documentList) ;
    setTagName(tagName) ;
    setHtmlTagStyle(htmlStyle) ;
  }
  
  public boolean hasDocuments() throws Exception {    
    if(getDocumentsList().size()>0) return true ;
    return false ;
  }

  public void setDocumentList(List<Node> list) {    
    this.documentList_ = list ; 

  }

  public List<Document> getDocumentsList() throws Exception {
    List<Document> taggedDocuments = new ArrayList<Document>() ;    
    for(Node node: documentList_) {      
      String workspaceName = node.getSession().getWorkspace().getName() ;      
      Document doc = new Document(workspaceName,node) ;
      taggedDocuments.add(doc) ;
    }
    ObjectPageList objPageList = new ObjectPageList(taggedDocuments,NUMBER_OF_DOCUMENTS) ;
    pageListIterator_.setPageList(objPageList) ;
    return pageListIterator_.getPageListDataHandler().getDatas() ;
    //return taggedDocuments ;
  }

  public String getDocumentOwner(Document document) {
    Node node = document.getNode() ;
    String str = null;
    try {
      str = ((ExtendedNode) node).getACL().getOwner();    //return doc. 
    }catch (Exception e) {
      e.printStackTrace();
    }
    return str;
  }     

  static public class ViewDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {    
      UITaggedDocuments uicomp= (UITaggedDocuments)event.getComponent() ;
      UIDetailDocument uidoc = (UIDetailDocument)uicomp.getSibling(UIDetailDocument.class);
      String uri = event.getParameter(OBJECTID) ;
      List<Document> documents = uicomp.getDocumentsList() ;             
      Node node = null ; 
      for(Document document:documents) {
        if(document.getUri().equals(uri)) {
          node = document.getNode() ;
        }
      }
      if(node != null){
        uidoc.setNode(node) ;
      } else {
        uidoc.setNodeByUri(uri) ;
      }
      UIContentExplorer uiExplorer = (UIContentExplorer)uicomp.getParent() ;
      if(uiExplorer.isShowingToolbar()){
        uiExplorer.setShowingToolbar(false) ;
      }
      if(uiExplorer.isShowingTagsCloud()) {
        uiExplorer.setShowingTagsCloud(false) ;
      }
      uicomp.setRendered(false) ;
      UIDetailDocument doc = (UIDetailDocument)uicomp.getSibling(UIDetailDocument.class) ;      
      doc.setOpener(UITaggedDocuments.class) ;
      doc.setRendered(true) ;  
    }    
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITaggedDocuments uicomp = (UITaggedDocuments)event.getComponent() ;      
      UIContentExplorer uiExplorer = (UIContentExplorer)uicomp.getParent() ;
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
      boolean isViewingToolBar = Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TOOLBAR,"")) ;
      uiExplorer.setShowingToolbar(isViewingToolBar) ;
      UIContentList contentList  = (UIContentList)uicomp.getSibling(UIContentList.class) ;
      uicomp.setRendered(false) ;
      contentList.setRendered(true) ;      
    }    
  }
}
