/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.folksonomy.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.jcr.Node;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.contentbrowser.component.UIContentExplorer;
import org.exoplatform.portlets.content.contentbrowser.component.UIContentList;
import org.exoplatform.portlets.content.contentbrowser.component.UIResults;
import org.exoplatform.portlets.content.contentbrowser.component.UISearchForm;
import org.exoplatform.portlets.content.contentbrowser.component.UITaggedDocuments;
import org.exoplatform.portlets.content.jcr.AlphaNodeComparator;
import org.exoplatform.services.cms.folksonomy.FolksonomyService;

/**
 * Created by The eXo Platform SAS
 * Author : Xuan Hoa Pham
 *          hoapham@exoplatform.com
 * 					phamvuxuanhoa@gmail.com
 * Dec 11, 2006  
 */
public class UITagCloud extends UIExoCommand {
  
  final static String VIEW_TAG_ACTION = "viewTag".intern() ;    
  final private static String TAG_STATUS_PROP = "exo:tagStatus".intern() ;
  
  public static final String ASCENDING_ORDER = "Ascending".intern();
  
  static Parameter VIEW_TAG_PARAM = new Parameter(ACTION,VIEW_TAG_ACTION) ;
  static Parameter[] VIEW_TAG_PARAMS = {new Parameter(ACTION,VIEW_TAG_ACTION) } ;
  
  private FolksonomyService folkService_ ;
  private List<Node> tagsCloud_ = new ArrayList<Node>() ;
  
  public UITagCloud(FolksonomyService folkService) throws Exception {
    setId("UITagCloud") ;
    setRendererType("VelocityRenderer") ;
    folkService_ = folkService ;
    List<Node> defaultTagsCloud = folkService_.getAllTags() ;    
    setTagsCloud(defaultTagsCloud) ;
    
    addActionListener(ViewTagDetailActionListener.class,VIEW_TAG_ACTION) ;    
  }
  
  public String getViewTagParamName() { return VIEW_TAG_PARAM.getName() ; }
  public String getViewTagParamValue() { return VIEW_TAG_PARAM.getValue() ;}  
  public Parameter[] getViewTagParams() { return VIEW_TAG_PARAMS ; }
  
  public List<Node> getTagsCloud() { return tagsCloud_ ; }
  public void setTagsCloud(List<Node> tagsList) throws Exception { 
    this.tagsCloud_ = sortTags(tagsList) ;
  }
    
  
  public String getHtmlStyleOfTag(Node tagNode) throws Exception {
    String tagStyleName = tagNode.getProperty(TAG_STATUS_PROP).getValue().getString() ;
    return folkService_.getTagStyle(tagStyleName) ;
  }
  
  public List<Node> getDocumentOnTag(String tagPath) throws Exception {
    return folkService_.getDocumentsOnTag(tagPath) ;
  }
  
  public Node getSelectedTag(String tagPath) throws Exception {
    return folkService_.getTag(tagPath) ;    
  }
  
  public void renderSiblingComponent(boolean isRendered) {
    this.getSibling(UISearchForm.class).setRendered(isRendered) ;
    this.getSibling(UIResults.class).setRendered(isRendered) ;
  }
  
  private List<Node> sortTags(List<Node> tagsList) throws Exception {
    Map<String,Node> sortedNodes = new TreeMap(new AlphaNodeComparator(ASCENDING_ORDER)) ;    
    List<Node> sortedListTags = new ArrayList<Node>() ;
    int j = 0 ;
    for(Node tag :tagsList) {        
      String key = 
        new StringBuffer(tag.getName()).append(j).toString().toLowerCase();
      sortedNodes.put(key, tag);
      j++;
    }               
    Collection<Node> collection = sortedNodes.values() ;
    for(Node tag:collection) {
      sortedListTags.add(tag) ;
    }
    return sortedListTags ;
  }       
  
  public static class ViewTagDetailActionListener  extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {
      UITagCloud uicomponent = (UITagCloud)event.getSource() ;
      String tagPath = event.getParameter(OBJECTID) ;      
      List<Node> documentList = uicomponent.getDocumentOnTag(tagPath) ;      
      UIContentExplorer explorer = (UIContentExplorer)uicomponent.getParent() ;
      if(explorer.isShowingToolbar()) {        
        explorer.setShowingToolbar(false) ;
      }
      
      UIContentList contentList = (UIContentList)uicomponent.getSibling(UIContentList.class) ;
      contentList.setRendered(false) ;
      UITaggedDocuments uidocuments = 
        (UITaggedDocuments)uicomponent.getSibling(UITaggedDocuments.class) ;
      Node selectedTag = uicomponent.getSelectedTag(tagPath) ;
      String htmlStyle = uicomponent.getHtmlStyleOfTag(selectedTag) ;
      uidocuments.setDatas(documentList,selectedTag.getName(),htmlStyle) ;      
      uidocuments.setRendered(true) ;      
    }    
  }  
}
