/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.Value;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIUploadInputStream;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.cms.i18n.MultiLanguageService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;
import org.exoplatform.services.resources.LocaleConfig;
import org.exoplatform.services.resources.LocaleConfigService;

/**
 * @email: nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04
 */
public class UIAddLanguageForm extends UISimpleMulltipartForm implements ChangeNodeListener{

  final static public String ADD_LANGUAGE_ACTION = "addLanguage";
  final static public String CHANGE_LANGUAGE_ACTION = "changeLanguage";
  final static public String EXO_LANGUAGE     = "exo:language";
  final static public String LANGUAGES     = "languages";
  final static public String  JCRCONTENT = "jcr:content";
  final static public String  JCRMIMETYPE = "jcr:mimeType";
  
  private TemplateService tservice_ ;
  private UISelectBox languageSelect_ ;
  private MultiLanguageService multiLangService_ ;

  public UIAddLanguageForm(TemplateService tservice, LocaleConfigService localConfigService,
                           MultiLanguageService multiLangService) throws Exception {
    super("uiAddLanguageForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    tservice_ = tservice ;
    multiLangService_ = multiLangService ;
    languageSelect_ = new UISelectBox("language", "", null);
    List<SelectItem> options = new ArrayList<SelectItem>() ;    
    options.add(new SelectItem("- - - -", ""));
    for( Object obj:localConfigService.getLocalConfigs()) {
      LocaleConfig lconfig = (LocaleConfig)obj ;      
      options.add(new SelectItem(lconfig.getLocale().getDisplayLanguage(), lconfig.getLocale().getDisplayLanguage()));      
    }
    languageSelect_.setOptions(options) ;
    languageSelect_.setUpdateOnChangeAction(CHANGE_LANGUAGE_ACTION) ;
    add(languageSelect_);
    add(new UIUploadInputStream("upload")); 
    UICheckBox defaultLanguage = new UICheckBox("default", "");
    add(defaultLanguage);

    addActionListener(ChangeLanguageActionListener.class, CHANGE_LANGUAGE_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  } 
  
  public boolean isShowTemplate() throws Exception {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Node node = uiExplorer.getSelectNode();
    if(node.getPrimaryNodeType().getName().equals("nt:file")){
      if(node.hasNode(JCRCONTENT)){
        String mimeType = node.getNode(JCRCONTENT).getProperty(JCRMIMETYPE).getString() ; 
        if(!mimeType.startsWith("text"))return false ;        
      }
    }
    return true ;
  }
  public String getTemplateToParse() throws Exception {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Node node = uiExplorer.getSelectNode();
    String path = tservice_.getTemplatePath(true, node.getPrimaryNodeType().getName()) ;
    return "jcr:" + path ;
  }
  
  public void update() {
    languageSelect_.setValue("") ;
  }
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
   setRendered(false) ; 
  }
  
  static public class ChangeLanguageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAddLanguageForm uicomp = (UIAddLanguageForm) event.getSource();
      String selectLanguage = uicomp.languageSelect_.getValue() ;
      if(selectLanguage == null || selectLanguage.length() < 1) return;
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Map inputs = uicomp.getUIInputs() ;
      Node node = uiExplorer.getSelectNode();
      Map propertyMap = getPropertyDefinitionMap(node) ;
      if(selectLanguage.equals(node.getProperty(EXO_LANGUAGE).getString())) {                  
        initValue(inputs, propertyMap, node) ;
      }else {
        Node languageNode = uicomp.multiLangService_.getLanguage(node, selectLanguage) ;
        if(languageNode != null){
          initValue(inputs, propertyMap, languageNode) ;
        }else {
          initValue(inputs, propertyMap, null) ;
        }
      }
    }
    
    private void initValue(Map inputs, Map propertyMap, Node node)throws Exception {
      Set keys = inputs.keySet();
      for (Iterator iter = keys.iterator(); iter.hasNext();) {
        String key = (String) iter.next();
        UIInput input = (UIInput) inputs.get(key);
        JcrInputProperty property = (JcrInputProperty) input.getProperty("jcrInfo");
        if(property != null) {
          int index = property.getJcrPath().lastIndexOf("/") + 1 ;
          String name = property.getJcrPath().substring(index) ;
          PropertyDefinition propertyDef = (PropertyDefinition)propertyMap.get(name) ;
          if(propertyDef != null && !propertyDef.isProtected()){
            try{
              if(node != null) input.setValue(node.getProperty(name).getString()) ;
              else input.setValue("") ;
            }catch(Exception e){}            
          }            
        }
      }   
    }
    
    private Map getPropertyDefinitionMap(Node node) throws Exception{
      Map propertyMap = new HashMap() ;      
      PropertyDefinition[] properties = node.getPrimaryNodeType().getPropertyDefinitions() ;
      for(PropertyDefinition pro : properties) {
        propertyMap.put(pro.getName(), pro) ;
      }
      return propertyMap ;
    }
  }  

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try {
        UIAddLanguageForm uiForm = (UIAddLanguageForm) event.getSource();
        ResourceBundle res = getApplicationResourceBundle() ;
        Information iprovider = findInformationProvider(uiForm);
        String selectLanguage = uiForm.getUISelectBox("language").getValue() ;
        if(selectLanguage == null || selectLanguage.length() < 1) {
          iprovider.addMessage(new ExoFacesMessage(
              res.getString("UIAddLanguageForm.msg.language-is-null")));
          return;
        }
        boolean isDefault = uiForm.getUICheckBox("default").getSelect() ;
        UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
        Node node = uiExplorer.getSelectNode();
        if(node.getPrimaryNodeType().getName().equals("nt:file")) { 
          String mimeType = node.getNode(JCRCONTENT).getProperty(JCRMIMETYPE).getString() ;
          if(mimeType.startsWith("text")){
            Map inputs = uiForm.getUIInputs() ;
            Set keys = inputs.keySet();
            for (Iterator iter = keys.iterator(); iter.hasNext();) {
              String key = (String) iter.next();
              UIInput input = (UIInput) inputs.get(key);
              JcrInputProperty property = (JcrInputProperty) input.getProperty("jcrInfo");
              if(property.getJcrPath().equals("/node/jcr:content/jcr:data")){
                ValueFactoryImpl valueFactory = (ValueFactoryImpl) uiExplorer.getSession().getValueFactory() ;
                Value contentValue = valueFactory.createValue(input.getValue()) ;
                uiForm.multiLangService_.addFileLanguage(node, contentValue, selectLanguage, isDefault) ;
                break ;
              }  
            }
          }else {
            UIUploadInputStream input = (UIUploadInputStream) uiForm.getUIInput("upload");
            InputStream content = input.getInputStream();
            ValueFactoryImpl valueFactory = (ValueFactoryImpl) uiExplorer.getSession().getValueFactory() ;
            Value contentValue = valueFactory.createValue(content) ;
            uiForm.multiLangService_.addFileLanguage(node, contentValue, selectLanguage, isDefault) ;
          }
        }else{
          Map inputs = uiForm.getUIInputs() ;
          Set keys = inputs.keySet();
          Map jcrInputs = new HashMap() ;
          for (Iterator iter = keys.iterator(); iter.hasNext();) {
            String key = (String) iter.next();
            UIInput input = (UIInput) inputs.get(key);
            JcrInputProperty property = (JcrInputProperty) input.getProperty("jcrInfo");
            if(property != null) {
              int index = property.getJcrPath().lastIndexOf("/") + 1 ;
              String name = property.getJcrPath().substring(index) ;
              jcrInputs.put(name, input.getValue()) ;
            }  
          }
          uiForm.multiLangService_.addLanguage(node, jcrInputs, selectLanguage, isDefault) ;
        }
        
        uiForm.setRenderedSibling(UIContentManager.class) ;
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAddLanguageForm uicomp = (UIAddLanguageForm) event.getSource();
      uicomp.setRenderedSibling(UIMultiLanguageForm.class) ;
    }
  }  
}