/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Stack;

import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Workspace;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.JCRSessionManager;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.drives.ManageDriveService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;

/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIExplorer.java,v 1.2 2004/07/31 15:07:15 tuan08 Exp $
 */
public class UIJCRExplorer extends UIExoCommand {
  final static public String SELECT_DRIVE_ACTION = "selectDrive".intern();
  final static public String CHANGE_NODE_ACTION = "changeNode".intern();
  final static public String EDIT_NODE_CONTENT = "editContent".intern();
  final static public String VIEW_NODE_CONTENT = "viewNodeContent".intern();
  final static public String UPLOAD_FILE = "uploadFile".intern();
  final static public String SAVE_SESSION = "saveSession".intern();
  final static public String PREFERENCES_ACTION = "preferences".intern();
  final static public String REFRESH_SESSION_ACTION = "refreshSession".intern();
  final static public String PASTE_ON_CLIPBOARD_ACTION = "pasteNode".intern() ;
  final static public String CLEAR_ALL_ACTION = "clearAll".intern() ;  
  final static public String CLEAR_CLIPBOARD_ACTION = "clear".intern() ;
  final static public String HIDE_CLIPBOARD_ACTION = "hideClipboard".intern() ;
  final static public String HIDE_EXPLORER_ACTION = "hideExplorer".intern() ;

  public static Parameter[] SELECT_DRIVE_PARAMS = { new Parameter(ACTION,  SELECT_DRIVE_ACTION) };
  public static Parameter[] SAVE_SESSION_PARAMS = { new Parameter(ACTION, SAVE_SESSION) };
  public static Parameter[] CHANGE_NODE = { new Parameter(ACTION, CHANGE_NODE_ACTION) };
  public static Parameter[] PREFERENCES_PARAMS = { new Parameter(ACTION, PREFERENCES_ACTION) };
  public static Parameter[] REFRESH_SESSION_PARAMS = { new Parameter(ACTION,  REFRESH_SESSION_ACTION) };
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION,  BACK_ACTION) };
  public static Parameter[] PASTE_ON_CLIPBOARD_PARAMS = {new Parameter(ACTION,PASTE_ON_CLIPBOARD_ACTION) } ;
  public static Parameter[] CLEAR_ALL_PARAMS = {new Parameter(ACTION,CLEAR_ALL_ACTION)} ;
  public static Parameter[] CLEAR_CLIPBOARD_PARAMS = {new Parameter(ACTION,CLEAR_CLIPBOARD_ACTION)} ;
  public static Parameter[] HIDE_CLIPBOARD_PARAMS = {new Parameter(ACTION,HIDE_CLIPBOARD_ACTION) } ;
  public static Parameter[] HIDE_EXPLORER_PARAMS = {new Parameter(ACTION,HIDE_EXPLORER_ACTION) } ;

  private LinkedList<ClipboardCommand> clipboards_ = new LinkedList<ClipboardCommand>() ;

  private Stack<Node> nodesHistory = new Stack<Node>();
  private Preference pref = new Preference();  
  private Session session ;
  protected Node currentNode_ ;
  protected Node previousNode_ ; 
  private ManageDriveService dservice_ ;
  private List<String> supportedContentType_ ;
  private CmsConfigurationService cmsConfService_ ;   
  private JCRSessionManager sessionManager_ ;

  public UIJCRExplorer(CmsConfigurationService cmsConfService, 
      ManageDriveService dservice,TemplateService templateService,JCRSessionManager sessionManager) throws Exception {
    setId("UIJCRExplorer");     
    setRendererType("VelocityRenderer");       
    cmsConfService_ = cmsConfService ;
    dservice_ = dservice ;    
    sessionManager_ = sessionManager ;
    supportedContentType_ = templateService.getDocumentTemplates() ;
    addChild(UIViewController.class);
    addChild(UISelectViewForm.class);
    addActionListener(SelectWorkspaceActionListener.class, SELECT_DRIVE_ACTION);
    addActionListener(ChangeNodeActionListener.class, CHANGE_NODE_ACTION);
    addActionListener(SaveSessionActionListener.class, SAVE_SESSION);
    addActionListener(RefreshSessionActionListener.class, REFRESH_SESSION_ACTION);
    addActionListener(SavePreferencesActionListener.class, PREFERENCES_ACTION);  	
    addActionListener(BackActionListener.class, BACK_ACTION); 
    addActionListener(PasteOnClipBoardActionListener.class,PASTE_ON_CLIPBOARD_ACTION) ;
    addActionListener(ClearClipboardActionListener.class,CLEAR_CLIPBOARD_ACTION) ;
    addActionListener(ClearAllClipBoardActionListener.class,CLEAR_ALL_ACTION) ;
    addActionListener(HideClipboardActionListener.class,HIDE_CLIPBOARD_ACTION) ;
    addActionListener(HideLeftExplorerActionListener.class,HIDE_EXPLORER_ACTION) ;
  }

  public Parameter[] getChangeNodeParams() { return CHANGE_NODE ; }
  public Parameter[] getPasteOnClipBoardParams() { return PASTE_ON_CLIPBOARD_PARAMS ;} 
  public Parameter[] getClearAllParams() { return CLEAR_ALL_PARAMS ;}
  public Parameter[] getClearClipBoardParams() { return CLEAR_CLIPBOARD_PARAMS ; }
  public Parameter[] getHideClipboardParams() { return HIDE_CLIPBOARD_PARAMS ;}
  public Parameter[] getHideExplorerParams() { return HIDE_EXPLORER_PARAMS ; }

  public void initToolbar(){
    List children = getChildren() ;
    UIToolbar toolbar = null ;
    for(int i = 0; i < children.size(); i ++) {
      try{
        toolbar = (UIToolbar)children.get(i) ;
        getChildren().remove(toolbar) ;
        break ;
      }catch(ClassCastException e) {}
    }

    UIToolbar newToolbar = new UIToolbar("MainToolbar");
    newToolbar.addLeftButton(new Button("#{UIJCRExplorer.button.preferences}", PREFERENCES_PARAMS));
    newToolbar.addLeftButton(new Button("#{UIJCRExplorer.button.refresh-session}", REFRESH_SESSION_PARAMS));
    if(pref.isJcrEnable())
      newToolbar.addLeftButton(new Button("#{UIJCRExplorer.button.save-session}", SAVE_SESSION_PARAMS));
    newToolbar.addLeftButton(new Button("#{UIJCRExplorer.button.back}",  BACK_PARAMS));
    PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
    String driveName = prefs.getValue(Utils.DRIVE, "") ;
    //TODO
    // Need replace this code
    Button curDrive = new Button("<span style='background:white; border: solid 1px #b7b7b7;font-weight:bold ;font-size: 11px;margin-right:10px; padding:0px 2px 0px 2px'>" +
        "<span style='color:gray;'>Current Drive: </span>" +
        "<span style='color:#009ad6'>" + driveName + "</span>"+
        "</span>"
        ,SELECT_DRIVE_PARAMS) ;    
    curDrive.setActivate(false) ;
    newToolbar.addRightButton(curDrive);
    newToolbar.addRightButton(new Button("#{UIJCRExplorer.button.select-drive}", SELECT_DRIVE_PARAMS));
    getChildren().add(newToolbar);
  }

  public void refreshSession() {
    try{
      String wsName = session.getWorkspace().getName() ;
      session.logout() ;
      session = sessionManager_.openSession(wsName,SessionContainer.getInstance().getSessionId()) ;
      currentNode_ = (Node) session.getItem(currentNode_.getPath()) ;
      broadcastOnChange(currentNode_) ;
    }catch(Exception e){      
    }    
  }
  public Session getSession() { return session; }
  public void setSession(Session session) throws Exception {
    this.session = session;   
  }

  public LinkedList<ClipboardCommand> getAllClipBoard() { return clipboards_ ;} 

  public Node getSelectNode() {  return currentNode_; }

  public String getPreferencesPath() {
    PortletPreferences prefs_ = FacesUtil.getPortletPreferences() ;
    String prefPath = prefs_.getValue(Utils.JCR_PATH, "") ;
    if (prefPath == null || prefPath.length() == 0 || prefPath == "/") return "" ;
    return prefPath ;
  }

  public String getPreferencesWorkspace() {  	    
    PortletPreferences prefs_ = FacesUtil.getPortletPreferences() ;
    String workspaceName = prefs_.getValue(Utils.WORKSPACE_NAME, "") ;
    if(workspaceName == null || workspaceName.length() == 0) return "" ;
    return workspaceName ;
  }

  public String getCurrentWorkspace() {
    return session.getWorkspace().getName() ;
  }
  public String[] getSelectedNodePath() throws Exception {
    String nodePath = currentNode_.getPath() ;		
    String[] array = StringUtils.split(nodePath, "/") ;    
    return array ;
  }

  public void setSelectNode(Node node) throws Exception {
    currentNode_ = node;        
    record(node);
    broadcastOnChange(node);
  }

  public void setSelectNode(String uri) throws Exception {  
    if (uri == null){      
      previousNode_ = null ;
      currentNode_ = (Node) session.getItem("/");
    } else{
      if(uri.indexOf(":/") > -1){
        previousNode_ = currentNode_ ;
        try{
          String[] array = uri.split(":") ;
          String wsName = array[0].trim() ;
          uri = array[1].trim() ;
          if(wsName.equals(getCurrentWorkspace())){
            currentNode_ = (Node)session.getItem(uri) ;
          }else{
            Session ses = sessionManager_.openSystemSession(wsName,SessionContainer.getInstance().getSessionId());
            currentNode_ = (Node)ses.getItem(uri) ;
          }
        }catch(Exception e) {          
        } 
      }else {
        previousNode_ = currentNode_ ;
        currentNode_ = (Node) session.getItem(uri);
      }      
    }
    if(previousNode_ != null ) record(previousNode_) ;
    broadcastOnChange(currentNode_)  ;  
  }

  public boolean isReadAuthorized(ExtendedNode node) throws RepositoryException {
    try {
      node.checkPermission(PermissionType.READ);
      return true;
    } catch(AccessControlException e) {
      return false;
    }    
  }  

  public Preference getPreference() {
    if(!pref.isEmpty()) return pref ;    
    PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
    String driveName = prefs.getValue(Utils.DRIVE, "") ;
    try{
      Node drive = dservice_.getDriveByName(driveName) ;
      pref.setLeftExplorer(drive.getProperty("exo:viewExplorer").getBoolean()) ;
      pref.setClipBoardListState(drive.getProperty("exo:viewClipboard").getBoolean()) ;
      pref.setShowNonDocumentType(drive.getProperty("exo:viewNonDocument").getBoolean()) ;
      pref.setShowPreferenceDocuments(drive.getProperty("exo:viewPreferences").getBoolean()) ;
      pref.setEmpty(false) ; 
    }catch (Exception e) { }                 
    return pref; 
  }

  public void broadcastOnChange(Node node) throws Exception { 
    List list = findDescendantsOfType(ChangeNodeListener.class);    
    for (int i = 0; i < list.size(); i++) {
      ChangeNodeListener listener = (ChangeNodeListener) list.get(i);      
      listener.onChange(this, node);
    }        
  }

  public void record(Node node) { nodesHistory.push(node); }

  public Node rewind() { return nodesHistory.pop(); }

  public List<Node> getSiblingNodes(Node node)  throws Exception {        
    List<Node> siblingNodeList = new ArrayList<Node>() ;
    if(node.getPath().equals("/")) return siblingNodeList ;
    Node parent = node.getParent();
    if(pref.isShowNonDocumentType()) {
      for(NodeIterator iter = parent.getNodes();iter.hasNext();) {
        siblingNodeList.add(iter.nextNode()) ;           
      } 
    }else {
      for(NodeIterator iter = parent.getNodes();iter.hasNext();) {
        Node childNode = iter.nextNode() ;
        if(childNode.isNodeType("nt:folder") || childNode.isNodeType("nt:unstructured") || 
            childNode.isNodeType("exo:taxonomy")||supportedContentType_.contains(childNode.getPrimaryNodeType().getName()))
          siblingNodeList.add(childNode) ;
      }
    }   
    Collections.sort(siblingNodeList,new NodeNameComparator(pref.getOrder()));
    return siblingNodeList ;
  }

  public List<Node> getChildrenNode(Node parent) throws Exception {
    List<Node> children = new ArrayList<Node>() ;
    if(pref.isShowNonDocumentType()) {
      for(NodeIterator iter = parent.getNodes(); iter.hasNext() ;) {        
        children.add(iter.nextNode()) ; 
      }
    }else {
      for(NodeIterator iter = parent.getNodes(); iter.hasNext() ;) {
        Node child = iter.nextNode() ;      
        if(child.isNodeType("nt:folder") || child.isNodeType("nt:unstructured") || 
            child.isNodeType("exo:taxonomy")||supportedContentType_.contains(child.getPrimaryNodeType().getName())) 
          children.add(child) ;      
      }
    }    
    Collections.sort(children,new NodeNameComparator(pref.getOrder())) ;
    return children ;
  }

  public String getNodeIcon(Node node)  throws Exception {
    return CMSUtils.getSmallNodeTypeIcon(node) ;
  }

  private class NodeNameComparator implements Comparator<Node> {

    private String order_;

    public NodeNameComparator(String pOrder) {
      order_ = pOrder ;
    }

    public int compare(Node o1, Node o2) {
      try{
        if(Preference.ASCENDING_ORDER.equals(order_))
          return (o1.getName()).compareToIgnoreCase(o2.getName()) ;
        return -1 * (o1.getName()).compareToIgnoreCase(o2.getName()) ;
      }catch (Exception e) {
      }
      return 0;
    }
  }

  static public class SavePreferencesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      UIPreferencesForm uiPref = (UIPreferencesForm)uicomp.getSibling(UIPreferencesForm.class) ;
      uiPref.update(uicomp.getPreference()) ;
      uicomp.setRenderedSibling(UIPreferencesForm.class);
    }
  }

  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uiExplorer = (UIJCRExplorer) event.getComponent();			
      String uri = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uiExplorer);
      ResourceBundle res = getApplicationResourceBundle() ;
      String prefPath = uiExplorer.getPreferencesPath() ;            
      if((!prefPath.equals(""))&&
          (uiExplorer.getCurrentWorkspace().equals(uiExplorer.getPreferencesWorkspace()))) {
        if(!uri.contains(prefPath)) {      		
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
              res.getString("UIJCRExplorer.msg.no-have-right-access-path")));
          return ;
        }
        try {
          if ((".."+prefPath).equals(uri)) {
            if (!prefPath.equals(uiExplorer.currentNode_.getPath())) {
              uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
            }
          } else {
            uiExplorer.setSelectNode(uri);
          }
        } catch (Exception e) {
          JCRExceptionManager.process(iprovider, e);
        }
      }
      else {
        try {
          if ("../".equals(uri)) {
            if (!"/".equals(uiExplorer.currentNode_.getPath())) {
              uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
            }
          } else {
            uiExplorer.setSelectNode(uri);
          }
        } catch (Exception e) {
          JCRExceptionManager.process(iprovider, e);
        }
      }      
    }
  }

  static public class SelectWorkspaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uiExplorer = (UIJCRExplorer) event.getComponent();
      ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext();
      PortletRequest request = (PortletRequest) eContext.getRequest();    
      PortletPreferences prefs_ = request.getPreferences();
      prefs_.setValue(Utils.WORKSPACE_NAME,"") ;
      prefs_.setValue(Utils.VIEWS,"") ;
      prefs_.setValue(Utils.JCR_PATH,"") ;
      prefs_.setValue(Utils.DRIVE,"") ;
      prefs_.store() ;
      uiExplorer.setRenderedSibling(UIDriveBrowser.class);
    }
  }

  static public class SaveSessionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.getSession().save();
        uicomp.refreshSession() ;        
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIJCRExplorer.msg.save-session-success")));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }

  static public class RefreshSessionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.getSession().refresh(false);
        uicomp.broadcastOnChange(uicomp.currentNode_) ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIJCRExplorer.msg.refresh-session-success")));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }  

  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      try {        
        Node previousNode = uicomp.rewind();
        uicomp.currentNode_ = previousNode;
        uicomp.broadcastOnChange(previousNode);
      } catch (Exception e) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIJCRExplorer.msg.no-node-history")));
      }
    }
  }

  protected Session getSystemSession() throws Exception {
    return sessionManager_.openSession(cmsConfService_.getWorkspace(),SessionContainer.getInstance().getSessionId()) ;    
  }

  static public class PasteOnClipBoardActionListener extends ExoActionListener {  	
    public void execute(ExoActionEvent event) throws Exception {	
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      String id = event.getParameter(OBJECTID) ;
      int index = Integer.parseInt(id) ;
      ClipboardCommand selectedClipboard = uicomp.clipboards_.get(index-1) ;			
      Node node = uicomp.getSelectNode();
      String type = selectedClipboard .getType();
      String srcPath = selectedClipboard .getSrcPath();      
      String nodePath = node.getPath();
      String destPath = nodePath + srcPath.substring(srcPath.lastIndexOf("/"));
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;      
      try {
        Session session = uicomp.getSession() ;
        Workspace workspace = session.getWorkspace();
        if (ClipboardCommand.COPY.equals(type)) {
          workspace.copy(srcPath, destPath);
        } else {
          session.move(srcPath, destPath);
          session.save() ;
          uicomp.clipboards_.remove(index-1) ;
        }
        ActionServiceContainer actionservice = (ActionServiceContainer)
        PortalContainer.getInstance().getComponentInstanceOfType(ActionServiceContainer.class) ;
        actionservice.initiateObservation(node) ;
        Object[] args = { srcPath, destPath };
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIJCRExplorer.msg.node-pasted"), args));
        uicomp.broadcastOnChange(node);
      } catch (Exception e) {        
        JCRExceptionManager.process(iprovider, e);
      }
    }

  }

  static public class ClearAllClipBoardActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      uicomp.clipboards_.clear() ;
    }  	
  }

  static public class ClearClipboardActionListener extends ExoActionListener {		
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      String id = event.getParameter(OBJECTID) ;
      int index = Integer.parseInt(id) ;
      uicomp.clipboards_.remove(index-1) ;			
    }		
  }

  static public class HideClipboardActionListener extends ExoActionListener {		
    public void execute(ExoActionEvent event) throws Exception {	
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Preference preference = uicomp.getPreference() ;
      preference.setClipBoardListState(false) ;
      UIPreferencesForm form = (UIPreferencesForm)uicomp.getSibling(UIPreferencesForm.class) ;
      form.clipboardBox_.setSelect(false) ;			
    }		
  }

  static public class HideLeftExplorerActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Preference preference = uicomp.getPreference() ;
      preference.setLeftExplorer(false) ;      
      UIPreferencesForm form = (UIPreferencesForm)uicomp.getSibling(UIPreferencesForm.class) ;
      form.leftExplorerBox_.setSelect(false) ;
    }
  }
}