/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionListener;
import javax.jcr.Node;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;

import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.cms.templates.TemplateService;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIContentDisplayer.java,v 1.2 2004/08/07 18:11:26 tuan08 Exp $
 */
public class UISelectContentTypeForm extends UISimpleForm  {
 
  final static public String CHANGE_CONTENT_TYPE_ACTION = "changeType" ;
  static final public String TEMPLATE_LABEL = "label" ;

  UISelectBox uiSelect_ ;
  
  private String defaultContentType_ ;
  private TemplateService templateService_;
  
  public UISelectContentTypeForm(TemplateService templateService) throws Exception {
    super("selectContentTypeForm", "post", null) ;
    setRendererType("SimpleFormVelocityRenderer");
    
    uiSelect_ = new  UISelectBox("contentType", "", null) ;
    uiSelect_.setUpdateOnChangeAction(CHANGE_CONTENT_TYPE_ACTION) ;
    add(uiSelect_) ;
    
    templateService_ = templateService;
    defaultContentType_ = "exo:article" ;
    addActionListener(ChangeContentTypeActionListener.class, CHANGE_CONTENT_TYPE_ACTION) ;
  }  
  
  public String getDefaultDocument(){ return defaultContentType_ ; }
  
  public void update(Node currentNode) throws Exception {    
    List<SelectItem> options = new ArrayList<SelectItem>();
    boolean hasDefaultDoc = false ;
    NodeTypeManager ntManager = currentNode.getSession().getWorkspace().getNodeTypeManager() ;
    NodeType currentNodeType = currentNode.getPrimaryNodeType() ;    
    NodeDefinition[] childDefs = currentNodeType.getChildNodeDefinitions() ;
    List templates = templateService_.getDocumentTemplates() ; 
    try {
      for(int i = 0; i < templates.size(); i ++){
        String nodeTypeName = templates.get(i).toString() ;              
        NodeType nodeType = ntManager.getNodeType(nodeTypeName) ;
        NodeType[] superTypes = nodeType.getSupertypes() ;
        boolean isCanCreateContent = false ;
        for(NodeDefinition childDef:childDefs){
          NodeType[] requiredChilds = childDef.getRequiredPrimaryTypes() ;
          for(NodeType requiredChild : requiredChilds) {          
            if(nodeTypeName.equals(requiredChild.getName())){            
              isCanCreateContent = true ;
              break ;
            }            
          }
          if(nodeTypeName.equals(childDef.getName()) || isCanCreateContent) {
            if(!hasDefaultDoc && nodeTypeName.equals("exo:article")) {
              defaultContentType_ = "exo:article" ;
              hasDefaultDoc = true ;
            }
            options.add(new SelectItem(templateService_.getTemplateLabel(nodeTypeName), nodeTypeName));          
            isCanCreateContent = true ;          
          }
        }      
        if(!isCanCreateContent){
          for(NodeType superType:superTypes) {
            for(NodeDefinition childDef:childDefs){          
              for(NodeType requiredType:childDef.getRequiredPrimaryTypes()) {              
                if (superType.getName().equals(requiredType.getName())) {
                  if(!hasDefaultDoc && nodeTypeName.equals("exo:article")) {
                    defaultContentType_ = "exo:article" ;
                    hasDefaultDoc = true ;
                  }
                  options.add(new SelectItem(templateService_.getTemplateLabel(nodeTypeName), nodeTypeName));                
                  isCanCreateContent = true ;
                  break;
                }
              }
              if(isCanCreateContent) break ;
            }
            if(isCanCreateContent) break ;
          }
        }            
      }
      uiSelect_.setOptions(options) ;
      if(hasDefaultDoc)
        uiSelect_.setValue(defaultContentType_);
      else if(options.size() > 0){
        defaultContentType_ = options.get(0).value_ ;
        uiSelect_.setValue(defaultContentType_);
      }
    }catch(Exception e) { 
    }
    
  }
  
  static public class ChangeContentTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectContentTypeForm uicomp = (UISelectContentTypeForm) event.getSource() ;
      String contentType = uicomp.uiSelect_.getValue() ;
      UIContentForm uiForm = (UIContentForm) uicomp.getSibling(UIContentForm.class) ;
      uiForm.showNewContent(contentType) ;          
      ActionListener[] actionListeners = uiForm.getActionListeners();
      for (ActionListener actionListener : actionListeners) {
        if(actionListener instanceof ExoActionListener) {
          ExoActionListener exoListener = (ExoActionListener)actionListener;
          exoListener.removeTransientInterceptors();
        }
      }
    }    
  }

}