package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.Value;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.security.SecurityService;

public class UISelectViewForm extends UISimpleForm implements ChangeNodeListener {

  protected Node node_;
  private List<SelectItem> options = new ArrayList<SelectItem>();
  private  UISelectBox views_ ;
  private ManageViewService viewService_ ;
  private SecurityService securityService_ ;

  public UISelectViewForm(ManageViewService viewService,SecurityService securityService) throws Exception {
    super("UISelectViewForm", "post", null);
    setId("UISelectViewForm") ;
    setRendererType("SimpleFormVelocityRenderer");
    viewService_ = viewService ;
    securityService_ = securityService ;
    views_ = new UISelectBox("views","",options) ;
    add(views_) ;        
    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }

  public void refresh()throws Exception{    
    ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext();
    String currentUser = eContext.getRemoteUser() ;
    PortletRequest request = (PortletRequest) eContext.getRequest();    
    PortletPreferences prefs_ = request.getPreferences();    
    String[] views = prefs_.getValues("views", new String[5]) ;
    options = new ArrayList<SelectItem>();        
    for(String view:views) {      
      Node viewNode = viewService_.getViewByName(view) ;
      if(viewNode.hasProperty("exo:permission")) {
        Value[] values = viewNode.getProperty("exo:permissions").getValues() ;
        if(values == null || values.length == 0) 
          options.add(new SelectItem(view,view)) ;
        else {
          for(Value value: values) {     
            String permission = value.getString() ;
            if( permission.length() == 0|| currentUser.equalsIgnoreCase(permission) || 
                securityService_.hasMembershipInGroup(currentUser,permission)) {
              options.add(new SelectItem(view,view)) ;
              break ;
            }               
          } 
        }                  
      }else {
        options.add(new SelectItem(view,view)) ;
      }                          
    }
    views_.setOptions(options) ;
    String currentView = views_.getValue() ;
    if(currentView == null || currentView.length() == 0)
      views_.setValue(views[0].trim()) ;                 
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectViewForm uicomp = (UISelectViewForm) event.getSource();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      String view = uicomp.getUISelectBox("views").getValue() ;
      Information iprovider = findInformationProvider(uicomp);
      try {
        UIViewController viewController = (UIViewController)uicomp.getSibling(UIViewController.class) ;
        viewController.renderTabs(view) ;
        uiExplorer.broadcastOnChange(uicomp.node_);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
}
