package org.exoplatform.portlets.content.metadata.component;

import java.util.Collection;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.metadata.MetadataService;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeType;
/**
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04
 */
public class UIMetadataManager extends UIExoCommand {

  final static public String METADATA_ACTION = "metadata".intern();
  final static public String INTERNAL_USE = "exo:internalUse".intern() ;

  public static Parameter[] VIEW_PARAMS = { new Parameter(ACTION , VIEW_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  public static Parameter[] METADATA_PARAMS = { new Parameter(ACTION , METADATA_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;

  private MetadataService mservice_ ;
  private UIPageListIterator uiPageIterator_ ;  

  public UIMetadataManager(MetadataService mservice) throws Exception{
    setId("UIMetadataManager") ;
    setRendererType("VelocityRenderer") ;     
    mservice_ = mservice ;    
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;    
    getChildren().add(uiPageIterator_);       
    addActionListener(ViewActionListener.class, VIEW_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;    
    addActionListener(BackActionListener.class, BACK_ACTION) ;                
  }  

  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }

  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
  public Parameter[] getMetadataParams() { return METADATA_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }

  public void update() throws Exception {
    uiPageIterator_.getPageListDataHandler().setDatas(mservice_.getAllMetadataTypes()) ;
  }

  public boolean isInternalUse(String metaTypeName) throws Exception {
    ExtendedNodeType metaType = (ExtendedNodeType)mservice_.getMetadataNodeType(metaTypeName) ;
    PropertyDefinition def = metaType.getPropertyDefinitions(INTERNAL_USE).getAnyDefinition();
    return def.getDefaultValues()[0].getBoolean() ;
  }

  public boolean hasDialogTemplate(String metaTypeName) throws Exception {
    if(mservice_.getMetadataTemplatePath(metaTypeName,true) == null) 
      return false ;
    return true;    
  }

  public boolean hasViewTemplate(String metaTypeName) throws Exception {
    if(mservice_.getMetadataTemplatePath(metaTypeName,true) == null) 
      return false ;
    return true;    
  }

  public String getIcon(String nodeType) {
    return CMSUtils.getIconByNodeTypeName(nodeType) ;
  }

  static public class ViewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager uicomp = (UIMetadataManager)event.getComponent() ;
      String metadataName = event.getParameter(OBJECTID) ;
      NodeType nodeType = uicomp.mservice_.getMetadataNodeType(metadataName) ;
      UIMetadataView metadataView = (UIMetadataView)uicomp.getSibling(UIMetadataView.class) ;
      metadataView.setMetadata(nodeType) ;
      uicomp.setRenderedSibling(UIMetadataView.class) ;
    }
  }

  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager uicomp = (UIMetadataManager)event.getComponent() ;
      String metadataName = event.getParameter(OBJECTID) ;
      UIMappingMetadataForm metadataForm = 
        (UIMappingMetadataForm)uicomp.getSibling(UIMappingMetadataForm.class) ;
      metadataForm.update(metadataName) ;
      uicomp.setRenderedSibling(UIMappingMetadataForm.class) ;
    }
  }

  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager uicomp = (UIMetadataManager)event.getComponent() ;
      String metadataName = event.getParameter(OBJECTID) ;
      Information iprovider = findInformationProvider(uicomp) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      try{
        uicomp.mservice_.removeMetadataTemplate(metadataName) ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIMetadataManager.msg.remove-successfully"),new Object[]{metadataName}));
      }catch (Exception e) {
        JCRExceptionManager.process(iprovider,e) ;
      }           
    }
  }

  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager metadadaManager = (UIMetadataManager)event.getComponent() ;
      metadadaManager.setRenderedSibling(UIECMAdminSummary.class) ;
    }
  }
}
