package org.exoplatform.portlets.content.namespace.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import javax.jcr.NamespaceRegistry;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.jcr.RepositoryService;

public class UINamespaceRegistry extends UIExoCommand{
  public final static String ADD_NAMESPACE_ACTION = "add".intern() ;
  public final static String REMOVE_NAMESPACE_ACTION = "remove".intern() ;
  public final static String EDIT_NAMESPACE_ACTION = "edit".intern() ;
  public final static String [] SYSTEM_NAMESPACES_PREFIX 
  = { "jcr","exo","dc","nt","mix","xml","sv","xs","fn","" } ;
  
  static Parameter[] ADD_NAMESPACE_PARAMS = {new Parameter(ACTION,ADD_NAMESPACE_ACTION)} ;
  static Parameter[] REMOVE_NAMESPACE_PARAMS = {new Parameter(ACTION,REMOVE_NAMESPACE_ACTION)} ;
  static Parameter[] EDIT_NAMESPACE_PARAMS = {new Parameter(ACTION,EDIT_NAMESPACE_ACTION)} ;
  static Parameter[] BACK_PARAMS     = { new Parameter(ACTION,BACK_ACTION) };		
  
  protected RepositoryService repositoryService_ ;
  protected NamespaceRegistry namespaceRegistry_ ;
  public UIPageListIterator uiPageIterator_ ;  
  
  public UINamespaceRegistry(RepositoryService repositoryService) throws Exception {
    setId("NamespaceRegistry") ;
    setRendererType("VelocityRenderer") ;    
    repositoryService_ = repositoryService ;
    namespaceRegistry_ = repositoryService_.getRepository().getNamespaceRegistry() ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(AddNamespaceActionListener.class,ADD_NAMESPACE_ACTION) ;
    addActionListener(RemoveNamespaceActionListener.class,REMOVE_NAMESPACE_ACTION) ;
    addActionListener(EditNamespaceActionListener.class,EDIT_NAMESPACE_ACTION) ;
    addActionListener(BackActionListener.class,BACK_ACTION) ;
    uiPageIterator_.setPageList(new ContentPageList(getAllNamsepacePrefix())); 
  }
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getAddNamespaceParams() { return ADD_NAMESPACE_PARAMS ; }
  public Parameter[] getRemoveNamespaceParams() { return REMOVE_NAMESPACE_PARAMS ; }
  public Parameter[] getEditNamespaceParams() { return EDIT_NAMESPACE_PARAMS ; }
  public Parameter[] getBackParams() {return BACK_PARAMS ; }		
  
  public List getAllNamsepacePrefix() throws Exception {	   
    List list = new ArrayList() ;   
    String[] prefixs = namespaceRegistry_.getPrefixes();
    for(int i = 0; i < prefixs.length; i++){      
      list.add(prefixs[i]);
    }
    return list;	
  }
  
  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getAllNamsepacePrefix()));        
  }  
  
  public String getNamespaceURI(String prefix) throws Exception {
    return namespaceRegistry_.getURI(prefix) ;
  }
  
  public boolean isSystemPrefix(String prefix) {
    for(String systemPrefix: SYSTEM_NAMESPACES_PREFIX) {
      if(prefix.equals(systemPrefix)) return true ;			
    }
    return false ;
  }
  
  public static class AddNamespaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINamespaceRegistry uiRegistry = (UINamespaceRegistry)event.getComponent() ;
      uiRegistry.setRenderedSibling(UINamespaceRegisterForm.class) ;
    }		
  }
  
  public static class RemoveNamespaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {		
      UINamespaceRegistry uiRegistry = (UINamespaceRegistry)event.getComponent() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiRegistry) ;
      iprovider.addMessage(new ExoFacesMessage(res.getString("UINamspaceRegistry.msg.remove-not-supported"))) ;
      return ;
    }
  }
  
  public static class EditNamespaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {		
      UINamespaceRegistry uiRegistry = (UINamespaceRegistry)event.getComponent() ;      
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiRegistry) ;
      iprovider.addMessage(new ExoFacesMessage(res.getString("UINamspaceRegistry.msg.edit-not-supported"))) ;       
      return ;
    }
  }
  
  public static class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINamespaceRegistry uiRegistry = (UINamespaceRegistry)event.getComponent() ;
      uiRegistry.setRenderedSibling(UIECMAdminSummary.class) ;
    }
  }
}
