package org.exoplatform.portlets.content.nodetypes.component;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.nodetypes.component.UINodeTypeList.NodeTypeNameComparator;
import org.exoplatform.services.download.DownloadResource;
import org.exoplatform.services.download.DownloadService;
import org.exoplatform.services.download.InputStreamDownloadResource;

public class UINodeTypeExport extends UISimpleForm {
  
  protected NodeType nodeType_ ;
  protected NodeTypeManager ntManager_ ;
  private List<NodeType> nodeTypeList_ = new ArrayList<NodeType>() ;
  private static String EXPORT_ACTION = "export" ;
  
  public static Parameter[] EXPORT_PARAMS = { new Parameter(ACTION , EXPORT_ACTION)} ;
  
  public UINodeTypeExport() {
    super("uiNodeTypeExport", "post", null) ;
    setRendererType("SimpleFormVelocityRenderer") ;
    
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(ExportActionListener.class, EXPORT_ACTION) ;
  }
  
  public void update(NodeTypeManager ntManager) throws Exception{
    ntManager_ = ntManager ;
    nodeTypeList_.clear() ;
    NodeTypeIterator nodeTypeIter = ntManager_.getAllNodeTypes() ;
    while(nodeTypeIter.hasNext()) {
      NodeType nodeType = nodeTypeIter.nextNodeType() ;
      UICheckBox checkbox = new UICheckBox(nodeType.getName(), "") ;
      checkbox.setSelect(true) ;
      add(checkbox) ;
      nodeTypeList_.add(nodeType) ;
    }
    Collections.sort(nodeTypeList_, new NodeTypeNameComparator()) ;
  }
  
  public List getNodeTypes() {
    return nodeTypeList_ ;
  }
  
  public String getIcon(String nodeType) {
    return CMSUtils.getIconByNodeTypeName(nodeType) ;
  }
  
  public String getExportParams() { return EXPORT_ACTION ;  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeExport uiForm = (UINodeTypeExport) event.getSource() ;
      uiForm.setRenderedSibling(UINodeTypeList.class) ;
    }
  }
  
  static public class ExportActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeExport uiForm = (UINodeTypeExport) event.getSource() ;
      List<NodeType> selectedNodes = new ArrayList<NodeType>() ;
      for(int i = 0; i < uiForm.nodeTypeList_.size(); i ++) {
        NodeType nodeType = uiForm.nodeTypeList_.get(i) ;
        boolean checked = uiForm.getUICheckBox(nodeType.getName()).getSelect() ;
        if(checked) selectedNodes.add(nodeType) ;
      }
      String nodeTypeXML = getNodeTypeXML(selectedNodes) ;
      ByteArrayInputStream is = new ByteArrayInputStream(nodeTypeXML.getBytes()) ;
      DownloadResource dresource ;
      DownloadService dservice =
        (DownloadService) PortalContainer.getComponent(DownloadService.class);
      FacesContext ctx = uiForm.getFacesContext();
      dresource = new InputStreamDownloadResource(is, "text/xml") ;
      dresource.setDownloadName("nodetypeexport.xml");
      String downloadId = dservice.addDownloadResource(dresource) ;
      String downloadLink = dservice.getDownloadLink(downloadId) ;
      HttpServletResponse response =(HttpServletResponse)ctx.getExternalContext().getResponse();      
      response.sendRedirect(downloadLink);
      ctx.responseComplete();
    }
    private String getNodeTypeXML(List selectedNodes) {
      StringBuffer nodeTypeXML = new StringBuffer() ;
      nodeTypeXML.append("<nodeTypes xmlns:nt=").append("\"") ;
      nodeTypeXML.append("http://www.jcp.org/jcr/nt/1.0").append("\" ") ;
      nodeTypeXML.append("xmlns:mix=").append("\"") ;
      nodeTypeXML.append("http://www.jcp.org/jcr/mix/1.0").append("\" ") ;
      nodeTypeXML.append("xmlns:jcr=").append("\"").append("http://www.jcp.org/jcr/1.0") ;
      nodeTypeXML.append("\" >").append("\n") ;
      for(int i = 0; i < selectedNodes.size(); i++) {
        NodeType nodeType = (NodeType)selectedNodes.get(i) ;
        nodeTypeXML.append("<nodeType ") ;
        nodeTypeXML.append("name=").append("\"").append(nodeType.getName()).append("\" ") ;
        nodeTypeXML.append("isMixin=").append("\"")
                   .append(String.valueOf(nodeType.isMixin())).append("\" ") ;
        nodeTypeXML.append("hasOrderableChildNodes=\"")
                   .append(String.valueOf(nodeType.hasOrderableChildNodes())).append("\" ") ;
        nodeTypeXML.append("primaryItemName=").append("\"")
               .append((nodeType.getPrimaryItemName() != null ? nodeType.getPrimaryItemName() : ""))
                   .append("\" >") ;
        nodeTypeXML.append("\n") ;
        // represent supertypes
        String representSuperType = representSuperTypes(nodeType) ;
        nodeTypeXML.append(representSuperType) ;
        // represent PropertiesDefinition
        String representPropertiesXML = representPropertyDefinition(nodeType) ; 
        nodeTypeXML.append(representPropertiesXML) ;
        // represent ChildNodeDefinition
        String representChildXML = representChildNodeDefinition(nodeType) ;
        nodeTypeXML.append(representChildXML) ;
        nodeTypeXML.append("</nodeType>").append("\n") ;
      }
      nodeTypeXML.append("</nodeTypes>") ;
      return nodeTypeXML.toString() ;
    }
    
    private String representSuperTypes(NodeType nodeType){
      StringBuffer superTypeXML = new StringBuffer() ;
      NodeType[] superType = nodeType.getDeclaredSupertypes() ;
      if(superType != null && superType.length > 0){
        superTypeXML.append("<supertypes>").append("\n") ;
        for(int i = 0 ; i < superType.length ; i ++){
          superTypeXML.append("<supertype>").append(superType[i].getName())
                      .append("</supertype>").append("\n") ;
        }
        superTypeXML.append("</supertypes>").append("\n") ;
      }
      return superTypeXML.toString() ;
    }
    private String representPropertyDefinition(NodeType nodeType) {
      String[] requireType = {"undefined","String","Binary","Long","Double","Date","Boolean",
                                                                      "Name","Path","References"} ;
      String[] onparentVersion = {"","COPY","VERSION","INITIALIZE","COMPUTE","IGNORE","ABORT"} ;
      StringBuffer propertyXML = new StringBuffer() ;
      propertyXML.append("<propertyDefinitions>").append("\n") ;        
        PropertyDefinition[] proDef = nodeType.getPropertyDefinitions() ;
        for(int j = 0 ; j < proDef.length; j ++){
         propertyXML.append("<propertyDefinition ") ;
         propertyXML.append("name=").append("\"").append(proDef[j].getName()).append("\" ") ;         
         propertyXML.append("requiredType=").append("\"")
                    .append(requireType[proDef[j].getRequiredType()]).append("\" ") ;         
         propertyXML.append("autoCreated=").append("\"")
                    .append(String.valueOf(proDef[j].isAutoCreated())).append("\" ") ;         
         propertyXML.append("mandatory=").append("\"")
                    .append(String.valueOf(proDef[j].isMandatory())).append("\" ") ;         
         propertyXML.append("onParentVersion=").append("\"")
                    .append(onparentVersion[proDef[j].getOnParentVersion()]).append("\" ") ;         
         propertyXML.append("protected=").append("\"")
                    .append(String.valueOf(proDef[j].isProtected())).append("\" ") ;
         propertyXML.append("multiple=").append("\"")
                    .append(String.valueOf(proDef[j].isMultiple())).append("\" >").append("\n") ;         
         String[] constraints = proDef[j].getValueConstraints() ;
         if(constraints != null && constraints.length > 0){
           propertyXML.append("<valueConstraints>").append("\n") ;
           for(int k = 0; k < constraints.length ; k++){
             propertyXML.append("<valueConstraint>").append(constraints[k].toString())
             .append("</valueConstraint>").append("\n") ;
           }
           propertyXML.append("</valueConstraints>").append("\n") ;
         }else {
           propertyXML.append("<valueConstraints/>").append("\n") ;
         }
         propertyXML.append("</propertyDefinition>").append("\n") ;
        }
      propertyXML.append("</propertyDefinitions>").append("\n") ;
      return propertyXML.toString() ;
    }
    
    private String representChildNodeDefinition(NodeType nodeType) {
      String[] onparentVersion = {"","COPY","VERSION","INITIALIZE","COMPUTE","IGNORE","ABORT"} ;
      StringBuffer childNodeXML = new StringBuffer() ;
      NodeDefinition[] childDef = nodeType.getChildNodeDefinitions() ;
      if(childDef != null && childDef.length > 0) {
        childNodeXML.append("<childNodeDefinitions>").append("\n") ;
        for(int j = 0 ; j < childDef.length ; j ++){
           childNodeXML.append("<childNodeDefinition ") ;
             childNodeXML.append("name=").append("\"").append(childDef[j].getName()).append("\" ") ;
             NodeType defaultType = childDef[j].getDefaultPrimaryType() ;
             if(defaultType != null){
               childNodeXML.append("defaultPrimaryType=").append("\"")
                           .append(defaultType.getName()).append("\" ") ;
             }else {
               childNodeXML.append("defaultPrimaryType=").append("\"").append("\" ") ;
             }               
             childNodeXML.append("autoCreated=").append("\"")
                         .append(String.valueOf(childDef[j].isAutoCreated())).append("\" ") ;
             childNodeXML.append("mandatory=").append("\"")
                         .append(String.valueOf(childDef[j].isMandatory())).append("\" ") ;
             childNodeXML.append("onParentVersion=").append("\"")
                         .append(onparentVersion[childDef[j].getOnParentVersion()]).append("\" ") ;
             childNodeXML.append("protected=").append("\"")
                         .append(String.valueOf(childDef[j].isProtected())).append("\" ") ;
             childNodeXML.append("sameNameSiblings=").append("\"")
                         .append(String.valueOf(childDef[j].allowsSameNameSiblings()))
                         .append("\" >").append("\n") ;
             NodeType[] requiredType = childDef[j].getRequiredPrimaryTypes() ;
             if(requiredType != null && requiredType.length > 0 ){
               childNodeXML.append("<requiredPrimaryTypes>").append("\n") ;
               for(int k = 0 ; k < requiredType.length ; k ++) {
                 childNodeXML.append("<requiredPrimaryType>")
                             .append(requiredType[k].getName())
                             .append("</requiredPrimaryType>").append("\n") ;
               }
               childNodeXML.append("</requiredPrimaryTypes>").append("\n") ;
             }
           childNodeXML.append("</childNodeDefinition>").append("\n") ; 
        }          
        childNodeXML.append("</childNodeDefinitions>").append("\n") ;
      }
      return childNodeXML.toString() ;
    }
  }
}
