package org.exoplatform.portlets.content.search.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.query.Query;
import javax.jcr.query.QueryResult;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.queries.QueryService;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;

public class UIQueryList extends UIExoCommand {

  public static final String EXECUTE_ACTION = "execute".intern();

  public static Parameter[] EXECUTE_PARAMS = { new Parameter(ACTION, EXECUTE_ACTION) };
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION, DELETE_ACTION) };

  private List<String> roles_ = new ArrayList<String>() ;
  protected QueryService queryService_;  
  private List<Node> sharedQueries_ = new ArrayList<Node> ();

  public UIQueryList(QueryService queryService, OrganizationService oservice) throws Exception{
    setId("UIQueryList");
    setRendererType("VelocityRenderer");    

    queryService_ = queryService;    

    String userName = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
    Collection memberships = oservice.getMembershipHandler().findMembershipsByUser(userName) ;
    if(memberships != null && memberships.size() > 0){
      Object[] objects = memberships.toArray() ;      
      for(int i = 0 ; i < objects.length ; i ++ ){
        Membership membership = (Membership)objects[i] ;
        String role = membership.getMembershipType() + ":" + membership.getGroupId() ;
        roles_.add(role) ;
      } 
    }
    roles_.add(userName) ;
    addActionListener(ExecuteActionListener.class, EXECUTE_ACTION);
    addActionListener(DeleteActionListener.class, DELETE_ACTION);
  }

  public Parameter[] getExecuteParams() { return EXECUTE_PARAMS; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS; }

  public boolean hasQueries() throws Exception {
    List<Query> queries = queryService_.getQueries();
    if (queries == null || queries.isEmpty())
      return false;
    return true;
  }

  public List<Query> getQueries() throws Exception {
    return queryService_.getQueries();
  }

  public boolean hasSharedQueries() throws Exception {
    if(roles_.size() > 0) {
      sharedQueries_ = queryService_.getSharedQueriesByPermissions(roles_) ;
      if(sharedQueries_.size() > 0) return true ;      
    }
    return false ;            
  }

  public List<Node> getSharedQueries() { return sharedQueries_ ; }  

  static public class ExecuteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryList uicomp = (UIQueryList) event.getComponent();
      String queryPath = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        long beforeTime = System.currentTimeMillis();
        QueryResult queryResult = uicomp.queryService_.execute(queryPath);
        long duration = System.currentTimeMillis() - beforeTime;
        UIResults results = (UIResults) uicomp.getSibling(UIResults.class);
        results.resultMap_.clear() ;
        results.setQueryResults(duration, queryResult);
        uicomp.setRenderedSibling(UIResults.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryList uicomp = (UIQueryList) event.getComponent();
      String queryPath = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.queryService_.removeQuery(queryPath);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

}
