/*
 * Created on Apr 8, 2005
 */
package org.exoplatform.portlets.content.templates.component;

import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * @author benjaminmestrallet
 */
public class UITemplatesList extends UIExoCommand {
  
  static public String DELETE_TEMPLATE = "deleteTemplate" ;
  static public String EDIT_TEMPLATE = "editTemplate" ;
  static public String ADD_TEMPLATE = "addTemplate" ;
  
  static Parameter[] DELETE_TEMPLATE_PARAMS = { new Parameter(ACTION , DELETE_TEMPLATE)} ;
  static Parameter[] EDIT_TEMPLATE_PARAMS = { new Parameter(ACTION , EDIT_TEMPLATE)} ;
  static Parameter[] ADD_TEMPLATE_PARAMS = { new Parameter(ACTION , ADD_TEMPLATE)} ;  
  
  protected String nodetype;
  protected boolean isDialog_;
  protected TemplateService templateService_;
  
  public UITemplatesList(TemplateService templateService) throws Exception {
    setId("UITemplatesList") ;
    setRendererType("VelocityRenderer") ;
    
    templateService_ = templateService;
    
    addActionListener(DeleteTemplateActionListener.class, DELETE_TEMPLATE)  ;
    addActionListener(EditTemplateActionListener.class, EDIT_TEMPLATE)  ;
    addActionListener(AddTemplateActionListener.class, ADD_TEMPLATE)  ;
  }
  
  public void setNodeType(String nodeType) { this.nodetype = nodeType; }
  
  public void setDialog(boolean isDialog) { isDialog_ = isDialog; }
  
  public boolean addDialog() {
    if (isDialog_) return true;   
    return false;
  }
  
  public NodeIterator getTemplateIterator() throws Exception {
    return templateService_.getAllTemplatesOfNodeType(isDialog_, nodetype);
  }
  
  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable")
    && !node.isNodeType("nt:frozenNode");
  }
     
  public String getBaseVersion(Node node) throws RepositoryException {
    if(!isVersionable(node))
      return "";
    return node.getBaseVersion().getName();
  }
  
  public Parameter[] getDeleteTemplateParams() { return DELETE_TEMPLATE_PARAMS ; }
  public Parameter[] getEditTemplateParams() { return EDIT_TEMPLATE_PARAMS ; }
  public Parameter[] getAddTemplateParams() { return ADD_TEMPLATE_PARAMS ; }
  
  static public class DeleteTemplateActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UITemplatesList uicomp = (UITemplatesList) event.getComponent() ;
      String templateName = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);      
      for (String template : TemplateService.UNDELETABLE_TEMPLATES) {        
        if(template.equals(templateName)){
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
              res.getString("UITemplatesList.msg.undeletable"))) ;          
          return;
        }
      }
      try {
        uicomp.templateService_.removeTemplate(uicomp.isDialog_, uicomp.nodetype, templateName);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class EditTemplateActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      
      UITemplatesList uicomp = (UITemplatesList) event.getComponent() ;
      String templateName = event.getParameter(OBJECTID);    
      UITemplatesManager manager = (UITemplatesManager) uicomp.getAncestorOfType(UITemplatesManager.class);
      UITemplateFormController templateFormController = 
        (UITemplateFormController) manager.getChildComponentOfType(UITemplateFormController.class);
      templateFormController.showEditTemplateForm(uicomp.isDialog_, uicomp.nodetype, templateName);      
      manager.setRenderedComponent(UITemplateFormController.class);        
    }
  }
  
  static public class AddTemplateActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplatesList uicomp = (UITemplatesList) event.getComponent();      
      UIListsManager manager = (UIListsManager) uicomp.getParent();            
      UITemplateFormController templateFormController = (UITemplateFormController) manager.getSibling(UITemplateFormController.class);
      templateFormController.showNewTemplateForm(uicomp.isDialog_, manager.nodeType);
      manager.setRenderedSibling(UITemplateFormController.class);
    }
  }
}
