package org.exoplatform.portlets.content.views.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.views.ManageViewService;

public class UIECMTemplateList extends UIExoCommand {

  final static public String ADD_ACTION = "add".intern(); 

  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;

  private ManageViewService vservice_ ;
  public UIPageListIterator uiPageIterator_ ;  
  
  public UIECMTemplateList(ManageViewService vservice) throws Exception{
    setId("UITemplateList") ;
    setRendererType("VelocityRenderer") ;
    vservice_ = vservice ;    
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(AddActionListener.class, ADD_ACTION) ;    
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    uiPageIterator_.setPageList(new ContentPageList(getAllTemplates()));  
  }  
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }  
  public Parameter[] getBackParams() { return BACK_PARAMS ; }

  public List getAllTemplates(){
    try{
      return vservice_.getAllTemplates(BasePath.ECM_EXPLORER_TEMPLATES) ;
    }catch(Exception e) {
      return new ArrayList() ;
    }    
  }

  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getAllTemplates()));        
  }
  
  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMTemplateList templateList = (UIECMTemplateList)event.getSource() ;
      UITemplateForm templateForm = (UITemplateForm)templateList.getSibling(UITemplateForm.class) ;
      Information iprovider = findInformationProvider(templateList) ;
      try {
        templateForm.update(true, null) ;
      }catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
        return ;
      }
      templateList.setRenderedSibling(UITemplateForm.class) ;
    }
  }

  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIECMTemplateList templateList = (UIECMTemplateList)event.getSource() ;
      String templatePath = event.getParameter(OBJECTID) ;
      UITemplateForm templateForm = (UITemplateForm)templateList.getSibling(UITemplateForm.class) ;
      try {
        templateForm.update(true, templatePath) ;        
      }catch (Exception e) {        
        JCRExceptionManager.process(findInformationProvider(templateList),e) ;
        return ;
      }
      templateList.setRenderedSibling(UITemplateForm.class) ;
    }
  }

  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMTemplateList templateList = (UIECMTemplateList)event.getSource() ;
      Information iprovider = findInformationProvider(templateList) ;
      String templatePpath = event.getParameter(OBJECTID) ;
      try {
        templateList.vservice_.removeTemplate(templatePpath) ;
      }catch (Exception e) {
        JCRExceptionManager.process(iprovider, e) ;        
      }    
      templateList.refresh();
    }
  } 

  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMTemplateList tempList = (UIECMTemplateList)event.getSource() ;
      UIManageView view = (UIManageView)tempList.getAncestorOfType(UIManageView.class) ;
      view.setRenderedSibling(UIECMAdminSummary.class) ;
    }
  }

}
