package org.exoplatform.portlets.content.views.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.version.Version;
import javax.jcr.version.VersionException;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.versions.component.VersionNode;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.cms.views.impl.ViewDataImpl.Tab;
import org.exoplatform.services.jcr.RepositoryService;

public class UIViewForm extends UISimpleForm implements SelectorHandler {

  final private static String ADD_TAB_ACTION = "addTab".intern() ;
  final private static String ADD_TEMPLATE_ACTION = "addTemplate".intern() ;
  final private static String EDIT_TAB_ACTION = "editTab".intern() ;
  final private static String REMOVE_TAB_ACTION = "removeTab".intern() ;
  static String VIEW_ENABLE_VERSIONING = "enableVersioning";
  static String RESTORE_VERSION_ACTION = "restoreVersion".intern() ;
  
  public static Parameter[] ADD_TAB_PARAMS = { new Parameter(ACTION , ADD_TAB_ACTION)} ;
  public static Parameter[] ADD_TEMPLATE_PARAMS = { new Parameter(ACTION , ADD_TEMPLATE_ACTION)} ;
  public static Parameter[] REMOVE_TAB_PARAMS = { new Parameter(ACTION , REMOVE_TAB_ACTION)} ;
  public static Parameter[] EDIT_TAB_PARAMS = { new Parameter(ACTION , EDIT_TAB_ACTION)} ;
  public static final String SEARCH_GROUP = "searchGroup";
  protected UIExoPermissionSelector userOrGroup_;
  private List<Node> tabs_ = new ArrayList<Node>() ;
  private List<Tab> newTabs_ = new ArrayList<Tab>() ;
  private boolean isView_ = false ;
  private ManageViewService vservice_ ;  
  private Node views_;
  private UISelectBox template_ ;
  protected boolean newView_;
  protected UICheckBox viewEnableVersioning_;
  protected Version baseVersion_;
  protected VersionNode selectedVersion_;
  
  public UIViewForm(ManageViewService vservice) throws Exception {
    super("uiViewForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    vservice_ = vservice ;
    UIStringInput name = new UIStringInput("name", "") ;
    userOrGroup_ = new UIExoPermissionSelector("userOrGroup", null);
    //userOrGroup_.addValidator(PermissionValidator.class) ;
    List<SelectItem> temp = new ArrayList<SelectItem>() ;
    Node ecmTemplateHome = vservice_.getTemplateHome(BasePath.ECM_EXPLORER_TEMPLATES) ;
    for(NodeIterator iter = ecmTemplateHome.getNodes();iter.hasNext();) {
      Node tempNode = iter.nextNode() ;
      temp.add(new SelectItem(tempNode.getName(),tempNode.getPath())) ;
    }
    template_ = new UISelectBox("template","List",temp) ;
    add(name) ;
    add(userOrGroup_) ;
    add(template_) ;    
    viewEnableVersioning_ = new UICheckBox(VIEW_ENABLE_VERSIONING, null);    
    viewEnableVersioning_.setSelect(false) ;
    add(viewEnableVersioning_) ;    
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(AddTabActionListener.class, ADD_TAB_ACTION);
    addActionListener(RemoveTabActionListener.class, REMOVE_TAB_ACTION);
    addActionListener(EditTabActionListener.class, EDIT_TAB_ACTION);
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP);
    addActionListener(RestoreVersionActionListener.class, RESTORE_VERSION_ACTION);  
  }
  public String getRestoreVersionAction() { return RESTORE_VERSION_ACTION; }
  public Parameter[] getEditTabParams() { return EDIT_TAB_PARAMS ; }
  public String getSearchGroupAction() { return SEARCH_GROUP; }
  public String getAddTabAction() { return ADD_TAB_ACTION ; }
  public String getRemoveTabAction() { return REMOVE_TAB_ACTION ; }

  public boolean isView() { return isView_ ; }
  public List getTabs() { return tabs_ ;  }
  public List getNewTabs() { return newTabs_ ;  }
  public void addNewTab(String tabName, String buttons){
    Tab tab = new Tab() ;
    tab.setTabName(tabName) ;
    tab.setButtons(buttons) ;
    newTabs_.add(tab) ;
  }
  public boolean hasTabs() {
    if(tabs_.size() > 0) return true ;
    return false ;
  }

  public void refresh() throws Exception {
    getUIStringInput("name").setValue("") ;
    userOrGroup_.setValue("") ;
    tabs_ = new ArrayList<Node>() ;
    newTabs_ = new ArrayList<Tab>() ;
    isView_ = false ;
    views_ = null ;
    newView_ = true;
    selectedVersion_ = null;
    getUIStringInput("name").setEditable(true) ;
    getUIInput("userOrGroup").setEditable(true) ;
    
    List<SelectItem> temp = new ArrayList<SelectItem>() ;
    Node ecmTemplateHome = vservice_.getTemplateHome(BasePath.ECM_EXPLORER_TEMPLATES) ;
    for(NodeIterator iter = ecmTemplateHome.getNodes();iter.hasNext();) {
      Node tempNode = iter.nextNode() ;
      temp.add(new SelectItem(tempNode.getName(),tempNode.getPath())) ;
    }
    template_.setOptions(temp) ;
    getUISelectBox("template").setEditable(true) ;     
  }

  public void update(Node viewNode, boolean isView, VersionNode selectedVersion) throws Exception {    
    if (selectedVersion != null) {      
      baseVersion_ = viewNode.getBaseVersion();
      viewNode.restore(selectedVersion.getVersion(), false) ;
      viewNode.checkout() ;        
      selectedVersion_ = selectedVersion;         
    } 
    newView_ = false ;
    isView_ = isView ;
    views_ = viewNode ;    
    getUIStringInput("name").setValue(viewNode.getName()) ;
    getUIInput("userOrGroup").setValue(viewNode.getProperty("exo:permissions").getString()) ;
    getUISelectBox("template").setValue(viewNode.getProperty("exo:template").getString()) ;
    getUIStringInput("name").setEditable(false) ;
    if(isView) {
      getUIInput("userOrGroup").setEditable(false) ;
      getUISelectBox("template").setEditable(false) ;
    }else{
      getUIInput("userOrGroup").setEditable(true) ;
      getUISelectBox("template").setEditable(true) ;      
    }      
    if (isVersioned()) {              
      viewEnableVersioning_.setSelect(true);  
      viewEnableVersioning_.setEditable(false);        
    } else if (canEnableVersionning()) {       
        viewEnableVersioning_.setSelect(false);   
        viewEnableVersioning_.setEditable(true);  
    }       
    tabs_ = new ArrayList<Node>() ;
    newTabs_ = new ArrayList<Tab>() ;    
    for(NodeIterator iter = viewNode.getNodes() ;iter.hasNext() ;) {
      tabs_.add(iter.nextNode()) ;
    }       
  }

  public boolean isVersioned() throws RepositoryException {          
    if (!newView_)      
      return views_.isNodeType("mix:versionable");    
    return false;
  }

  public boolean isEnableRestoring() throws Exception {    
    if (isVersioned()) {
      long versionSize = views_.getVersionHistory().getAllVersions().getSize();
      if (versionSize > 2) return true;     
    }
    return false;
  }
  
  public boolean isNewView() throws Exception {        
    return newView_;
  }
  
  public boolean canEnableVersionning() throws Exception {
    return views_.canAddMixin("mix:versionable");
  }
  
  public void cancel(Selector selector) { 
    selector.setRenderedSibling(UIViewFormController.class) ;
  }

  public void select(Selector selector) throws Exception {     
    UIExoPermissionBrowser browser = (UIExoPermissionBrowser)selector ;
    if(browser.isGroup()) {
      userOrGroup_.setPermission(selector.getInputValue());
    }else {
      userOrGroup_.setValue(browser.getSelectedUser()) ;
    }
    selector.setRenderedSibling(UIViewFormController.class);
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIViewForm uiForm = (UIViewForm)event.getSource() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiForm) ;

      String viewName = uiForm.getUIStringInput("name").getValue() ;
      if(viewName == null || viewName.length() < 1){
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIViewForm.msg.view-name-invalid"))) ;
        return ;
      }
      if(uiForm.tabs_.size() < 1 && uiForm.newTabs_.size() < 1){
        String[] error = {viewName} ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIViewForm.msg.mustbe-add-tab"),error)) ;
        return ;
      }
      String permissions = uiForm.userOrGroup_.getValue() ;
      String template = uiForm.getUISelectBox("template").getValue() ;

      ManageViewService vservice = (ManageViewService) PortalContainer.getInstance()
      .getComponentInstanceOfType(ManageViewService.class) ;

      if(uiForm.views_ != null){
        String viewPath = vservice.addView(viewName, permissions, template, uiForm.tabs_) ;
        RepositoryService repositoryService = (RepositoryService)PortalContainer.getComponent(RepositoryService.class) ;
        CmsConfigurationService cmsConfigService = (CmsConfigurationService)PortalContainer.getComponent(CmsConfigurationService.class) ;
        Session session = repositoryService.getRepository().getSystemSession(cmsConfigService.getWorkspace()) ;
        Node node = (Node)session.getItem(viewPath) ;        
        boolean isRuleEnableVersioning = uiForm.viewEnableVersioning_.getSelect();
        if (isRuleEnableVersioning) {
          if (uiForm.canEnableVersionning() && !uiForm.isVersioned()) {              
            node.addMixin("mix:versionable");
            node.save();                    
          } 
          node.checkin();
          node.checkout();                
          node.save();
          session.save();
        }        
        session.logout() ;
      }else {
        vservice.addView(viewName, permissions, template, uiForm.newTabs_) ;
      } 
      UIViewFormController viewFormController = (UIViewFormController) uiForm.getParent();  
      UIViewList viewList = (UIViewList)viewFormController.getSibling(UIViewList.class) ;
      viewList.refresh();
      viewFormController.setRenderedSibling(UIViewList.class) ;
    }
  }

  static public class RestoreVersionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIViewForm uiForm = (UIViewForm) event.getSource();        
      ResourceBundle res = getApplicationResourceBundle() ;      
      Information iprovider = findInformationProvider(uiForm);
      try {                      
        if (uiForm.selectedVersion_ != null) {                   
          Object[] args = {uiForm.selectedVersion_.getName()};        
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
              res.getString("UIViewForm.msg.version-restored"), args));
        }
        UIViewFormController ruleFormController = (UIViewFormController) uiForm.getParent();            
        UIViewList viewList = (UIViewList) ruleFormController.getSibling(UIViewList.class);         
        viewList.refresh();
        viewList.setRenderedSibling(UIViewList.class);      
      } catch (VersionException ex) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
            ex.getMessage()));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
    
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewForm uiForm = (UIViewForm)event.getSource() ;      
      if (uiForm.selectedVersion_ != null) { 
        if (!uiForm.selectedVersion_ .equals(uiForm.baseVersion_)) {
          uiForm.views_.restore(uiForm.baseVersion_, true);
          uiForm.views_.checkout();
        }
      }              
      UIViewFormController viewFormController = (UIViewFormController) uiForm.getParent();             
      viewFormController.setRenderedSibling(UIViewList.class) ;            
    }
  }  

  static public class AddTabActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewForm viewForm = (UIViewForm)event.getSource() ;
      UIViewFormController viewFormController = (UIViewFormController) viewForm.getParent();
      UITabForm tabForm = (UITabForm)viewFormController.getSibling(UITabForm.class) ;           
      tabForm.refresh(viewForm.views_) ;
      viewFormController.setRenderedSibling(UITabForm.class) ;
    }
  }

  static public class RemoveTabActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewForm viewForm = (UIViewForm)event.getSource() ;
      String viewName = viewForm.getUIStringInput("name").getValue() ;
      UIViewFormController viewFormController = (UIViewFormController) viewForm.getParent();
      UIRemoveTabForm tabForm = (UIRemoveTabForm)viewFormController.getSibling(UIRemoveTabForm.class) ;
      tabForm.update(viewName, viewForm.tabs_) ;
      viewFormController.setRenderedSibling(UIRemoveTabForm.class) ;
    }
  }

  static public class EditTabActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewForm viewForm = (UIViewForm)event.getSource() ;
      UIViewFormController viewFormController = (UIViewFormController) viewForm.getParent();
      UITabForm tabForm = (UITabForm)viewFormController.getSibling(UITabForm.class) ;      
      String tabName = event.getParameter(OBJECTID) ;
      Node tab = viewForm.views_.getNode(tabName) ;  
      tabForm.refresh(viewForm.views_) ;
      tabForm.update(tab , viewForm.isView_) ;
      viewFormController.setRenderedSibling(UITabForm.class) ;
    }
  }

  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIViewForm uiForm = (UIViewForm) event.getSource();
      UIViewFormController viewFormController = (UIViewFormController) uiForm.getParent();
      UIExoPermissionBrowser uiGroupBrowser = (UIExoPermissionBrowser) viewFormController
      .getSibling(UIExoPermissionBrowser.class);           
      uiGroupBrowser.setHandler(uiForm);
      viewFormController.setRenderedSibling(UIExoPermissionBrowser.class);     
    }
  }

}
