package org.exoplatform.portlets.content.views.component;

import java.util.Collection;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.cms.views.ManageViewService;

public class UIViewList extends UIExoCommand {
  
  final static public String ADD_ACTION = "add".intern(); 
  final static public String TEMPLATE_ACTION = "template".intern();
  
  public static Parameter[] VIEW_PARAMS = { new Parameter(ACTION , VIEW_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
  
  private ManageViewService vservice_ ;  
  public UIPageListIterator uiPageIterator_ ;  
  
  public UIViewList(ManageViewService vservice) throws Exception{    
    setId("UIViewList") ;
    setRendererType("VelocityRenderer") ;
    vservice_ = vservice;     
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(ViewActionListener.class, VIEW_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    uiPageIterator_.setPageList(new ContentPageList(getAllViews()));       
  }  
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }

  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  
  public List getAllViews() throws Exception{
    return vservice_.getAllViews() ; 
  }
  
  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getAllViews()));        
  }
  
  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable")
    && !node.isNodeType("nt:frozenNode");
  }
  
  public String getBaseVersion(String name) throws Exception {
    Node node = vservice_.getViewByName(name);
    if(!isVersionable(node))
      return "";
    return node.getBaseVersion().getName();    
  }

  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewList viewList = (UIViewList)event.getSource() ;            
      UIViewFormController viewFormController = (UIViewFormController)viewList.getSibling(UIViewFormController.class) ;
      viewFormController.showNewViewForm() ;
      viewList.setRenderedSibling(UIViewFormController.class);                
    }
  }
  
  static public class ViewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewList viewList = (UIViewList)event.getSource() ;
      String viewName = event.getParameter(OBJECTID) ;
      Node viewNode = viewList.vservice_.getViewByName(viewName) ;
      UIViewFormController viewFormController = (UIViewFormController)viewList.getSibling(UIViewFormController.class) ;
      viewFormController.showEditViewForm(viewNode, true) ;
      viewList.setRenderedSibling(UIViewFormController.class);       
    }
  }
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewList viewList = (UIViewList)event.getSource() ;
      String viewName = event.getParameter(OBJECTID) ;
      Node viewNode = viewList.vservice_.getViewByName(viewName) ;
      UIViewFormController viewFormController = (UIViewFormController)viewList.getSibling(UIViewFormController.class) ;      
      viewFormController.showEditViewForm(viewNode, false) ;
      viewList.setRenderedSibling(UIViewFormController.class);                
    }
  }
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewList viewList = (UIViewList)event.getSource() ;
      String viewName = event.getParameter(OBJECTID) ;
      viewList.vservice_.removeView(viewName) ;
      viewList.refresh();
      viewList.setRendered(true) ;
    }
  }
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewList viewList = (UIViewList)event.getSource() ;
      UIManageView view = (UIManageView)viewList.getAncestorOfType(UIManageView.class) ;
      view.setRenderedSibling(UIECMAdminSummary.class) ;
    }
  }
}
