/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.portlets.workflow.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.Locale;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.session.PortalResources;
import org.exoplatform.services.workflow.Form;
import org.exoplatform.services.workflow.Process;
import org.exoplatform.services.workflow.WorkflowFormsService;
import org.exoplatform.services.workflow.WorkflowServiceContainer;

/**
 * Created y the eXo platform team User: Benjamin Mestrallet Date: 15 mai 2004
 */
public class UIBPDefinition extends UIExoCommand {
  public static String START_PROCESS_ACTION = "manageStart";

  public static Parameter[] START_PROCESS_PARAMS = { new Parameter(ACTION,
      START_PROCESS_ACTION) };

  private WorkflowServiceContainer service_;

  private WorkflowFormsService workflowFormsService;

  public UIBPDefinition(WorkflowServiceContainer workflowServiceContainer,
      WorkflowFormsService workflowFormsService) {
    setRendererType("VelocityRenderer");
    setId("UIBPDefinition");
    service_ = workflowServiceContainer;
    this.workflowFormsService = workflowFormsService;

    addActionListener(StartListener.class, START_PROCESS_ACTION);
  }

  public Parameter[] getStartProcessParams() {
    return START_PROCESS_PARAMS;
  }

  public List getProcessDefinitions() {
    Locale locale = ((PortalResources) SessionContainer
        .getComponent(PortalResources.class)).getLocale();

    List<Process> processes = service_.getProcesses();
    
    List visibleDefinitions = new ArrayList();
    for (ListIterator<Process> iter = processes.listIterator(); iter.hasNext();) {
      Process process = iter.next();      
      
      Form form = workflowFormsService.getForm(process.getId(), 
          process.getStartStateName(), locale);
      if (form != null && !form.isDelegatedView()) {
        visibleDefinitions.add(process);
      }
    }
    return visibleDefinitions;
  }

  public static class StartListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIBPDefinition uicomp = (UIBPDefinition) event.getComponent();
      String processId = event.getParameter(OBJECTID);
      if(uicomp.service_.hasStartTask(processId)) {      
        UITask start = (UITask) uicomp.getSibling(UITask.class);
        start.setIdentification(processId);
        start.setIsStart(true);
        start.updateUITree();
        uicomp.setRenderedSibling(UITask.class);
      } else {
        uicomp.service_.startProcess(processId);
      }
    }
  }

}