/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.cms.transform.impl;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.PropertiesParam;

import com.artofsolving.jodconverter.DocumentConverter;
import com.artofsolving.jodconverter.DocumentFormat;
import com.artofsolving.jodconverter.DocumentFormatRegistry;
import com.artofsolving.jodconverter.XmlDocumentFormatRegistry;
import com.artofsolving.jodconverter.openoffice.connection.OpenOfficeConnection;
import com.artofsolving.jodconverter.openoffice.connection.SocketOpenOfficeConnection;
import com.artofsolving.jodconverter.openoffice.converter.OpenOfficeDocumentConverter;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Xuan Hoa
 *          hoa.pham@exoplatform.com
 * Mar 23, 2007  
 */
public class OpenOfficeTransformer extends BaseDocumentTransformer {

  private static DocumentFormatRegistry formatRegistry ;   

  private String ooConnectionHost_ = "localhost";
  private int ooConnectionPort_ = 8100;
  private String ooConnectionType_ ;  
  private MimeTypesConfig mimeTypeConfig_ ;  
  private OpenOfficeConnection connection_ = null;

  public OpenOfficeTransformer(InitParams initParams) {    
    mimeTypeConfig_ = (MimeTypesConfig)initParams.getObjectParamValues(MimeTypesConfig.class).get(0) ;
    PropertiesParam OOConnectionPropperties = initParams.getPropertiesParam("openoffice.connection.properties") ;        
    ooConnectionHost_ = OOConnectionPropperties.getProperty("openoffice.connection.host") ;
    ooConnectionPort_ = Integer.parseInt(OOConnectionPropperties.getProperty("openoffice.connection.port")) ;
    ooConnectionType_ = OOConnectionPropperties.getProperty("openoffice.connection.type") ;
    loadDocumentFomatConfiguaration() ;
  }

  public boolean transform(InputStream input, String soureMimeType, OutputStream output, String destinationMimeType) throws Exception {
    if(!canTransform(soureMimeType,destinationMimeType)) 
      return false ;
    DocumentFormat sourceFormat = formatRegistry.getFormatByMimeType(soureMimeType) ;
    DocumentFormat destinationFormat = formatRegistry.getFormatByMimeType(destinationMimeType) ;                           
    try{
      if(connection_ == null) {
        connection_ = new SocketOpenOfficeConnection(ooConnectionHost_,ooConnectionPort_) ;
      }
      if(!connection_.isConnected())
        connection_.connect() ;
      DocumentConverter converter = new OpenOfficeDocumentConverter(connection_) ;
      converter.convert(input,sourceFormat,output,destinationFormat) ;        
    }catch (Exception e) {                 
      return false ;
    }    
    return true ;
  }

  public boolean canTransform(String sourceMimeType, String destinationMimeType) throws Exception {
    DocumentFormat sourceFormat = formatRegistry.getFormatByMimeType(sourceMimeType) ;
    DocumentFormat destinationFormat = formatRegistry.getFormatByMimeType(destinationMimeType) ;    
    if(sourceFormat == null || destinationFormat == null)      
      return false ;
    return destinationFormat.isExportableFrom(sourceFormat.getFamily()) ;
  }

  public String getDestinationMimeType() throws Exception {   
    return mimeTypeConfig_.getDestinationMimeType();
  }

  public List<String> getSourceMimeTypes() throws Exception {
    return mimeTypeConfig_.getSourceMimeTypes() ;
  }

  private void loadDocumentFomatConfiguaration() {
    try {
      InputStream inputStream = getClass().getResourceAsStream("document-formats.xml") ;
      formatRegistry = new XmlDocumentFormatRegistry(inputStream) ;
    }catch (Exception e) {
      formatRegistry = new XmlDocumentFormatRegistry() ;
    }    
  }
  
}
