/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: TestJCEUtil.java 9514 2006-10-16 21:36:34Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.signer;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import junit.framework.TestCase;

import org.exoplatform.services.signer.impl.JCEUtil;

/**
 * This test case do the following against the HashFunctionService:
 * <ul>
 * <li>validate service generated digest with a known resulting digest 
 * (created using the Digest program in this package).</li>
 * <li>use the following algorithms: MD2, MD5, SHA-1, SHA-256, SHA-384, and
 * SHA-512</li>
 * </ul>
 * 
 * @version $Rev: 9514 $ $Date: 2006-10-17 04:36:34 +0700 (Tue, 17 Oct 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class TestJCEUtil extends TestCase {

	/**
	 * "eXo" string.
	 */
	public static final byte[] EXO_TEXT_TO_DIGEST = "eXo".getBytes();

	private JCEUtil jceUtil;

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
	    jceUtil = JCEUtil.getInstance();
	    super.setUp();
	}

	public void testMD5HashValue() throws Exception {

		InputStream is = new ByteArrayInputStream(EXO_TEXT_TO_DIGEST);
		String digest = Digest.byteArrayToHexString(jceUtil.calculateDigest(is,
				"MD5"));

		// asserting value generated with Digest program (testing package)
		assertEquals("c819aaa891119a3a8592b08d8491038f", digest.toLowerCase());
	}

	public void testMD2HashValue() throws Exception {

		InputStream is = new ByteArrayInputStream(EXO_TEXT_TO_DIGEST);
		String digest = Digest.byteArrayToHexString(jceUtil.calculateDigest(is,
				"MD2"));

		// asserting value generated with Digest program (testing package)
		assertEquals("28a8161fe5e04129c09de283db2fb593", digest.toLowerCase());
	}

	public void testSHA1HashValue() throws Exception {

		InputStream is = new ByteArrayInputStream(EXO_TEXT_TO_DIGEST);
		String digest = Digest.byteArrayToHexString(jceUtil.calculateDigest(is,
				"SHA-1"));

		// asserting value generated with Digest program (testing package)
		assertEquals("6656dbddcac36eab9fe72b7b32c596f03d847a5", digest.toLowerCase());
	}

	public void testSHA256HashValue() throws Exception {

		InputStream is = new ByteArrayInputStream(EXO_TEXT_TO_DIGEST);
		String digest = Digest.byteArrayToHexString(jceUtil.calculateDigest(is,
				"SHA-256"));

		// asserting value generated with Digest program (testing package)
		assertEquals(
				"865fd15bf1d9ecb6549c02eae764c5b9a84a9af124db9fad21d43a72cedfec",
				digest.toLowerCase());
	}

	public void testSHA384HashValue() throws Exception {

		InputStream is = new ByteArrayInputStream(EXO_TEXT_TO_DIGEST);
		String digest = Digest.byteArrayToHexString(jceUtil.calculateDigest(is,
				"SHA-384"));

		// asserting value generated with Digest program (testing package)
		assertEquals(
				"b05440e8a7a146e1451c46232af196369673f605167f57b966ab9e851a37d7e3889c288de6ed8bedd36683c4eb88",
				digest.toLowerCase());
	}

	public void testSHA512HashValue() throws Exception {

		InputStream is = new ByteArrayInputStream(EXO_TEXT_TO_DIGEST);
		String digest = Digest.byteArrayToHexString(jceUtil.calculateDigest(is,
				"SHA-512"));

		// asserting value generated with Digest program (testing package)
		assertEquals(
				"44e927cc5fc8724414e2b7b5aa629fef95578717de25aba6bc25a38c18b4422870484e94b5ba6ceec7a71be62f8d217313ba13841e20e38d2cf2c81077e785a8",
				digest.toLowerCase());
	}
}
