package org.exoplatform.services.workflow.impl.oncepi;

import java.util.List;
import java.util.ResourceBundle;
import java.util.Locale;
import java.util.Hashtable;
import java.util.Map;
import java.util.ArrayList;
import java.util.HashMap;

import org.exoplatform.services.workflow.Form;

import cn.ac.ios.tcse.isflow.ProcessServer.processdeploy.ISWFDeployPackage;
import cn.ac.ios.tcse.isflow.xpdl.Activity;
import cn.ac.ios.tcse.isflow.xpdl.Activities;
import cn.ac.ios.tcse.isflow.xpdl.WorkflowProcess;

public class AutomaticFormImpl implements Form {
	/** Localized Resource Bundle corresponding to this Form */
	private ResourceBundle bundle;

	/** Name of the State corresponding to this Form */
	private String stateName;

	/** Submit buttons corresponding to this Form */
	private List<Map<String, Object>> submitButtons;

	/** Variables corresponding to this Form */
	private Hashtable<String, Map<String, Object>> variables;

	public List getVariables() {
		List ret = new ArrayList<Map<String, Object>>();

		for (Map<String, Object> attributes : variables.values()) {
			ret.add(attributes);
		}

		return ret;
	}

	public List getSubmitButtons() {
		return this.submitButtons;
	}

	public String getStateName() {
		return this.stateName;
	}

	public ResourceBundle getResourceBundle() {
		return this.bundle;
	}

	public boolean isCustomizedView() {

		return false;
	}

	public String getCustomizedView() {

		return null;
	}

	public boolean isDelegatedView() {

		return false;
	}

	public String getIconURL() {
		return "";
	}

	public String getStateImageURL() {
		return "";
	}

	public AutomaticFormImpl(String processId, String stateName, Locale locale) {
		try {
			String packageid = null;
			String workflowid = null;
			String[] data = new IDMappingImpl().getProcessIdTuple(processId);
			if (data == null) {
				/**
				 * Cannot find the key int our map failes
				 */
				return;
			}
			packageid = data[0];
			workflowid = data[1];

			WorkflowProcess wp = null;

			ISWFDeployPackage deploy = new ISWFDeployPackage("");
			cn.ac.ios.tcse.isflow.xpdl.Package pkg = deploy
					.getProcessTemplate(packageid);
			WorkflowProcess[] wparray = pkg.getWorkflowProcesses()
					.getWorkflowProcess();
			if (wparray == null) {
				/**
				 * cannot find workflows in this package
				 */
				return;
			}
			for (int index = 0; index < wparray.length; index++) {
				if (workflowid.equals(wparray[index].getId())) {
					wp = wparray[index];
					break;
				}
			}
			if (wp == null)
				return;

			if (ProcessData.START_STATE_NAME.equals(stateName)) {
				// The Form corresponds to a Start Process one
				this.stateName = ProcessData.START_STATE_NAME;
				// TODO Translate me
				this.bundle = new AutomaticFormBundle("start");
				this.variables = new Hashtable<String, Map<String, Object>>();

				/*
				 * By convention, eXo considers that a panel corresponds to a
				 * start Process one when it contains no submit button. In
				 * addition there needs to be a bundle string for the default
				 * button.
				 */
				this.submitButtons = new ArrayList<Map<String, Object>>();
				((AutomaticFormBundle) this.bundle).addDefaultButton();

				// Only project variables need to be processed when starting a
				// process
				initializeProjectVariables(wp);
			} else {
				// The Form corresponds to an Activity. Retrieve the
				// corresponding Node.
				Activity node = null;
				Activities nodes = wp.getActivities();
				java.util.Iterator iter = nodes.getActivityList().iterator();
				while (iter.hasNext()) {
					if (node.getName().equalsIgnoreCase(this.stateName))
						break;
				}
				if (node == null)
					return;

				this.stateName = node.getName();
				this.bundle = new AutomaticFormBundle(this.stateName);
				this.variables = new Hashtable<String, Map<String, Object>>();

				/*
				 * In Bonita, we consider this is the Workflow duty to determine
				 * which activity comes next hence a single default button for
				 * all activities.
				 */
				this.submitButtons = new ArrayList<Map<String, Object>>();
				Map<String, Object> attributes = new HashMap<String, Object>();
				// TODO Translate me
				attributes.put("name", "submit");
				attributes.put("transition", "");
				this.submitButtons.add(attributes);
				((AutomaticFormBundle) this.bundle).addButton(attributes);

				// Project variables are processed last so they have a higher
				// priority
				initializeNodeVariables(node);
				initializeProjectVariables(wp);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * I am not sure how to implement it!
	 * @param node
	 */
	private void initializeNodeVariables(Activity node) {

	}

	private void initializeProjectVariables(WorkflowProcess wp) {

	}
}
