package org.exoplatform.services.workflow.impl.oncepi;

import java.util.Locale;

import org.exoplatform.services.workflow.Form;
import org.exoplatform.services.workflow.WorkflowFormsService;

public class WorkflowFormsServiceImpl implements WorkflowFormsService {

	 /** Caches the Forms that have been created so far */
	  private FormCache cache = new FormCache();
	  
	  /** Reference to a File Definition Service implementation */
	  private WorkflowFileDefinitionService fileDefinitionService = null;
	  
	public Form getForm(String processId, String stateName,
			Locale locale) {
		 // Determine if the Form is cached yet
	    Form form = cache.getForm(processId, stateName, locale);

	    if(form == null) {
	      // The Form is not found. Retrieve it from the persistent storage
	      FileDefinition fileDefinition =
	        this.fileDefinitionService.retrieve(processId);
	      
	      if(fileDefinition != null && fileDefinition.isFormDefined(stateName)) {
	        // The Form is found in the storage and defined
	        form = new SpecifiedFormImpl(fileDefinition, stateName, locale);
	      }
	      else {
	        // The Form is not found in the storage and not defined
	        form = new AutomaticFormImpl(processId, stateName, locale);
	      }
	      
	      // Cache the Form to speed up subsequent accesses
	      cache.setForm(processId, stateName, locale, form);
	    }

	    return form;
	}

	public void removeForms(String processDefinitionId) {
//		 Remove the specified Forms from the cache
	    this.cache.removeForms(processDefinitionId);

	}
   
	public WorkflowFormsServiceImpl(WorkflowFileDefinitionService fileDefinitionService){
		this.fileDefinitionService=fileDefinitionService;
	}
}
