/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.util.io;

import java.io.File;
import java.io.IOException;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 10.10.2006
 * 
 * Special class for use in binary temp storage (incoming value data storage).
 * 
 * A option is moveTo(File) method which move actual file (this stream source) 
 * to another location on file system (OS dependent).
 * After that this stream can't be used as InputStream, except InputStream.close() method. 
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: LazyTempFileInputStream.java 10024 2006-11-02 17:52:33Z peterit $
 */
public class LazyTempFileInputStream extends LazySharedFileInputStream {

  public LazyTempFileInputStream(TempFile file) {
    super(file);
  }
  
  @Override
  public LazyTempFileInputStream clone() {
    return new LazyTempFileInputStream(getSourceFile());
  }

  @Override
  public TempFile getSourceFile() {
    return (TempFile) file;
  }

  /**
   * Do java.io.File.renameTo(File) work inside with source file.
   * NOTE: The stream will be unavailable as InputStream after this method call and positive result.
   * In case of attempt of the stream use after this method an exception FileNotFoundException will be thrown.
   * 
   * Only one operation can be used after - it's close() method. 
   * 
   * @param destFile - destenation file
   * @return boolean flag if success
   */
  public boolean moveTo(File destFile) {
    boolean res = getSourceFile().renameTo(destFile);
    if (res) {
      dataStream = null;
    }
    return res;
  }

  @Override
  public void close() throws IOException {
    try {
      super.close();
    } finally {
      getSourceFile().delete();
    }
  }

  /**
   * Request of operation of file deletion on finalization.
   * 
   * Actually will be deleted by last live (finalizing currently) TempFile instance.
   */
  @Override
  protected void finalize() throws Throwable {
    
    try {
      this.close();
    } finally {
      super.finalize();
    }
  }
}
