/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.util.Stack;

import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.util.TraversingItemVisitor;
import javax.jcr.version.OnParentVersionAction;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: FrozenNodeInitializer.java 9172 2006-09-27 15:21:21Z peterit $
 */

public class FrozenNodeInitializer extends TraversingItemVisitor {

  private Stack<NodeImpl> contextNodes;

  // private Node rootNode;

  // private SessionDataManager manager;

  public FrozenNodeInitializer(NodeImpl frozen) throws RepositoryException {

    contextNodes = new Stack<NodeImpl>();
    contextNodes.push(frozen);
    // this.rootNode = rootNode;
    // toLog(">>>>>>>> frozen " + frozen.getPath()+"
    // "+currentNode().getPath());
  }

  protected void toLog(String message) {
    System.err.println(message);
  }
  
  protected void entering(Property property, int level) throws RepositoryException {

    PropertyImpl frozenProperty = null;
    if (property.getName().equals("jcr:primaryType") && level == 1) {
      frozenProperty = currentNode().createChildPropertyInmemory("jcr:frozenPrimaryType", 
          PropertyType.NAME, new Value[] {property.getValue()}, 
          ((PropertyImpl) property).isMultiValued(), 
          false, false);
    } else if (property.getName().equals("jcr:uuid") && level == 1) {
      frozenProperty = currentNode().createChildPropertyInmemory("jcr:frozenUuid", 
          PropertyType.STRING, new Value[] {property.getValue()}, 
          ((PropertyImpl) property).isMultiValued(), 
          false, false);
    } else if (property.getName().equals("jcr:mixinTypes") && level == 1) {
      frozenProperty = currentNode().createChildPropertyInmemory("jcr:frozenMixinTypes", 
          PropertyType.NAME, property.getValues(), 
          ((PropertyImpl) property).isMultiValued(), 
          false, false);
    } else if (currentNode() == null) {
      // property.getParent().getDefinition().getOnParentVersion()
      // != OnParentVersionAction.COPY) {
      // special behaviour of parent - ignore this;
      // toLog(">>>>>>>>>>>>> frozenProperty special behaviour of parent - ignore this ");
      return;
    } else {
      PropertyImpl propertyImpl = (PropertyImpl) property;
      int action = property.getDefinition().getOnParentVersion();
      if (action == OnParentVersionAction.IGNORE)
        return;
      else if (action == OnParentVersionAction.ABORT)
        throw new VersionException("Property is aborted " + property.getPath());
      else if (action == OnParentVersionAction.COPY || action == OnParentVersionAction.VERSION
          || action == OnParentVersionAction.INITIALIZE || action == OnParentVersionAction.COMPUTE) {
        frozenProperty = currentNode().createChildPropertyInmemory(property.getName(), 
            property.getType(), propertyImpl.getValueArray(), 
            ((PropertyImpl) property).isMultiValued(), 
            false, false);
        // toLog(">>>>>>>>>>>>> frozenProperty " + frozenProperty.getPath());
        // COMPUTE - primary and mixin types
        // IITIALIZE - uuid and created
        // should be ok with just copy
      } else
        throw new RepositoryException("Unknown OnParentVersion value " + action);
    }
    // toLog(">>>>>>>> entering prop " + property.getPath() + " cur: "+frozenProperty.getPath());
  }

  protected void entering(Node node, int level) throws RepositoryException {
    //toLog(">>>>>>>> entering node " + node.getPath() + " level " + level);
    
    // this node is not taken in account
    if (level == 0) {
      //toLog(">>>>>>>> entering node (level == 0) " + node.getPath());
      return;
    }

    // ignored parent
    if (currentNode() == null) {
      //toLog(">>>>>>>> entering node (ignored parent) " + node.getPath());
      contextNodes.push(null);
      return;
    }

    int action = node.getDefinition().getOnParentVersion();
    //toLog(">>>>>>>> entering node " + node.getPath() + " action " + OnParentVersionAction.nameFromValue(action));
    
    NodeImpl frozenNode = null; // (NodeImpl) node;
    if (action == OnParentVersionAction.IGNORE) {
      //toLog(">>>>>>>> entering node (IGNORE) " + node.getPath());
      contextNodes.push(null);
      // return;
    } else if (action == OnParentVersionAction.ABORT) {
      //toLog(">>>>>>>> entering node (ABORT) " + node.getPath());
      throw new VersionException("Node is aborted " + node.getPath());
    } else if (action == OnParentVersionAction.COPY) {
      frozenNode = createChildNode(currentNode(), (NodeImpl) node);
      contextNodes.push(frozenNode);
      //toLog(">>>>>>>> entering node (COPY) " + node.getPath() + " frozenNode: " + frozenNode.getPath());
    } else if (action == OnParentVersionAction.VERSION) {
      if (node.isNodeType("mix:versionable")) {
        frozenNode = createChildNode(currentNode(), node.getName(), "nt:versionedChild");
        SessionImpl nodeSession = (SessionImpl) node.getSession();
        Value vh = nodeSession.getValueFactory().createValue(node.getVersionHistory());        
        frozenNode.createChildPropertyInmemory("jcr:childVersionHistory", 
            PropertyType.REFERENCE, new Value[] { vh }, false, false);        
        contextNodes.push(null);
        //toLog(">>>>>>>> entering reference (VERSION) " + node.getPath() + " frozenNode: " + frozenNode.getPath());
      } else { // behaviour of COPY
        frozenNode = createChildNode(currentNode(), (NodeImpl) node);
        contextNodes.push(frozenNode);
        //toLog(">>>>>>>> entering node (VERSION with behaviour of COPY) " + node.getPath() + " frozenNode: " + frozenNode.getPath());
      }

    } else if (action == OnParentVersionAction.INITIALIZE) {
      // 8.2.11.3 INITIALIZE
      // On checkin of N, a new node C will be created and placed in version
      // storage as a child of VN. This new C will be initialized just as it
      // would be if created normally in a workspace. No state information
      // of the current C in the workspace is preserved.
      //toLog(">>>>>>>> entering reference (INITIALIZE) " + node.getPath());
      // [PN] 10.04.06 Creatimg simply an new node with same name and same node type
      createChildNode(currentNode(), node.getName(), node.getPrimaryNodeType().getName());
      contextNodes.push(null);
      // return;
    } else if (action == OnParentVersionAction.COMPUTE) {
      // 8.2.11.4 COMPUTE
      // On checkin of N, a new node C will be created and placed in version
      // storage as a child of VN. This new C will be initialized by some
      // procedure defined for that type of child node.
      //toLog(">>>>>>>> entering reference (COMPUTE) " + node.getPath());
      // [PN] 10.04.06 Creatimg simply an new node with same name and same node type
      createChildNode(currentNode(), node.getName(), node.getPrimaryNodeType().getName());
      contextNodes.push(null);
      // return;
    } else {
      //toLog(">>>>>>>> entering reference (Unknown onParentVersion type) " + node.getPath());
      throw new RepositoryException("Unknown onParentVersion type " + action);
    }
  }

  private NodeImpl createChildNode(NodeImpl nodeParent, NodeImpl node) throws RepositoryException {

    //boolean isReferenceable = node.isNodeType("mix:referenceable");
    TransientNodeData nodeData = (TransientNodeData) node.getActualItemData();
    InternalQName nodeName = nodeData.getQName();
    InternalQName ptName = nodeData.getPrimaryTypeName();
    InternalQName[] mtNames = nodeData.getMixinTypeNames();
    //String uuid = isReferenceable ? node.getUUID() : null;

//    toLog("====== FrozenInit createChildNode() " + nodeParent.getSession().getWorkspace().getName()
//       + " destParent: " + nodeParent.getPath()
//       + " src:" + node.getPath() + " name:" + nodeName.getAsString());

    NodeImpl newNode = nodeParent.createChildNodeInmemory(nodeName, ptName, mtNames, 
        null, node.getIndex(), false, false, true);
    return newNode;
  }

  private NodeImpl createChildNode(NodeImpl nodeParent, String name, 
      String primaryTypeName) throws RepositoryException {

    SessionImpl nodeSession = (SessionImpl) nodeParent.getSession();
    InternalQName nodeName = nodeSession.getLocationFactory().parseJCRName(name).getInternalName();
    InternalQName ptName = nodeSession.getLocationFactory().parseJCRName(primaryTypeName).getInternalName(); 
    
//    toLog("====== FrozenInit createChildNode() NEW " + nodeParent.getSession().getWorkspace().getName()
//        + " destParent: " + nodeParent.getPath()
//        + " src:" + nodeParent.getPath() + " name:" + nodeName.getAsString());

    NodeImpl newNode = nodeParent.createChildNodeInmemory(nodeName, ptName, null, 
        null, -1, false, false, true);
    return newNode;
  }

  protected void leaving(Property property, int level) throws RepositoryException {
  }

  protected void leaving(Node node, int level) throws RepositoryException {
    contextNodes.pop();
  }

  private NodeImpl currentNode() {
    return contextNodes.peek();
  }

}