/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.dataflow;

/**
 * Created by The eXo Platform SARL        .
 * <br>
 * Newly added Node's data (used for mock inmemory repository as well). 
 * Besides NodeData's methods includes child items adders
 *
 * @author Gennady Azarenkov
 * @version $Id: TransientNodeData.java 9819 2006-10-26 17:44:01Z geaz $
 */

import java.util.ArrayList;
import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.dataflow.ItemDataVisitor;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.MutableNodeData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.TraverseableNodeData;
import org.exoplatform.services.jcr.util.UUIDGenerator;

public class TransientNodeData extends TransientItemData implements Comparable,
    MutableNodeData, TraverseableNodeData  {

  private List nodes;

  private List properties;

  private InternalQName primaryTypeName;

  private InternalQName[] mixinTypeNames;

  private int orderNum;

  private AccessControlList acl;
  
  private int orderDirection = 0;

  public TransientNodeData(InternalQPath path, String uuid, int version,
      InternalQName primaryTypeName, InternalQName[] mixinTypeNames,
      int orderNum, String parentUUID, AccessControlList acl) {
    super(path, uuid, version, parentUUID);
    this.primaryTypeName = primaryTypeName;
    this.mixinTypeNames = mixinTypeNames;
    this.orderNum = orderNum;    
    this.nodes = new ArrayList();
    this.properties = new ArrayList();
    this.acl = acl;
  }
  
  public TransientNodeData(InternalQPath path, String uuid, int version,
      InternalQName primaryTypeName, InternalQName[] mixinTypeNames,
      int orderNum, String parentUUID, int orderDirection, AccessControlList acl) {
    super(path, uuid, version, parentUUID);
    this.primaryTypeName = primaryTypeName;
    this.mixinTypeNames = mixinTypeNames;
    this.orderNum = orderNum;
    this.nodes = new ArrayList();
    this.properties = new ArrayList();
    this.orderDirection = orderDirection;
    this.acl = acl;
  }
  
  // --------------- ItemData ------------
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.ItemData#isNode()
   */
  public boolean isNode() {
    return true;
  }
  
  //---------------- NodeData -------------
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getOrderNumber()
   */
  public int getOrderNumber() {
    return orderNum;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getPrimaryTypeName()
   */
  public InternalQName getPrimaryTypeName() {
    return primaryTypeName;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getMixinTypeNames()
   */
  public InternalQName[] getMixinTypeNames() {
    return mixinTypeNames;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getACL()
   */
  public AccessControlList getACL() {
    return acl;
  }
  
  //---------------- TraverseableNodeData -------------
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getChildNodes()
   */
  public List getChildNodes() {
    return nodes;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getChildProperties()
   */
  public List getChildProperties() {
    return properties;
  }
 
  //---------------- MutableNodeData 

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setOrderNumber(int)
   */
  public void setOrderNumber(int orderNum) {
    this.orderNum = orderNum;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setMixinTypeNames(org.exoplatform.services.jcr.datamodel.InternalQName[])
   */
  public void setMixinTypeNames(InternalQName[] mixinTypeNames) {
    this.mixinTypeNames = mixinTypeNames;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#addNode(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public NodeData addNode(NodeData node) {
    nodes.add(node);
    return node;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#addProperty(org.exoplatform.services.jcr.datamodel.PropertyData)
   */
  public PropertyData addProperty(PropertyData property) {
    properties.add(property);
    return property;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#removeProperty(org.exoplatform.services.jcr.datamodel.PropertyData)
   */
  public void removeProperty(PropertyData property) {
    properties.remove(property);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#removeNode(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public void removeNode(NodeData node) {
    nodes.remove(node);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setUUID(java.lang.String)
   */
  public void setUUID(String uuid) {
    UUID = uuid;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setACL(org.exoplatform.services.jcr.access.AccessControlList)
   */
  public void setACL(AccessControlList acl) {
    this.acl = acl;
  }

  
  /**
   * Factory method
   * 
   * @param parent
   * @param name
   * @param primaryTypeName
   * @return
   */
  public static TransientNodeData createNodeData(NodeData parent,
      InternalQName name, InternalQName primaryTypeName) {
    TransientNodeData nodeData = null;
    InternalQPath path = InternalQPath.makeChildPath(parent.getQPath(), name);
    nodeData = new TransientNodeData(path,
        UUIDGenerator.generate(), -1, primaryTypeName, 
        new InternalQName[0], 0, parent.getUUID(), parent.getACL());
    return nodeData;
  }
    
  /**
   * Factory method
   * 
   * @param parent
   * @param name
   * @param primaryTypeName
   * @return
   */
  public static TransientNodeData createNodeData(NodeData parent,
      InternalQName name, InternalQName primaryTypeName, int index) {
    TransientNodeData nodeData = null;
    InternalQPath path = InternalQPath.makeChildPath(parent.getQPath(), name, index);
    nodeData = new TransientNodeData(path,
        UUIDGenerator.generate(), -1, primaryTypeName, 
        new InternalQName[0], 0, parent.getUUID(), parent.getACL());
    return nodeData;
  }
  
  /**
   * Factory method
   * 
   * @param parent
   * @param name
   * @param primaryTypeName
   * @return
   */
  public static TransientNodeData createNodeData(NodeData parent,
      InternalQName name, InternalQName primaryTypeName, String uuid) {
    TransientNodeData nodeData = null;
    InternalQPath path = InternalQPath.makeChildPath(parent.getQPath(), name);
    nodeData = new TransientNodeData(path,
        uuid, -1, primaryTypeName, 
        new InternalQName[0], 0, parent.getUUID(), parent.getACL());
    return nodeData;
  }

//  /**
//   * @return order status in the node session
//   * */
//  public boolean isOrdered() {
//    // For persisted node data - not ordered always 
//    return getOrderDirection() != 0;
//  }

  public int getOrderDirection() {
    return orderDirection;
  }  

  // ------------- Comparable  /////

  public int compareTo(Object obj) {
    return ((NodeData) obj).getOrderNumber() - orderNum;
  }
  
  public void accept(ItemDataVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }
  
}

