/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.dataflow.session;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.InvalidItemStateException;
import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.ItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.ItemDataFactory;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.persistent.BatcheableWorkspaceDataManager;
/**
 * Created by The eXo Platform SARL        .<br/>
 * proxy of local workspace storage. "local" means that backended workspace data manager 
 * is located on the same JVM as session layer. WorkspaceStorageDataManagerProxy can 
 * be pluggable in a case of other storage-session transport applied (for ex RMI)
 * this implementation is responsible for making copy of persisted (shared) data objects
 * for session data manager and pass it on top (to TransactionableDM) (and vice versa?) 
 *     
 * @author Gennady Azarenkov
 * @version $Id: $
 */

public class LocalWorkspaceStorageDataManagerProxy implements WorkspaceStorageDataManagerProxy {

  protected final BatcheableWorkspaceDataManager storageDataManager;
  
  
  public LocalWorkspaceStorageDataManagerProxy(BatcheableWorkspaceDataManager storageDataManager) {
    this.storageDataManager = storageDataManager;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#save(org.exoplatform.services.jcr.impl.dataflow.ItemDataChangesLog)
   */
  public void save(ItemDataChangesLog changes)
      throws InvalidItemStateException, UnsupportedOperationException,
      RepositoryException {
    
//    versionHistoryInitializer.saveVersionable(changes);

    List <ItemState> states = new ArrayList<ItemState>(changes.getSize());
    for(ItemState change: changes.getAllStates()) {
      states.add(new ItemState(copyItemData((TransientItemData)change.getData()), change.getState()));
    }
    storageDataManager.save(new ItemDataChangesLog(states, changes.getSession()));
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getItemData(org.exoplatform.services.jcr.datamodel.InternalQPath)
   */
  public ItemData getItemData(InternalQPath path) throws RepositoryException {
    return ItemDataFactory.makeItemData(storageDataManager.getItemData(path));
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getItemData(java.lang.String)
   */
  public ItemData getItemData(String uuid) throws RepositoryException {
    return ItemDataFactory.makeItemData(storageDataManager.getItemData(uuid));
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getChildNodesData(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public List<NodeData> getChildNodesData(NodeData parent)
      throws RepositoryException {
    return ItemDataFactory.makeCopyOfNodes(storageDataManager.getChildNodesData(parent));
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getChildPropertiesData(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public List<PropertyData> getChildPropertiesData(NodeData parent)
      throws RepositoryException {
    return ItemDataFactory.makeCopyOfProperties(storageDataManager.getChildPropertiesData(parent));
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getReferencesData(java.lang.String)
   */
  public List<PropertyData> getReferencesData(String uuid)
      throws RepositoryException {
    return ItemDataFactory.makeCopyOfProperties(storageDataManager.getReferencesData(uuid));
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getChildPropertiesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public int getChildPropertiesCount(NodeData parent)
      throws RepositoryException {
    return storageDataManager.getChildPropertiesCount(parent);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getChildNodesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public int getChildNodesCount(NodeData parent) throws RepositoryException {
    return storageDataManager.getChildNodesCount(parent);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getACL(org.exoplatform.services.jcr.datamodel.InternalQPath)
   */
  public AccessControlList getACL(InternalQPath path)
      throws RepositoryException {
    AccessControlList acl = storageDataManager.getACL(path);
    return new AccessControlList(acl.getOwner(), acl.getPermissions());
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#beginSessionBatch(org.exoplatform.services.jcr.impl.core.SessionImpl)
   */
  public synchronized void beginSessionBatch(SessionImpl session) {
    storageDataManager.beginSessionBatch(session);
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#endSessionBatch(org.exoplatform.services.jcr.impl.core.SessionImpl)
   */
  public synchronized void endSessionBatch(SessionImpl session) throws RepositoryException {
    storageDataManager.endSessionBatch(session);
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.session.WorkspaceStorageDataManagerProxy#getCurrentTime()
   */
  public Calendar getCurrentTime() {
    return storageDataManager.getCurrentTime();
  }

  
  private TransientItemData copyItemData(TransientItemData item) {
    // make a copy
    if (item.isNode()) {

      TransientNodeData node = (TransientNodeData)item;
      return new TransientNodeData(node.getQPath(), node.getUUID(), 
        node.getPersistedVersion(), node.getPrimaryTypeName(), node.getMixinTypeNames(),
        node.getOrderNumber(), node.getParentUUID(), node.getOrderDirection(), node.getACL());
    } else {
      
      TransientPropertyData prop = (TransientPropertyData)item;
      // make a copy
      TransientPropertyData newData = new TransientPropertyData(
          prop.getQPath(), prop.getUUID(), prop.getPersistedVersion(),
          prop.getType(), prop.getParentUUID(), prop.isMultiValued());
      newData.setValues(prop.getValues());
      return newData;
    }
  }

}
