/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.util;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.value.StringValue;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;
import org.exoplatform.services.jcr.impl.dataflow.DataManager;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;
import org.exoplatform.services.jcr.util.io.BLOBUtil;

/**
 * Created by The eXo Platform SARL 15.05.2006
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: PropertyDataReader.java 9660 2006-10-21 16:14:41Z peterit $
 */
public class PropertyDataReader extends ItemDataReader {

  private HashMap<String, PropertyInfo> propeties = new HashMap<String, PropertyInfo>();
  
  private class PropertyInfo {
    private InternalQName propertyName = null;
    private boolean multiValued = false;
    private List<Value> mValue = null;
    private Value value = null;
    private int type = PropertyType.STRING;

    PropertyInfo(InternalQName propertyName, int type) {
      this.propertyName = propertyName;
      this.type = type;
    }
    
    public InternalQName getPropertyName() {
      return propertyName;
    }

    public boolean isMultiValued() {
      return multiValued;
    }
    
    public void setMultiValued(boolean multiValued) {
      this.multiValued = multiValued;
    }
    
    public List<Value> getValues() throws ValueFormatException, PathNotFoundException {
      if (multiValued) {
        if (mValue != null) {
          return mValue;
        }
      } else if (value != null) {
        throw new ValueFormatException("Property " + parent.getQPath().getAsString()  
            + propertyName.getAsString() + " is multi-valued");
      }
      throw new PathNotFoundException("Property " + parent.getQPath().getAsString() 
          + propertyName.getAsString() + " not found (multi-valued)");
    }
    
    public void setValues(List<Value> mValue) {
      this.mValue = mValue;
      this.multiValued = true;
    }

    public Value getValue() throws ValueFormatException, PathNotFoundException {
      if (!multiValued) {
        if (value != null) {
          return value;
        }
      } else if (mValue != null) {
        throw new ValueFormatException("Property " + parent.getQPath().getAsString() 
            + propertyName.getAsString() + " is single-valued");
      }
      throw new PathNotFoundException("Property " + parent.getQPath().getAsString() 
          + propertyName.getAsString() + " not found (single-valued)");
    }
    
    public void setValue(Value value) {
      this.value = value;
      this.multiValued = false;
    }

    public int getType() {
      return type;
    }
  }
  
  public PropertyDataReader(NodeData parent, DataManager dataManager, ValueFactoryImpl valueFactory) {
    super(parent, dataManager, valueFactory);
  }
    
  public PropertyDataReader forProperty(InternalQName name, int type) {
    propeties.put(name.getAsString(), new PropertyInfo(name, type));
    return this;
  }
  
  public List<Value> getPropertyValues(InternalQName name) throws ValueFormatException, PathNotFoundException {
    return propeties.get(name.getAsString()).getValues();
  }
  
  public Value getPropertyValue(InternalQName name) throws ValueFormatException, PathNotFoundException {
    return propeties.get(name.getAsString()).getValue();
  }
  
  public void read() throws RepositoryException {
    List<PropertyData> ndProps = dataManager.getChildPropertiesData(parent);
    for (PropertyData prop : ndProps) {
      if (propeties.containsKey(prop.getQPath().getName().getAsString())) {
        PropertyInfo propInfo = propeties.get(prop.getQPath().getName().getAsString());
        List<ValueData> valueDataList = prop.getValues();
        if (prop.isMultiValued()) {
          List<Value> values = new ArrayList<Value>();
          for (ValueData vd : valueDataList) {
            values.add(makeValue(vd, propInfo.getType()));
          }
          propInfo.setValues(values);
        } else {
          if (valueDataList.size()>0)
            propInfo.setValue(makeValue(valueDataList.get(0), propInfo.getType()));
        }
      }
    }
  }

  private Value makeValue(ValueData valueData, int type) throws RepositoryException {
    if (valueFactory != null) {
      try {
        TransientValueData tvd = new TransientValueData(
            valueData.getAsByteArray(), 
            valueData.getOrderNumber()); 
        return valueFactory.loadValue(tvd, type);
      } catch (IOException e) {
        throw new RepositoryException("Can't make value from value data: " + e.getMessage(), e);
      }
    }
    try {
      // TODO Same logic as in protected constructor StringValue(ValueData) 
      return new StringValue(
          new String(BLOBUtil.readValue(valueData), Constants.DEFAULT_ENCODING));
    } catch(UnsupportedEncodingException e) {
      try {
        return new StringValue(new String(valueData.getAsByteArray()));
      } catch (IOException e1) {
        throw new RepositoryException("Can't make value from value data: " + e1.getMessage(), e1);
      }
    } catch(IOException e) {
      throw new RepositoryException("Can't make value from value data: " + e.getMessage(), e);
    }
  }
  
}
