/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.api.nodetypes;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;

/**
 * Created by The eXo Platform SARL        .
 */

public class TestValueConstraints extends JcrAPIBaseTest {
  
  private Node testValueConstraintsNode = null;
  private NodeTypeManagerImpl ntManager = null;
  private Node refNodeNtUnstructured = null;
  private Node refNodeNtBase = null;
  private String nodeTypeName = "jcr:testValueConstraints";
  private static String LOCAL_BIG_FILE = "src/test/resources/BigFile.zip";
  private static String LOCAL_SMALL_FILE = "src/test/resources/SmallFile.zip";
  private static String LOCAL_NORMAL_FILE = "src/test/resources/NormalFile.zip";
	
  public void setUp() throws Exception {
	super.setUp();
	byte[] xmlData = readXmlContent("/org/exoplatform/services/jcr/api/nodetypes/nodetypes-api-test.xml");
    ByteArrayInputStream xmlInput = new ByteArrayInputStream(xmlData);
    ntManager = (NodeTypeManagerImpl) session.getWorkspace().getNodeTypeManager();
    ntManager.registerNodeTypes(xmlInput, 0);
    assertNotNull(ntManager.getNodeType(nodeTypeName));
    Node ntRoot = (Node) repository.getSystemSession().getItem(NodeTypeManagerImpl.NODETYPES_ROOT);
    assertTrue(ntRoot.hasNode(nodeTypeName));
    testValueConstraintsNode = root.addNode("testValueConstraints", nodeTypeName);
    testValueConstraintsNode.addMixin("mix:referenceable");
    refNodeNtUnstructured = root.addNode("testref", "nt:unstructured");
    refNodeNtUnstructured.addMixin("mix:referenceable");
    refNodeNtBase = root.addNode("testref", "nt:base");
    refNodeNtBase.addMixin("mix:referenceable");
    session.save();
  }
  
  public void testSTRING1Property() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testSTRING1", "abc");
	root.save();
	try {
	  testProperty.setValue("abcd");
	  root.save();
	  fail("setValue(STRING value) must throw a ConstraintViolationException ");
	} catch (Exception e) {
	 //success   
	}
  }

  public void testSTRING2Property() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testSTRING2", "abc");
	testProperty.setValue("abcd");
	testProperty.setValue("");
	testProperty.setValue("1234");
	testProperty.setValue("true");
	root.save();
  }
    
  public void testPATHProperty() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testPATH", valueFactory.createValue("/abc", PropertyType.PATH));
	//testProperty.setValue(valueFactory.createValue("/exojcrtest/ghi", PropertyType.PATH));
	testProperty.setValue(valueFactory.createValue("../exojcrtest:def/ghi", PropertyType.PATH));
	root.save();
	try {
	  testProperty.setValue(valueFactory.createValue("/abcd", PropertyType.PATH));
	  root.save();
	  fail("setValue(PATH value) must throw a ConstraintViolationException ");
	} catch (Exception e) {
	 //success   
	}
	try {
	  testProperty.setValue(valueFactory.createValue("../abc", PropertyType.PATH));
          root.save();
	  fail("setValue(PATH value) must throw a ConstraintViolationException ");
	} catch (Exception e) {
	  // success
	}			
  }

  public void testNAMEProperty() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testNAME", valueFactory.createValue("abc:"));
	testProperty.setValue(valueFactory.createValue("abc:def"));
	root.save();
	try {
	  testProperty.setValue(valueFactory.createValue("/abcd"));
	  root.save();
	  fail("setValue(NAME value) must throw a ConstraintViolationException ");
	} catch (Exception e) {
	 //success   
	}
	try {
	  testProperty.setValue(valueFactory.createValue("abc:de"));	
      root.save();
	  fail("setValue(NAME value) must throw a ConstraintViolationException ");
	} catch (Exception e) {
	  // success
	}			
  }  
  
  public void testREFERENCEProperty() throws Exception {
	  
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testREFERENCE", refNodeNtUnstructured);
	root.save();
	try {
	  testProperty.setValue(refNodeNtBase);
	  root.save();
	  fail("setValue(REFERENCE value) must throw a ConstraintViolationException ");
	} catch (Exception e) {
	 //success   
	}
  }

  public void testBINARYINCLUSIVEProperty() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testBINARYINCLUSIVE", new FileInputStream(LOCAL_SMALL_FILE));
    testProperty.setValue(new FileInputStream(LOCAL_BIG_FILE));
	root.save();
  }
  
  public void testBINARYEXCLUSIVEProperty() throws Exception {
	
	try {  
	  Property testProperty = testValueConstraintsNode.
	      setProperty("jcr:testBINARYEXCLUSIVE", new FileInputStream(LOCAL_SMALL_FILE));
	  testProperty.setValue(new FileInputStream(LOCAL_BIG_FILE));
	  root.save();
	  fail("setValue(BINARY value) must throw a ConstraintViolationException ");
	}catch (Exception e) {
	 //succes 	
	}
	//trying to use another file to get constraint viol.  
	try {  
	  Property testProperty = testValueConstraintsNode.
	      setProperty("jcr:testBINARYEXCLUSIVE", new FileInputStream(LOCAL_NORMAL_FILE));
	  root.save();
	  fail("setValue(BINARY value) must throw a ConstraintViolationException ");
    }catch (Exception e) {
	 //succes 	
	}
  }
  
  public void testDATEINCLUSIVEProperty() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testDATEINCLUSIVE", valueFactory.createValue("1111-11-11T11:11:11.111Z",PropertyType.DATE));
	testProperty.setValue(valueFactory.createValue("1222-11-11T11:11:11.111Z", PropertyType.DATE));
	root.save();
  }
  
  public void testDATEEXCLUSIVEProperty() throws Exception {
	
	try {  
	  Property testProperty = testValueConstraintsNode.
		  setProperty("jcr:testDATEEXCLUSIVE", valueFactory.createValue("1111-11-11T11:11:11.111Z",PropertyType.DATE));
	  testProperty.setValue(valueFactory.createValue("1222-11-11T11:11:11.111Z", PropertyType.DATE));
	  root.save();
	  fail("setValue(DATE value) must throw a ConstraintViolationException ");
	}catch (Exception e) {
	 //succes 	
	}
	//trying to use another date to get constr.viol.ex.
	try {  
	  Property testProperty = testValueConstraintsNode.
	  	  setProperty("jcr:testDATEEXCLUSIVE", valueFactory.createValue("1155-11-11T11:11:11.111Z",PropertyType.DATE));
	  root.save();
	  fail("setValue(DATE value) must throw a ConstraintViolationException ");
	}catch (Exception e) {
	 //succes 	
	}
  }
  
  public void testLONGINCLUSIVEProperty() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testLONGINCLUSIVE", 100);
	testProperty.setValue(200);
	root.save();
  }

  public void testLONGEXCLUSIVEProperty() throws Exception {
	  
	try {  
	  Property testProperty = testValueConstraintsNode.
		  setProperty("jcr:testLONGEXCLUSIVE", 100);
	  testProperty.setValue(200);
	  root.save();
	  fail("setValue(LONG value) must throw a ConstraintViolationException ");
	}catch (Exception e) {
	 //succes 	
	}
	//trying to use another Long to get ex.
	try {  
	  Property testProperty = testValueConstraintsNode.
	  	  setProperty("jcr:testLONGEXCLUSIVE", 150);
	  root.save();
	  fail("setValue(LONG value) must throw a ConstraintViolationException ");
	}catch (Exception e) {
	 //succes 	
	}
  }
  
  public void testDOUBLEINCLUSIVEProperty() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testDOUBLEINCLUSIVE", 100);
	testProperty.setValue(200);
	root.save();
  }

  public void testDOUBLEEXCLUSIVEProperty() throws Exception {
	  
	try {  
	  Property testProperty = testValueConstraintsNode.
		  setProperty("jcr:testDOUBLEEXCLUSIVE", 100);
	  testProperty.setValue(200);
	  root.save();
	  fail("setValue(DOUBLE value) must throw a ConstraintViolationException ");
	}catch (Exception e) {
	 //succes 	
	}
	//trying to use another Double to get ex.
	try {  
	  Property testProperty = testValueConstraintsNode.
	  	  setProperty("jcr:testDOUBLEEXCLUSIVE", 150);
	  root.save();
	  fail("setValue(DOUBLE value) must throw a ConstraintViolationException ");
	}catch (Exception e) {
	 //succes 	
	}
  }
  
  public void testBOOLEANProperty() throws Exception {
	
	Property testProperty = testValueConstraintsNode.
		setProperty("jcr:testBOOLEAN", true);
	root.save();
	try {
	  testProperty.setValue(false);
	  root.save();
	} catch (Exception e) {
	  fail("setValue(BOOLEAN value) here should be no Exception ");   
	}
  }

  private byte[] readXmlContent(String fileName) {
    try {
      InputStream is = TestValueConstraints.class.getResourceAsStream(fileName);
      ByteArrayOutputStream output = new ByteArrayOutputStream();
	      int r = is.available();
      byte[] bs = new byte[r];
      while (r > 0) {
        r = is.read(bs);
        if (r > 0) {
          output.write(bs, 0, r);
        }
        r = is.available();
      }
      is.close();
      return output.toByteArray();
    } catch (Exception e) {
      log.error("Error read file '" + fileName + "' with NodeTypes. Error:" + e);
      return null;
    }
  }
  
  protected void tearDown() throws Exception {
	if (session.getRootNode().hasNode("testValueConstraints")) {
	  session.getRootNode().getNode("testValueConstraints").remove();
	}
	if (session.getRootNode().hasNode("testref")) {
		  session.getRootNode().getNode("testref").remove();
		}
	super.tearDown();
  }
  
}
