/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.api.writing;


import javax.jcr.InvalidItemStateException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.nodetype.ConstraintViolationException;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.impl.core.SessionImpl;


/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TestItem.java 7405 2006-07-28 16:00:07Z peterit $
 */
public class TestItem extends JcrAPIBaseTest{


  public void testSave() throws RepositoryException {
    Node root = session.getRootNode();
    try {
        root.addNode("childNode", "nt:folder").addNode("childNode2", "nt:propertyDefinition");
    	root.save();
    	fail("exception should have been thrown");
    } catch (ConstraintViolationException e) {
    }

    //assertEquals("/childNode/childNode2", root.getNode("childNode/childNode2").getPath());
    session.refresh(false);


    session = (SessionImpl)repository.login(credentials, WORKSPACE);
    root = session.getRootNode();

    try {
      root.getNode("childNode/childNode2");
      fail("exception should have been thrown");
    } catch (PathNotFoundException e) {
    }

    Node node = root.addNode("testSave1", "exo:mockNodeType");
    node.setProperty("jcr:nodeTypeName", "test");
    root.save();

    node = root.addNode("testSave2", "nt:unstructured");
    node.addNode("node2BRem", "nt:unstructured");
    node.setProperty("existingProp", "existingValue");
    node.setProperty("existingProp2", "existingValue2");
    root.save();
    node.setProperty("prop", "propValue");
    node.setProperty("existingProp", "existingValueBis");
    node.getProperty("existingProp2").remove();
    node.getNode("node2BRem").remove();
    node.addNode("addedNode", "nt:unstructured");
//System.out.println(">>>>>>>>>>>>"+node.getProperty("prop"));
    node.save();
//System.out.println(">>>>>>>>>>>>"+node.getProperty("prop"));

//System.out.println(">>>>>>>>>>>>"+((Node)session.getItem("/childNode")).getProperty("prop"));

    session = (SessionImpl)repository.login(credentials, WORKSPACE);
    node = session.getRootNode().getNode("testSave2");
    root = session.getRootNode();
    try {
      node.getProperty("prop");
    } catch (PathNotFoundException e) {
      //e.printStackTrace();
      fail("exception should not be thrown");
    }

    assertEquals("existingValueBis", node.getProperty("existingProp").getString());
    try {
      node.getProperty("existingProp2");
      fail("exception should have been thrown");
    } catch (PathNotFoundException e) {
    }
    try {
      node.getNode("node2BRem");
      fail("exception should have been thrown");
    } catch (PathNotFoundException e) {
    }

//    try {
//      node.getNode("addedNode");
//      fail("exception should have been thrown");
//    } catch (PathNotFoundException e) {
//    }

    root.getNode("testSave1").remove();
    root.getNode("testSave2").remove();
    session.save();
  }

  public void testSaveWithNonPersistedParent() throws RepositoryException {
    Node root = session.getRootNode();
    Node n = root.addNode("childNode", "nt:folder").addNode("childNode2", "nt:file").addNode("jcr:content", "nt:unstructured");
    try {
      n.save();
      fail("exception should have been thrown");
    } catch (RepositoryException e) {
      session.refresh(false);
    }
//    root.getNode("childNode").remove();
//    session.save();
  }


  public void testState() throws RepositoryException {
    Node root = session.getRootNode();
    assertFalse(root.isNew());
    assertFalse(root.isModified());
    Node node = root.addNode("testnode", "nt:unstructured");
    assertTrue(node.isNew());
    assertFalse(node.isModified());

    Property prop = node.setProperty("testProp", "val");
    assertTrue(prop.isNew());
    assertFalse(prop.isModified());
    root.save();

    prop = node.setProperty("testProp", "val1");
    assertFalse(prop.isNew());
    assertTrue(prop.isModified());
    node.save();

    assertFalse(node.isNew());
    assertFalse(node.isModified());
    assertFalse(node.isNew());
    assertFalse(node.isModified());
  }


  public void testInvalid() throws RepositoryException {
    Node root = session.getRootNode();
  	Node node = root.addNode("testInvalidNode");
  	root.save();
 	
    Session session1 = (SessionImpl)repository.login(credentials, WORKSPACE);
    session1.getItem("/testInvalidNode").remove();
    session1.save();
    
    // stale node
    Node testNode = root.getNode("testInvalidNode");
    testNode.setProperty("testProp", "invalid");
    try {
      root.save();
      fail("exception should have been thrown");
    } catch (InvalidItemStateException e) {
    }  
  }

  public void testRefresh() throws RepositoryException {
    Node root = session.getRootNode();
  	Node node = root.addNode("testRefreshNode");
  	root.setProperty("testRefreshProp", "test");
  	root.refresh(true);
  	
  	// exists after refresh(true)
  	root.getNode("testRefreshNode");
  	root.getProperty("testRefreshProp");
  	
  	root.refresh(false);

    try {
      	root.getNode("testRefreshNode");
        fail("exception should have been thrown");
      } catch (PathNotFoundException e) {
      }

    try {
      	root.getProperty("testRefreshProp");
        fail("exception should have been thrown");
      } catch (PathNotFoundException e) {
      }

  }

}
