/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.ext.audit;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import javax.jcr.Item;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.jcr.dataflow.ItemState;
import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.ItemImpl;
import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionDataManager;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;
import org.exoplatform.services.jcr.impl.dataflow.session.SessionChangesLog;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL        .
 * @author Gennady Azarenkov
 * @version $Id: $
 */

public class AuditServiceImpl implements AuditService {
 
  private static Log log = ExoLogger.getLogger("jcr.AuditService");
  private final ManageableRepository repository;
  private final String workspaceName;
  
  public AuditServiceImpl(InitParams params, RepositoryService repService) 
  throws RepositoryException, RepositoryConfigurationException {
    ValueParam valParam = null;
    if(params != null)
      valParam = params.getValueParam("repository");
    if(valParam != null)
      repository = repService.getRepository(valParam.getValue());
    else
      repository = repService.getRepository();
    workspaceName = repository.getConfiguration().getSystemWorkspaceName();
  }

  public void addRecord(Item item, int eventType) throws RepositoryException {
      
    AuditSession auditSession = new AuditSession(item);
    NodeImpl storage = auditSession.getAuditStorage();
    SessionDataManager dm = auditSession.getDataManager();
    SessionImpl session = (SessionImpl)item.getSession();
    
    LocationFactory locationFactory = session.getLocationFactory();
    // add /jcr:system/exo:auditStorage/itemID/recordId (exo:auditRecord)
    
    // here should be added to SessionDataManager:
    // nodeData: /exo:audit/itemUUID/<get lastName + 1> 
    // its primaryType exo:auditRecord 
    // exo:user = session.getUserId()
    // exo:created = current date
    // exo:event = eventType
    
    NodeData aHistory = auditSession.getAuditHistory(item);

    // make path to the AUDITHISTORY_LASTNAME property
    InternalQPath path = InternalQPath.makeChildPath(aHistory.getQPath(),
        AuditService.EXO_AUDITHISTORY_LASTNAME);
    // searching last name of node
    PropertyData pData = (PropertyData) dm.getItemData(path);
//    String auditRecordName = "";
//    try {
//      auditRecordName = (new Integer(new String(pData.getValues().get(0).getAsByteArray(),
//          Constants.DEFAULT_ENCODING)) + 1);
//    } catch (Exception e) {
//      throw new RepositoryException("Error on add audit record");
//    }
//    //Update lastName
//    TransientPropertyData pLastName = TransientPropertyData.createPropertyData(aHistory,
//        AuditService.EXO_AUDITHISTORY_LASTNAME, PropertyType.STRING, false, new TransientValueData(
//            String.valueOf(auditRecordName)));
//    //exo:auditRecord
//    //new JCRPath().p
//    TransientNodeData arNode = TransienztNodeData.createNodeData(aHistory,
//        locationFactory.parseJCRName()), AuditService.EXO_AUDITHISTORY);
//
//    session.getTransientNodesManager().update(new ItemState(pLastName, ItemState.UPDATED, 
//        true, ((ItemImpl) item).getInternalPath()), true);
    
    
    //ahNode.setProperty(locationFactory.createJCRName(AuditService.EXO_AUDITHISTORY_LASTNAME).getAsString(),auditRecordName);
    //Node aRecord = ahNode.addNode(String.valueOf(auditRecordName),locationFactory.createJCRName(AuditService.EXO_AUDITRECORD).getAsString());
    session.save();
  }

  public void createHistory(Item item) throws RepositoryException {
    AuditSession auditSession = new AuditSession(item);
    NodeImpl storage = auditSession.getAuditStorage();
    // add /jcr:system/exo:auditStorage/itemID (exo:auditHistory)

    // here should be added to SessionDataManager:
    // nodeData: /exo:audit/itemUUID 
    // its primaryType exo:auditHistory
    // exo:targetNode (ref to item)
    // exo:lastName = "0"
    // in itemData/auditHistory - pointer to history (UUID)
    
    
    SessionImpl session = (SessionImpl)item.getSession();
    

    
     
    InternalQName aiName = new InternalQName(null, ((ItemImpl)item).getData().getUUID());
    //exo:auditHistory
    TransientNodeData ahNode = TransientNodeData.createNodeData((NodeData) storage.getData(),
        aiName, AuditService.EXO_AUDITHISTORY);
    
    // jcr:primaryType
    TransientPropertyData aPrType = TransientPropertyData
        .createPropertyData(ahNode, Constants.JCR_PRIMARYTYPE, PropertyType.NAME, false);
    aPrType.setValue(new TransientValueData(ahNode.getPrimaryTypeName()));

    // jcr:uuid
    TransientPropertyData ahUuid = TransientPropertyData.createPropertyData(
        ahNode, Constants.JCR_UUID, PropertyType.STRING, false);
    ahUuid.setValue(new TransientValueData(ahNode.getUUID()));

    // jcr:mixinTypes
    TransientPropertyData ahMixinTypes = TransientPropertyData
        .createPropertyData(ahNode, Constants.JCR_MIXINTYPES, PropertyType.NAME, false);
    ahMixinTypes.setValue(new TransientValueData(Constants.MIX_REFERENCEABLE));

    //exo:targetNode
    TransientPropertyData ahTargetNode = TransientPropertyData.createPropertyData(
        ahNode, AuditService.EXO_AUDITHISTORY_TARGETNODE, PropertyType.REFERENCE, false);
    ahTargetNode.setValue(new TransientValueData(((ItemImpl)item).getData().getUUID()));
    
    //exo:lastName
    TransientPropertyData ahLastName = TransientPropertyData.createPropertyData(
        ahNode, AuditService.EXO_AUDITHISTORY_LASTNAME, PropertyType.STRING, false);
    ahLastName.setValue(new TransientValueData("0"));
    
    //node exo:auditHistory
    TransientPropertyData pAuditHistory = TransientPropertyData
    .createPropertyData((NodeData) ((ItemImpl)item).getData(), AuditService.EXO_AUDITHISTORY,PropertyType.STRING, false);
    pAuditHistory.setValue(new TransientValueData(new Uuid(ahNode.getUUID())));

    
    session.getTransientNodesManager().update(new ItemState(ahNode, ItemState.ADDED, 
        true, ((ItemImpl) item).getInternalPath()), true);

    session.getTransientNodesManager().update(new ItemState(aPrType, ItemState.ADDED, 
        true, ((ItemImpl) item).getInternalPath()), true);
        
    session.getTransientNodesManager().update(new ItemState(ahUuid, ItemState.ADDED, 
        true, ((ItemImpl) item).getInternalPath()), true);
    
    session.getTransientNodesManager().update(new ItemState(ahMixinTypes, ItemState.ADDED, 
        true, ((ItemImpl) item).getInternalPath()), true);
    
//    session.getTransientNodesManager().update(new ItemState(ahCreated, ItemState.ADDED, 
//        true, ((ItemImpl) item).getInternalPath()), true);
    
    session.getTransientNodesManager().update(new ItemState(ahTargetNode, ItemState.ADDED, 
        true, ((ItemImpl) item).getInternalPath()), true);
    
    session.getTransientNodesManager().update(new ItemState(ahLastName, ItemState.ADDED, 
        true, ((ItemImpl) item).getInternalPath()), true);
    
    session.getTransientNodesManager().update(new ItemState(pAuditHistory, ItemState.ADDED, 
        true, ((ItemImpl) item).getInternalPath()), true);

  }

  public void removeHistory(Item item) throws RepositoryException {
    AuditSession session = new AuditSession(item);
    NodeImpl storage = session.getAuditStorage();
    // remove /jcr:system/exo:auditStorage/itemID
    // (delete in SessionDataManager)

    session.logout();
  }

  private class AuditSession {
    
    private SessionImpl session;
    private SessionDataManager dm ;
    
//    private AuditSession() throws RepositoryException {
//      session = (SessionImpl)repository.getSystemSession(workspaceName);
//    }
    
    private AuditSession(Item item) throws RepositoryException {
      session = (SessionImpl)item.getSession();
      ExtendedNode node;
      if(item.isNode())
        node = (ExtendedNode) item;
      else
        node = (ExtendedNode) item.getParent();
      if(!node.isNodeType(AuditService.EXO_AUDITABLE))
        throw new RepositoryException("Node is not exo:auditable "+node.getPath());
      
      dm = session.getTransientNodesManager();
    }
    
    private NodeImpl getAuditStorage() throws RepositoryException {
      NodeImpl storage;
      try {
        storage = (NodeImpl)session.getNodeByUUID(AUDIT_STORAGE_ID);
      } catch (ItemNotFoundException e) {
        SessionChangesLog changesLog = new SessionChangesLog(session.getId());
  
        // here should be added to TransactionalDataManager (i.e. saved immediatelly!):
        // nodeData: /exo:audit with UUID = AUDIT_STORAGE_ID
        // its primaryType exo:auditStorage
        
        TransientNodeData exoAuditNode = TransientNodeData.createNodeData(
            (NodeData) ((NodeImpl) session.getRootNode()).getData(), AuditService.EXO_AUDIT, AuditService.EXO_AUDITSTORAGE,
                AuditService.AUDIT_STORAGE_ID);
        
        // jcr:primaryType
        TransientPropertyData exoAuditPrType = TransientPropertyData
            .createPropertyData(exoAuditNode, Constants.JCR_PRIMARYTYPE, PropertyType.NAME, false);
        exoAuditPrType.setValue(new TransientValueData(exoAuditNode.getPrimaryTypeName()));

        // jcr:uuid
        TransientPropertyData exoAuditUuid = TransientPropertyData.createPropertyData(
            exoAuditNode, Constants.JCR_UUID, PropertyType.STRING, false);
        exoAuditUuid.setValue(new TransientValueData(exoAuditNode.getUUID()));

        // jcr:mixinTypes
        TransientPropertyData exoAuditMixinTypes = TransientPropertyData
            .createPropertyData(exoAuditNode, Constants.JCR_MIXINTYPES, PropertyType.NAME, true);
        exoAuditMixinTypes.setValue(new TransientValueData(Constants.MIX_REFERENCEABLE));

//        // jcr:created
//        TransientPropertyData exoAuditCreated = TransientPropertyData.createPropertyData(
//            exoAuditNode, Constants.JCR_CREATED, PropertyType.DATE, false);
//        exoAuditCreated.setValue(new TransientValueData(
//            session.getTransientNodesManager().getTransactManager().getStorageDataManager().getCurrentTime()));
        
        
        changesLog.add(ItemState.createAddedState(exoAuditNode));
        changesLog.add(ItemState.createAddedState(exoAuditPrType));
        changesLog.add(ItemState.createAddedState(exoAuditUuid));
        changesLog.add(ItemState.createAddedState(exoAuditMixinTypes));
//        changesLog.add(ItemState.createAddedState(exoAuditCreated));
        
        session.getTransientNodesManager().getTransactManager().save(changesLog);
        storage = (NodeImpl)session.getNodeByUUID(AUDIT_STORAGE_ID);
      }
      return storage;
    }
    
    private SessionDataManager getDataManager() {
      return dm;
    }
    private NodeData getAuditHistory(Item item) throws RepositoryException{
      InternalQPath path = null;
      //create path to holding node
      if (item instanceof PropertyImpl) {
        path = ((PropertyImpl)item).getInternalPath().makeParentPath();
      }else if ( item instanceof NodeImpl){
        path = ((NodeImpl)item).getInternalPath();
      }
      //make path to the audithistory property 
      path = InternalQPath.makeChildPath(path,AuditService.EXO_AUDITHISTORY);
      //searching uuid of corresponding EXO_AUDITHISTORY node
      PropertyData pData = (PropertyData) dm.getItemData(path);
      String ahUuid;
      try {
        ahUuid =  new String(pData.getValues().get(0).getAsByteArray(),Constants.DEFAULT_ENCODING);
      } catch (Exception e) {
        throw new RepositoryException("Error getAuditHistory converting to string");
      }
      return (NodeData) dm.getItemData(ahUuid);
    }
    private void logout() {
      session.logout();
    }
  }

}
