/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.util.io;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 12.09.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: LazyByteArrayInputStream.java 9852 2006-10-28 11:34:29Z peterit $
 */
public class LazyByteArrayInputStream extends CloneableInputStream implements BytesArray {

  protected final byte[] buff;
  protected ByteArrayInputStream dataStream; 
  
  public LazyByteArrayInputStream(byte[] buff) {
    this.buff = buff;
  }
  
  @Override
  public LazyByteArrayInputStream clone() {
    return new LazyByteArrayInputStream(buff);
  }

  @Override
  public void close() throws IOException {
    if (dataStream != null)
      super.close();
    // else nothing to close
  }
  
  @Override
  protected InputStream stream() throws IllegalStateException, IOException { 
    if (dataStream == null) {
      // TODO Copy buff (use System.arraycopy())...
      // byte[] newBuff = new byte[buff.length];
      // System.arraycopy(buff, 0, newBuff, 0, buff.length);
      // buff = newBuff;
      // In that way we will use personal copy of data, copied in memory by demand (first read) 
      
      dataStream = new ByteArrayInputStream(buff);
    }
    return dataStream;
  }

  public byte[] getBytes() {
    return buff;
  }
  
  /**
   * Return size of this BytesArray 
   */
  public long size() {
    return buff.length;
  }
}
