/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.dataflow.persistent;

import java.io.File;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.HashMap;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.dataflow.ItemDataChangesLog;
import org.exoplatform.services.jcr.dataflow.persistent.ItemsPersistenceListener;
import org.exoplatform.services.jcr.impl.dataflow.replication.FixupStream;
import org.exoplatform.services.jcr.impl.dataflow.replication.Packet;
import org.exoplatform.services.jcr.impl.dataflow.replication.PendingChangesLog;
import org.exoplatform.services.jcr.util.UUIDGenerator;
import org.exoplatform.services.log.ExoLogger;
import org.jgroups.Address;
import org.jgroups.Channel;
import org.jgroups.ChannelException;
import org.jgroups.JChannel;
import org.jgroups.MembershipListener;
import org.jgroups.Message;
import org.jgroups.View;
import org.jgroups.blocks.GroupRequest;
import org.jgroups.blocks.MessageDispatcher;
import org.jgroups.blocks.RequestHandler;
import org.jgroups.util.Util;

/**
 * Created by The eXo Platform SARL .<br/> responsible for data replication
 * (both storage and cache) in a case of cluster environment it is optional
 * component, its presense is dependent on whether cluster environment is
 * configured
 * 
 * @author Gennady Azarenkov
 * @version $Id: $
 */

public class WorkspaceDataReplicator implements ItemsPersistenceListener,
    MembershipListener, RequestHandler {

  protected static Log log = ExoLogger.getLogger("jcr.WorkspaceDataReplicator");
  private final CacheableWorkspaceDataManager dataManager;
  private final String                        systemId;
  private Channel                             channel;
//  private String                              channelName;
  private MessageDispatcher                   disp;
//  private String                              propsTCP_NIO = "";
//  private String                              props        = "";
  private HashMap<String, PendingChangesLog>  mapPendingChangesLog;
  private Vector<Address>                     members;

  public WorkspaceDataReplicator(CacheableWorkspaceDataManager dataManager,
      WorkspaceEntry wsConfig, RepositoryEntry rConfig) {
    this.dataManager = dataManager;
    this.systemId = UUIDGenerator.generate();
    dataManager.addItemPersistenceListener(this);
    String channelName = wsConfig.getUniqueName();
    log.info(channelName);
    try {
      String localAdaress = getLocalIP(Util.getFirstNonLoopbackAddress());
      String propsTCP_NIO = rConfig.getReplication().getChannelConfig();
      String props = propsTCP_NIO.replaceAll("/LocalAddress/", localAdaress);
      channel = new JChannel(props);
      disp = new MessageDispatcher(channel, null, this, this);
      log.info("Channel name is '" + channelName +"'");
      System.out.println("Channel name is '" + channelName +"'");
      channel.connect(channelName);
      
    } catch (ChannelException e) {
      e.printStackTrace();
    } catch (SocketException e) {
      e.printStackTrace();
    }
    
    mapPendingChangesLog = new HashMap<String, PendingChangesLog>();
  }

  public void onSaveItems(ItemDataChangesLog changesLog) {
    if ((changesLog.getSystemId() == null) && (changesLog.getSessionId() != null) ) {
      changesLog.setSystemId(systemId);
      // broadcast messages
      try {
        this.send(changesLog);
        log.info("save");
        log.info("After save message -->" + systemId);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    // else changesLog is from other sources,
    // no needs to broadcast again, ignore silently
    

  }

  public void receive(ItemDataChangesLog changesLog) throws Exception {
    if (changesLog.getSystemId() == null || changesLog.getSystemId().equals(this.systemId)) {
      throw new Exception("Invalid or same systemId " + changesLog.getSystemId());
    }
    dataManager.save(changesLog);
  }

  private void send(ItemDataChangesLog itemDataChangesLog) throws Exception {
    PendingChangesLog container = new PendingChangesLog(itemDataChangesLog);

    switch (container.getConteinerType()) {
    case PendingChangesLog.Type.ItemDataChangesLog_without_Streams: {
      byte[] buf = PendingChangesLog.getAsByteArray(container.getItemDataChangesLog());

      Packet firstPacket = new Packet(Packet.PacketType.ItemDataChangesLog, buf.length, buf,
          container.getUUID());
      sendPacket(firstPacket);

      log.info("Send-->ItemDataChangesLog_without_Streams-->");
      log.info("---------------------");
      log.info("Size of damp --> " + buf.length);
      log.info("ItemStates   --> " + itemDataChangesLog.getAllStates().size());
      log.info("---------------------");
      break;
    }
    case PendingChangesLog.Type.ItemDataChangesLog_with_Streams: {
      byte[] buf = PendingChangesLog.getAsByteArray(container.getItemDataChangesLog());

      Packet packet = new Packet(Packet.PacketType.First_ItemDataChangesLog_with_Streams,
          buf.length, buf, container.getUUID());
      sendPacket(packet);

      for (int i = 0; i < container.getInputStreams().size(); i++)
        sendStream(container.getInputStreams().get(i), container.getFixupStreams().get(i),
            container.getUUID());

      Packet lastPacket = new Packet(Packet.PacketType.Last_ItemDataChangesLog_with_Streams,
          container.getUUID());
      sendPacket(lastPacket);
      
      log.info("Send-->ItemDataChangesLog_with_Streams-->");
      log.info("---------------------");
      log.info("Size of damp --> " + buf.length);
      log.info("ItemStates   --> " + itemDataChangesLog.getAllStates().size());
      log.info("Streams      --> " + container.getInputStreams().size());
      log.info("---------------------");
      
      break;
    }
    }
  }

  public Object handle(Message msg) {
    System.out.println();
    try {
      Packet packet = Packet.getAsPacket(msg.getBuffer());

      switch (packet.getPacketType()) {
      case Packet.PacketType.ItemDataChangesLog:
        ItemDataChangesLog changesLog = PendingChangesLog.getAsItemDataChangesLog(packet
            .getByteArray());

        log.info("Recive-->ItemDataChangesLog_without_Streams-->");
        log.info("---------------------");
        log.info("Size of recive damp --> " + packet.getByteArray().length);
        log.info("ItemStates          --> " + changesLog.getAllStates().size());
        log.info("---------------------");

        this.receive(changesLog);
        System.out.println("Item DataChangesLog of type 'ItemDataChangesLog only'");
        break;

      case Packet.PacketType.First_ItemDataChangesLog_with_Streams:
        changesLog = PendingChangesLog.getAsItemDataChangesLog(packet.getByteArray());

        PendingChangesLog container = new PendingChangesLog(changesLog, packet.getUUID(),
            PendingChangesLog.Type.ItemDataChangesLog_with_Streams);

        mapPendingChangesLog.put(packet.getUUID(), container);
        log.info("Item DataChangesLog of type 'ItemDataChangesLog first whith stream'");
        break;

      case Packet.PacketType.First_Packet_of_Stream:
        if (mapPendingChangesLog.containsKey(packet.getUUID())) {
          container = mapPendingChangesLog.get(packet.getUUID());

          container.getFixupStreams().add(packet.getFixupStream());

          File f = File.createTempFile("tempFile" + packet.getUUID() + UUIDGenerator.generate(), ".tmp"); 
            //new File(System.getProperty("java.io.tmpdir") + "/tmp" + packet.getUUID() + UUIDGenerator.generate() + ".file");
            
          container.getListFile().add(f);
          container.getListRandomAccessFiles().add(new RandomAccessFile(f, "rw"));
          log.info("First pocket of stream'");
        }
        break;

      case Packet.PacketType.Packet_of_Stream:
        if (mapPendingChangesLog.containsKey(packet.getUUID())) {
          container = mapPendingChangesLog.get(packet.getUUID());

          RandomAccessFile randomAccessFile = container
              .getRandomAccessFile(packet.getFixupStream());

          if (randomAccessFile != null) {
            randomAccessFile.seek(packet.getFileOffset());
            randomAccessFile.write(packet.getByteArray());
          }
        }
        break;

      case Packet.PacketType.Last_Packet_of_Stream:
        if (mapPendingChangesLog.containsKey(packet.getUUID())) {
          container = mapPendingChangesLog.get(packet.getUUID());

          RandomAccessFile randomAccessFile = container
              .getRandomAccessFile(packet.getFixupStream());

          if (randomAccessFile != null) {
            randomAccessFile.seek(packet.getFileOffset());
            randomAccessFile.write(packet.getByteArray());
            randomAccessFile.close();
          }
          log.info("Last pocket of stream'");
        }
        break;

      case Packet.PacketType.Last_ItemDataChangesLog_with_Streams:
        if (mapPendingChangesLog.get(packet.getUUID()) != null)
          mapPendingChangesLog.get(packet.getUUID()).restore();

        ItemDataChangesLog dataChangesLog = (mapPendingChangesLog.get(packet.getUUID()))
            .getItemDataChangesLog();
        if (dataChangesLog != null) {
          
          log.info("Send-->ItemDataChangesLog_with_Streams-->");
          log.info("---------------------");
          log.info("ItemStates   --> " + dataChangesLog.getAllStates().size());
          log.info("Streams      --> " + (mapPendingChangesLog.get(packet.getUUID()).getInputStreams().size()));
          log.info("---------------------");
          
          this.receive(dataChangesLog);
          mapPendingChangesLog.remove(packet.getUUID());
        }
        break;
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return new String("Success !");
  }

  private void sendPacket(Packet packet) throws Exception {
    byte[] buffer = Packet.getAsByteArray(packet);// os.toByteArray();

    Message msg = new Message(null, null, buffer);
    disp.castMessage(members , msg, GroupRequest.GET_NONE/* GET_ALL */, 0);
//    channel.send(null,null, msg);
  }

  private void sendStream(InputStream in, FixupStream fixupStream, String uuid) throws Exception {
    Packet packet = new Packet(Packet.PacketType.First_Packet_of_Stream, fixupStream, uuid);
    sendPacket(packet);

    byte[] buf = new byte[1024*200];
    int len;
    long offset = 0;

    // int sendWindow = 1000;
    // int sendCount = 0;

    try {
      while ((len = in.read(buf)) > 0) {
        if (len == buf.length) {
          packet = new Packet(Packet.PacketType.Packet_of_Stream, fixupStream, uuid, buf);
          packet.setOffset(offset);
          sendPacket(packet);
        } else {
          byte[] buffer = new byte[len];
          for (int i = 0; i < len; i++)
            buffer[i] = buf[i];

          packet = new Packet(Packet.PacketType.Last_Packet_of_Stream, fixupStream, uuid, buffer);
          packet.setOffset(offset);
          sendPacket(packet);
        }
        offset += len;

        log.info("Send  --> " + offset);
        
        Thread.sleep(30);
        // sendCount++;
        // if (sendCount == sendWindow){
        // Thread.sleep(10000);
        // sendCount = 0;
        // }

      }
    } catch (Exception e) {
      e.printStackTrace();
    }

  }

  public void viewAccepted(View new_view) {
    log.info(new_view);
    members = new_view.getMembers();
    log.info(members.size()); 
    
    for (int i = 0; i < members.size(); i++) {
      Address address = (Address)(members.get(i));
      if(address.compareTo(channel.getLocalAddress()) == 0) 
        members.remove(i);
    }
    log.info(members.size());    
    
  }

  public void suspect(Address suspected_mbr) {
  }

  public void block() { 
  }

  public void unblock() {
  }
  
  public byte[] getState() {
    return null;
  }

  public void setState(byte[] state) {
  }
  
  private String getLocalIP(InetAddress adr) {
    String str = adr.toString();
    return str.replaceAll("/", "");
  }
}
