/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.ext.action;

import java.util.Iterator;

import javax.jcr.RepositoryException;

import org.apache.commons.chain.Context;
import org.apache.commons.chain.impl.ContextBase;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.command.action.Action;
import org.exoplatform.services.command.action.ActionCatalog;
import org.exoplatform.services.command.action.Condition;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeType;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.observation.ExtendedEvent;


/**
 * Created by The eXo Platform SARL        .
 * @author Gennady Azarenkov
 * @version $Id: $
 */

public class SessionActionHandler {
  
  private final ActionCatalog catalog;
  private final ExoContainer container;

  public SessionActionHandler(ActionCatalog catalog, ExoContainer container) {
    this.catalog = catalog;
    this.container = container;
  }
  
  public void onAddNode(NodeImpl node) {
    if(catalog == null)
      return;
    Condition conditions = new Condition();
    conditions.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);
    try {
      conditions.put(SessionEventMatcher.NODETYPE_KEY, ((ExtendedNodeType)node.getPrimaryNodeType()).getQName());
//      InternalQName[] ns = new InternalQName[1];
//      ns[0] = ((NodeTypeImpl)node.getPrimaryNodeType()).getQName();
//      conditions.put(SessionEventMatcher.PARENT_NODETYPES_KEY, ns);
      conditions.put(SessionEventMatcher.PATH_KEY, node.getInternalPath());
    } catch (RepositoryException e) {
      e.printStackTrace();
    }
    Context ctx = new ContextBase();
    ctx.put("jcr:newNode", node);
    ctx.put("exocontainer", container);
    launch(conditions, ctx);
  }
  
  public void onSetProperty(PropertyImpl property) {
    if(catalog == null)
      return;

    int state;
    if(property.isNew()) {
      state = ExtendedEvent.PROPERTY_ADDED;
    } else {
      state = ExtendedEvent.PROPERTY_CHANGED;
    }
    Condition conditions = new Condition();
    conditions.put(SessionEventMatcher.EVENTTYPE_KEY, state);
    NodeImpl parent = null;;
    try {
      parent = (NodeImpl)property.getParent();
//      InternalQName[] ns = new InternalQName[1];
//      ns[0] = ((NodeTypeImpl)parent.getPrimaryNodeType()).getQName();
      conditions.put(SessionEventMatcher.NODETYPE_KEY, 
          ((ExtendedNodeType)parent.getPrimaryNodeType()).getQName());
      conditions.put(SessionEventMatcher.PARENT_NODETYPE_KEY, 
          ((ExtendedNodeType)parent.getPrimaryNodeType()).getQName());
      conditions.put(SessionEventMatcher.PATH_KEY, property.getInternalPath());
//      conditions.put(SessionEventMatcher.NAME_KEY, property.getInternalName());
      
    } catch (RepositoryException e) {
      e.printStackTrace();
    }
    Context ctx = new ContextBase();
//    ctx.put("jcr:parent", parent);
    ctx.put("currentItem", property);
    ctx.put("exocontainer", container);
    launch(conditions, ctx);
  }
  


  protected final void launch(Condition conditions, Context context) {
    Iterator <Action> i = catalog.getActions(conditions).iterator();
    while(i.hasNext()) {
      try {
        i.next().execute(context);
        //System.out.println(">>>>launch >>> !!!!! ");
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
}
