/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.storage.jdbc.db;

import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.util.Properties;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.impl.util.io.FileCleaner;
import org.exoplatform.services.jcr.storage.value.ValueStoragePluginProvider;

/**
 * Created by The eXo Platform SARL
 *
 * 23.03.2007
 *
 * Access Oracle implicit connection caching and pooling stuff using reflection 
 * to prevent Maven dependecies on ora drivers from POM. 
 *
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: OracleConnectionFactory.java 13869 2007-03-28 13:50:50Z peterit $
 */
public class OracleConnectionFactory extends GenericConnectionFactory {

  public static int CONNCACHE_MAX_LIMIT = 25;
  public static int CONNCACHE_MIN_LIMIT = 2;
  
  public static int CONNCACHE_INACTIVITY_TIMEOUT = 3600;
  public static int CONNCACHE_ABADONDED_TIMEOUT = 1800;
  
  protected final Object ociDataSource;
  
  public OracleConnectionFactory (
      String dbDriver,
      String dbUrl, 
      String dbUserName, 
      String dbPassword, 
      String containerName, 
      boolean multiDb, 
      ValueStoragePluginProvider valueStorageProvider, 
      int maxBufferSize, 
      File swapDirectory, 
      FileCleaner swapCleaner) throws RepositoryException {
    
    // ;D:\Devel\oracle_instantclient_10_2\;C:\oracle\ora92\bin;
    
    /* ERROR: if no oci in path and oci url requested
      
       Error: java.lang.reflect.InvocationTargetException
java.lang.reflect.InvocationTargetException
        at sun.reflect.NativeConstructorAccessorImpl.newInstance0(Native Method)
        at sun.reflect.NativeConstructorAccessorImpl.newInstance(NativeConstructorAccessorImpl.java:39)
        at sun.reflect.DelegatingConstructorAccessorImpl.newInstance(DelegatingConstructorAccessorImpl.java:27)
        at java.lang.reflect.Constructor.newInstance(Constructor.java:494)
        at ocipool.ConnPoolAppl.main(ConnPoolAppl.java:58)
Caused by: java.lang.UnsatisfiedLinkError: no ocijdbc10 in java.library.path
        at java.lang.ClassLoader.loadLibrary(ClassLoader.java:1682)
    ---------------------------------------------------------------------------
    
       ERROR: if thin url used and trying obtain oci data source
      
       java.lang.reflect.InvocationTargetException
        at sun.reflect.NativeConstructorAccessorImpl.newInstance0(Native Method)
        at sun.reflect.NativeConstructorAccessorImpl.newInstance(NativeConstructorAccessorImpl.java:39)
        at sun.reflect.DelegatingConstructorAccessorImpl.newInstance(DelegatingConstructorAccessorImpl.java:27)
        at java.lang.reflect.Constructor.newInstance(Constructor.java:494)
        at ocipool.ConnPoolAppl.main(ConnPoolAppl.java:58)
Caused by: java.lang.ClassCastException: oracle.jdbc.driver.T4CConnection
        at oracle.jdbc.pool.OracleOCIConnectionPool.createConnectionPool(OracleOCIConnectionPool.java:893)
     */
    
    super(dbDriver, dbUrl, dbUserName, dbPassword, containerName, multiDb, valueStorageProvider, maxBufferSize, swapDirectory, swapCleaner);
    
    Object cds = null;
    try {
      Class cdsClass = OracleConnectionFactory.class.getClassLoader().loadClass("oracle.jdbc.pool.OracleDataSource");
      Constructor cdsConstructor = cdsClass.getConstructor(new Class[] {});
      cds = cdsConstructor.newInstance(new Object[] {});
      
      // set cache properties    
      Properties prop = new java.util.Properties();    
      prop.setProperty("InitialLimit", String.valueOf(CONNCACHE_MIN_LIMIT));    
      prop.setProperty("MinLimit", String.valueOf(CONNCACHE_MIN_LIMIT));    
      prop.setProperty("MaxLimit", String.valueOf(CONNCACHE_MAX_LIMIT));    
      //prop.setProperty("MaxStatementsLimit", "2"); 
      prop.setProperty("InactivityTimeout", String.valueOf(CONNCACHE_INACTIVITY_TIMEOUT));  
      prop.setProperty("AbandonedConnectionTimeout", String.valueOf(CONNCACHE_ABADONDED_TIMEOUT)); 
      
      // ods.setURL(url);
      Method setURL = cds.getClass().getMethod("setURL", new Class[] {String.class});
      setURL.invoke(cds, new Object[] {this.dbUrl});
          
      //ods.setUser("qaadmin");
      Method setUser = cds.getClass().getMethod("setUser", new Class[] {String.class});
      setUser.invoke(cds, new Object[] {this.dbUserName});
      
      // ods.setPassword("qa12321");
      Method setPassword = cds.getClass().getMethod("setPassword", new Class[] {String.class});
      setPassword.invoke(cds, new Object[] {this.dbPassword});
      
      // ods.setConnectionCachingEnabled(true); // be sure set to true
      Method setConnectionCachingEnabled = cds.getClass().getMethod("setConnectionCachingEnabled", new Class[] {boolean.class});
      setConnectionCachingEnabled.invoke(cds, new Object[] {true});
      
      // ods.setConnectionCacheProperties (prop);
      Method setConnectionCacheProperties = cds.getClass().getMethod("setConnectionCacheProperties", new Class[] {Properties.class});
      setConnectionCacheProperties.invoke(cds, new Object[] {prop});
      
      // ods.setConnectionCacheName("ImplicitCache01"); // this cache's name
      Method setConnectionCacheName = cds.getClass().getMethod("setConnectionCacheName", new Class[] {String.class});
      setConnectionCacheName.invoke(cds, new Object[] {"EXOJCR_OCI__" + containerName});
      
    } catch(Throwable e) {
      cds = null;
      String err = "Oracle OCI connection cache is unavailable due to error " + e;
      if (e.getCause() != null) {
        err += " (" + e.getCause() + ")";
      }
      err += ". Standard JDBC DriverManager will be used for connections opening.";
      if (log.isDebugEnabled())
        log.warn(err, e);
      else
        log.warn(err);
    }
    this.ociDataSource = cds; // actually instance of javax.sql.DataSource 
  }
  
  @Override
  public Connection getJdbcConnection() throws RepositoryException {
    if (ociDataSource != null)
      try {
        return getCachedConnection();
      } catch(Throwable e) {
        throw new RepositoryException("Oracle OCI cached connection open error " + e, e);
      }

    return super.getJdbcConnection();
  }

  protected Connection getCachedConnection() throws NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
//    Method getConnection = ociPool.getClass().getMethod("getConnection", 
//        new Class[] {String.class, String.class}
//    );
//    return (Connection) getConnection.invoke(ociPool, new Object[] {dbUserName, dbPassword});
    
    // NOTE: ociDataSource - actually instance of javax.sql.DataSource 
    Method getConnection = ociDataSource.getClass().getMethod("getConnection", new Class[] {});
    Connection conn = (Connection) getConnection.invoke(ociDataSource, new Object[] {});
    
//    try {
//      switch (conn.getTransactionIsolation()) {
//      case Connection.TRANSACTION_NONE : log.info("TransactionIsolation: TRANSACTION_NONE"); break;
//      case Connection.TRANSACTION_READ_COMMITTED : log.info("TransactionIsolation: TRANSACTION_READ_COMMITTED"); break;
//      case Connection.TRANSACTION_READ_UNCOMMITTED : log.info("TransactionIsolation: TRANSACTION_READ_UNCOMMITTED"); break;
//      case Connection.TRANSACTION_REPEATABLE_READ : log.info("TransactionIsolation: TRANSACTION_REPEATABLE_READ"); break;
//      case Connection.TRANSACTION_SERIALIZABLE : log.info("TransactionIsolation: TRANSACTION_SERIALIZABLE"); break;
//      }
//    } catch (SQLException e) {
//      log.error(e);
//    }
    return conn;
  }

}
