/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.util.io;

import java.util.Map;
import java.util.WeakHashMap;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 05.10.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: StorageFile.java 12843 2007-02-16 09:11:18Z peterit $
 */
public class StorageFile extends SharedFile {
  
  static private Map<String, StorageFile> cache = new WeakHashMap<String, StorageFile>();
  
  protected StorageFile(String fileName) {
    super(fileName);
    this.FILE_SUBJECT_NAME = "storage";
  }
  
  static public StorageFile get(final String path) {
    final StorageFile sf = cache.get(path);
    if (sf != null && !sf.isFinalyzed() && sf.exists())
      return sf;
    
    return null;
  }
  
  static public StorageFile create(final String path) {
    final StorageFile sf = get(path);
    if (sf == null) {
      return createInCache(path);
    }
    
    return sf;
  }
  
  static protected StorageFile createInCache(final String path) {
    final StorageFile sf = new StorageFile(path);
    synchronized(cache) {
      cache.put(path, sf);
    }
    return sf;
  }
  
  static protected StorageFile removeFromCache(final StorageFile file) {
    synchronized(cache) {
      return cache.remove(file.getAbsolutePath());
    }
  }
  
  // ------------------------- SharedFile work ----------------------------
  
  @Override
  protected void onDelete() {
    removeFromCache(this);
  }
}
