/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.core;

import java.util.Map;
import java.util.WeakHashMap;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.log.ExoLogger;
import org.picocontainer.Startable;

/**
 * SessionRegistry ported from 1.6.
 * Cause: observation refactoring http://jira.exoplatform.org/browse/JCR-276
 * 
 * Changes: SessionRegistry isn't configurable through the repository configuration.
 * There are no Session live timeout and as result are no a SessionCleaner.
 *  
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: SessionRegistry.java 16905 2007-06-16 13:28:39Z peterit $
 */
public final class SessionRegistry implements Startable {
  
  private final Map<String, SessionImpl> sessionsMap;

  protected Log log = ExoLogger.getLogger("jcr.SessionRegistry");

  public SessionRegistry(RepositoryEntry config) {
    sessionsMap = new WeakHashMap<String, SessionImpl>();
  }

  public void registerSession(SessionImpl session) {
    synchronized (sessionsMap) {
      sessionsMap.put(session.getId(), session);
    }
  }

  public void unregisterSession(String sessionId) {
    synchronized (sessionsMap) {
      sessionsMap.remove(sessionId);
    }
  }

  public SessionImpl getSession(String sessionId) {
    return sessionsMap.get(sessionId);
  }
  
  public boolean isInUse(String workspaceName){
    if (workspaceName == null)
      return sessionsMap.size() > 0;
    for (SessionImpl session : sessionsMap.values()) {
      if (session.getWorkspace().getName().equals(workspaceName))
        return true;
    }
    return false;
  }
  
  public void start() {
    sessionsMap.clear();
  }

  public void stop() {
    sessionsMap.clear();
  }
}
