/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.webdav.property.factory;

import java.util.ArrayList;

import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.property.DavProperty;
import org.exoplatform.services.webdav.property.dav.CheckedInProp;
import org.exoplatform.services.webdav.property.dav.CheckedOutProp;
import org.exoplatform.services.webdav.property.dav.ChildCountProp;
import org.exoplatform.services.webdav.property.dav.ContentLengthProp;
import org.exoplatform.services.webdav.property.dav.CreationDateProp;
import org.exoplatform.services.webdav.property.dav.DisplayNameProp;
import org.exoplatform.services.webdav.property.dav.HasChildrenProp;
import org.exoplatform.services.webdav.property.dav.IsCollectionProp;
import org.exoplatform.services.webdav.property.dav.IsFolderProp;
import org.exoplatform.services.webdav.property.dav.IsRootProp;
import org.exoplatform.services.webdav.property.dav.IsVersioned;
import org.exoplatform.services.webdav.property.dav.LastModifiedProp;
import org.exoplatform.services.webdav.property.dav.LockDiscoveryProp;
import org.exoplatform.services.webdav.property.dav.ParentNameProp;
import org.exoplatform.services.webdav.property.dav.PredecessorSet;
import org.exoplatform.services.webdav.property.dav.ResourceTypeProp;
import org.exoplatform.services.webdav.property.dav.SuccessorSetProp;
import org.exoplatform.services.webdav.property.dav.SupportedLockProp;
import org.exoplatform.services.webdav.property.dav.SupportedMethodSetProp;
import org.exoplatform.services.webdav.property.dav.SupportedQueryGrammarSetProp;
import org.exoplatform.services.webdav.property.dav.VersionHistoryProp;
import org.exoplatform.services.webdav.property.dav.VersionNameProp;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: PropertyFactory.java 12635 2007-02-07 12:57:47Z gavrikvetal $
 */

public class PropertyFactory {
  
  private MappingTable mappingTable;
  private PropertyConfigTable configTable;
  
  public static final String [][]PROPERTIES = {
    {DavProperty.CHECKEDIN,
          CheckedInProp.class.getCanonicalName()},    
    {DavProperty.CHECKEDOUT,
          CheckedOutProp.class.getCanonicalName()},    
    {DavProperty.CREATIONDATE,
          CreationDateProp.class.getCanonicalName()},    
//    {DavProperty.CREATORDISPLAYNAME,
//          CreatorDisplayNameProp.class.getCanonicalName()},    
    {DavProperty.DISPLAYNAME,
          DisplayNameProp.class.getCanonicalName()},    
    {DavProperty.GETCONTENTLENGTH, 
          ContentLengthProp.class.getCanonicalName()},    
//    {DavProperty.GETCONTENTTYPE,
//          ContentTypeProp.class.getCanonicalName()},    
    {DavProperty.GETLASTMODIFIED, 
          LastModifiedProp.class.getCanonicalName()},
    {DavProperty.ISCOLLECTION,
          IsCollectionProp.class.getCanonicalName()},
    {DavProperty.LOCKDISCOVERY,
          LockDiscoveryProp.class.getCanonicalName()},
    {DavProperty.PREDECESSORSET,
          PredecessorSet.class.getCanonicalName()},
    {DavProperty.RESOURCETYPE,
          ResourceTypeProp.class.getCanonicalName()},
    {DavProperty.SUCCESSORSET,
          SuccessorSetProp.class.getCanonicalName()},
    {DavProperty.SUPPORTEDLOCK,
          SupportedLockProp.class.getCanonicalName()},
    {DavProperty.VERSIONHISTORY,
          VersionHistoryProp.class.getCanonicalName()},
    {DavProperty.VERSIONNAME,
          VersionNameProp.class.getCanonicalName()},
    {DavProperty.Search.SUPPORTEDQUERYGRAMMARSET,
          SupportedQueryGrammarSetProp.class.getCanonicalName()},
    {DavProperty.SUPPORTEDMETHODSET,
          SupportedMethodSetProp.class.getCanonicalName()},
    {DavProperty.PARENTNAME,
          ParentNameProp.class.getCanonicalName()},
    {DavProperty.ISROOT,
          IsRootProp.class.getCanonicalName()},
    {DavProperty.ISVERSIONED,
          IsVersioned.class.getCanonicalName()},
    {DavProperty.HASCHILDREN,
          HasChildrenProp.class.getCanonicalName()},
    {DavProperty.CHILDCOUNT,
          ChildCountProp.class.getCanonicalName()},
    {DavProperty.ISFOLDER,
          IsFolderProp.class.getCanonicalName()}
  };
  
  public PropertyFactory(MappingTable mappingTable, PropertyConfigTable configTable) {
    this.mappingTable = mappingTable;
    this.configTable = configTable;
  }
    
  public PropertyDefine getDefine(String propertyNameSpace, String propertyName) {
    return new PropertyDefine(propertyNameSpace, propertyName, mappingTable, configTable);
  }
  
  public ArrayList<PropertyDefine> getDefines(String nodeTypeName) {    
    ArrayList<PropertyDefine> defines = new ArrayList<PropertyDefine>();
    
    ArrayList<String> includes = configTable.getIncludes(nodeTypeName);
    for (int i = 0; i < includes.size(); i++) {
      String curPropertyName = includes.get(i);

      int pos = curPropertyName.indexOf(":"); 
      
      if (pos > 0) {
        String propertyNameSpace = curPropertyName.substring(0, pos + 1);
        String propertyName = curPropertyName.substring(pos + 1);
        defines.add(getDefine(propertyNameSpace, propertyName));
      } else {
        defines.add(getDefine(DavConst.DAV_NAMESPACE, curPropertyName));
      }
            
    }
    
    return defines;
  }
  
}

//DAV:lastaccessed
//DAV:defaultdocument
//DAV:isstructureddocument
//DAV:getcontentlanguage
//DAV:contentclass
//DAV:isreadonly
//DAV:ishidden
//DAV:href
//DAV:name
