/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: TestConfigSerialization.java 10464 2006-11-21 18:55:33Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.deploy;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.exoplatform.applet.deploy.Component;
import org.exoplatform.applet.deploy.LauncherConfiguration;
import org.exoplatform.applet.util.Base64Serializer;

/**
 * @version $Rev: 10464 $ $Date: 2006-11-22 01:55:33 +0700 (Wed, 22 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class TestConfigSerialization extends TestCase {

	private File serFile;

	protected void setUp() throws Exception {

		 serFile = new File("Configuration.ser");
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test method for {@link org.exoplatform.applet.util.Base64Serializer#serialize(java.io.Serializable)}.
	 * @throws IOException 
	 */
	public void testSerializeToFile() throws IOException {

		if (serFile.exists()) {
			serFile.delete();
		}

		FileOutputStream out = new FileOutputStream(serFile);

		LauncherConfiguration cfg = getConfiguration();

		out.write(Base64Serializer.serialize(cfg));
		out.close();

		System.out.println("File created: " + serFile.getAbsolutePath());
	}

	/**
	 * Test method for {@link org.exoplatform.applet.util.Base64Serializer#deserialize(byte[])}.
	 * @throws IOException 
	 * @throws ClassNotFoundException 
	 */
	public void testDeserializeFromFile() throws IOException, ClassNotFoundException {

		if (serFile.exists()) {
			System.out.println("File found: " + serFile.getAbsolutePath());
		}
		else {
			fail("No serialization file found.");
		}

		FileInputStream in = new FileInputStream(serFile);

		ByteArrayOutputStream encoded = new ByteArrayOutputStream();

		int read = 0;
		byte[] block = new byte[128];

		while ((read = in.read(block)) != -1) {
			encoded.write(block, 0, read);
		}

		in.close();

		LauncherConfiguration cfg = (LauncherConfiguration) Base64Serializer.deserialize(encoded.toByteArray());

		serFile.deleteOnExit();

		assertNotNull(cfg);
	}

	private static LauncherConfiguration getConfiguration() {
		LauncherConfiguration config = new LauncherConfiguration();

		config.setAppletClassName(
				"org.exoplatform.applications.signerclient.ExoSignerApplet");
		config.setAppletDependencyId("exo-ecm.exo-ecm.applications.signer-client");
		config.setAppletInfo("Copyright (c) 2001-2006 eXo Platform SARL");
		config.setAppletName("eXo Signer Client");
		config.setInstallBaseDirName("signer");
		config.setRepoBaseUrl("/applet/distro/");

		List deps = getComponents();
		config.setComponents(deps);

		return config;
	}

	private static List getComponents() {

        List items = new ArrayList();
        
        items.add(createItem(
                "exo-ecm",
				"exo-ecm.applications.signer-client",
				"1.0",
				"jar",
				Component.APPLET_SCOPE,
				null,
				new String[] {
					"org.exoplatform.applications.signerclient.crypto.jce.JCECryptoPlugin"
				}));
        items.add(createItem(
                "bouncycastle",
				"lcrypto-jdk14",
				"134",
				"jar",
				Component.SHARED_SCOPE,
				null,
				null));

		return items;
	}

	private static Component createItem(String groupId, String artifactId,
			String version, String type, String scope,
			String classifier, String[] pluginsClassNames) {

		Component dep = new Component(groupId, artifactId, version, type,
				classifier, scope);

		if (pluginsClassNames != null) {
			for (int i=0; i<pluginsClassNames.length; i++) {
				String p = pluginsClassNames[i];
				dep.addPluginClassName(p);
			}
		}
				
		return dep;
	}

}
