/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: AbstractLogger.java 8955 2006-09-18 13:46:06Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.logging;

import java.util.ResourceBundle;

/**
 * @version $Rev: 8955 $ $Date: 2006-09-18 20:46:06 +0700 (Mon, 18 Sep 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public abstract class AbstractLogger implements Logger {

	private static final String ENTERING_MSG = "ENTRY";
	private static final String EXITING_MSG = "EXIT";
	private static final String THROWING_MSG = "THROW";

	private static final int offValue = Level.OFF.intValue();

	private String key;

	private String resourceBundleName;

	private Level loggingLevel;
	private int levelValue;

	protected AbstractLogger(String key, String resourceBundleName) {
		super();
		this.key = key;
		this.resourceBundleName = resourceBundleName;
	}

    public String getName() {
	    return key;
    }

    public String getResourceBundleName() {
	    return resourceBundleName;
    }

    /* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#config(java.lang.String)
	 */
	public void config(String msg) {
		log(Level.CONFIG, msg);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#entering(java.lang.String, java.lang.String)
	 */
	public void entering(String sourceClass, String sourceMethod) {
		logp(Level.FINER, sourceClass, sourceMethod, ENTERING_MSG);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#entering(java.lang.String, java.lang.String, java.lang.Object)
	 */
	public void entering(String sourceClass, String sourceMethod, Object param) {
		logp(Level.FINER, sourceClass, sourceMethod, ENTERING_MSG, param);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#entering(java.lang.String, java.lang.String, java.lang.Object[])
	 */
	public void entering(String sourceClass, String sourceMethod,
	        Object[] params) {
		logp(Level.FINER, sourceClass, sourceMethod, ENTERING_MSG, params);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#exiting(java.lang.String, java.lang.String)
	 */
	public void exiting(String sourceClass, String sourceMethod) {
		logp(Level.FINER, sourceClass, sourceMethod, EXITING_MSG);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#exiting(java.lang.String, java.lang.String, java.lang.Object)
	 */
	public void exiting(String sourceClass, String sourceMethod, Object result) {
		logp(Level.FINER, sourceClass, sourceMethod, EXITING_MSG, result);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#fine(java.lang.String)
	 */
	public void fine(String msg) {
		log(Level.FINE, msg);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#finer(java.lang.String)
	 */
	public void finer(String msg) {
		log(Level.FINER, msg);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#finest(java.lang.String)
	 */
	public void finest(String msg) {
		log(Level.FINEST, msg);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#info(java.lang.String)
	 */
	public void info(String msg) {
		log(Level.INFO, msg);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#isEnabled()
	 */
	public boolean isLogging() {
		return !(Level.OFF.equals(getCurrentLevel()));
	}
	
	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#throwing(java.lang.String, java.lang.String, java.lang.Throwable)
	 */
	public void throwing(String sourceClass, String sourceMethod,
	        Throwable thrown) {
		logp(Level.FINER, sourceClass, sourceMethod, THROWING_MSG, thrown);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#warning(java.lang.String)
	 */
	public void warning(String msg) {
		log(Level.WARNING, msg);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.Logger#severe(java.lang.String)
	 */
	public void severe(String msg) {
		log(Level.SEVERE, msg);
	}

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#getCurrentIntLevel()
     */
    public int getCurrentIntLevel() {
	    return loggingLevel.intValue();
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#getCurrentLevel()
     */
    public Level getCurrentLevel() {
	    return loggingLevel;
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#log(org.exoplatform.applet.logging.Level, java.lang.String)
     */
    public void log(Level level, String msg) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#log(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.Object)
     */
    public void log(Level level, String msg, Object param) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setParameters(new Object[] { param });
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#log(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.Object[])
     */
    public void log(Level level, String msg, Object[] params) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setParameters(params);
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#log(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.Throwable)
     */
    public void log(Level level, String msg, Throwable thrown) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setThrown(thrown);
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logp(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String)
     */
    public void logp(Level level, String sourceClass, String sourceMethod, String msg) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logp(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String, java.lang.Object)
     */
    public void logp(Level level, String sourceClass, String sourceMethod, String msg, Object param) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	lr.setParameters(new Object[] {param});
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logp(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String, java.lang.Object[])
     */
    public void logp(Level level, String sourceClass, String sourceMethod, String msg, Object[] params) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	lr.setParameters(params);
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logp(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String, java.lang.Throwable)
     */
    public void logp(Level level, String sourceClass, String sourceMethod, String msg, Throwable thrown) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	lr.setThrown(thrown);
    	doLog(lr, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logrb(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	doLog(lr, bundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logrb(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.Object)
     */
    public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg, Object param) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	lr.setParameters(new Object[] {param});
    	doLog(lr, bundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logrb(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.Object[])
     */
    public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg, Object[] params) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	lr.setParameters(params);
    	doLog(lr, bundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#logrb(org.exoplatform.applet.logging.Level, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.Throwable)
     */
    public void logrb(Level level, String sourceClass, String sourceMethod, String bundleName, String msg, Throwable thrown) {
    	if (exitCondition(level)) return;

    	LogRecord lr = new LogRecord(level, msg);
    	lr.setSourceClassName(sourceClass);
    	lr.setSourceMethodName(sourceMethod);
    	lr.setThrown(thrown);
    	doLog(lr, bundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.Logger#setLevel(org.exoplatform.applet.logging.Level)
     */
    public void setLevel(Level level) {
		this.loggingLevel = (level == null ? Level.OFF : level);
	    this.levelValue = loggingLevel.intValue();
    }

    private boolean exitCondition(Level level) {
    	return (level.intValue() > levelValue || levelValue == offValue);
    }

    private void doLog(LogRecord lr, String bundleName) {
    	lr.setLoggerName(key);
    	if (bundleName != null) {
    		ResourceBundle rb = ResourceBundle.getBundle(bundleName);
    		lr.setResourceBundle(rb);
    	}
    	log(lr);
    }

    protected abstract void log(LogRecord lr);

}
