/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: FileLogger.java 9981 2006-11-01 12:56:42Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.logging.file;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.exoplatform.applet.install.Installer;
import org.exoplatform.applet.logging.AbstractLogger;
import org.exoplatform.applet.logging.LogRecord;
import org.exoplatform.applet.security.RestrictedAction;
import org.exoplatform.applet.security.RestrictedActionFactory;

/**
 * @version $Rev: 9981 $ $Date: 2006-11-01 19:56:42 +0700 (Wed, 01 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class FileLogger extends AbstractLogger {

	private static RestrictedAction access = RestrictedActionFactory.getInstance();

	/**
     * @param key
     * @param resourceBundleName
     */
    FileLogger(String key, String resourceBundleName) {
	    super(key, resourceBundleName);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applet.logging.AbstractLogger#log(org.exoplatform.applet.logging.LogRecord)
     */
    protected void log(LogRecord lr) {

    	OutputStream out = null;
    	PrintWriter file = null;

		synchronized (FileLogger.class) {
			try {

				String pattern = "{0} - {1} {2} - {3}({4}) - {5}";

				String date = DateFormat.getDateTimeInstance().format(new Date(
						System.currentTimeMillis()));

				String params = (lr.getParameters() == null ? "" : paramsArray(lr.getParameters()));

				String line = MessageFormat.format(pattern, new Object[] {
						date,
						padd( "<" + lr.getLevel().toString() + ">", 9),
						padd(lr.getSourceClassName(), 20),
						padd(lr.getSourceMethodName(), 20),
						padd(params, 50),
						(lr.getMessage() == null ? "" : lr.getMessage()),
						});

   			    logToConsole(line);

   			    out = getFileLogOutputStream();
		    	file = new PrintWriter(out);

		    	file.println(line);

   			    if (lr.getThrown() != null) {
   			    	lr.getThrown().printStackTrace(file);
   			    }

			} catch (Throwable t1) {
				t1.printStackTrace();
			} finally {
				if (file != null) {
					file.flush();
					file.close();
				}
				if (out != null) {
	                try {
	                    out.close();
                    } catch (IOException e) {
                    	// do nothing
                    }
				}
			}
		}
	    
    }

    /**
     * @param parameters
     * @return
     */
    private String paramsArray(Object[] parameters) {
	    StringBuffer result = new StringBuffer("[");
	    for(int i=0;i<parameters.length;i++) {
	    	Object p = parameters[i];
	    	if (result.length() > 2)
	    		result.append(',');
	    	result.append(p.toString());
	    }
	    result.append("]");
	    return result.toString();
    }

	private static String padd(String text, int size) {
    	return padd(text, size, true);
    }

    private static String padd(String text, int size, boolean alignLeft) {

    	StringBuffer result = new StringBuffer();

		int diff = (text == null ? 0 - size : text.length() - size);

		if (diff < 0) {
    		
			if (alignLeft) {
    			if (text != null) result.append(text);
	    		for (int i=0; i < Math.abs(diff); i++)
	    			result.append(' ');
			}
			else {
	    		for (int i=0; i < Math.abs(diff); i++)
	    			result.append(' ');
    			if (text != null) result.append(text);
    		}
    	}
    	else if (diff > 0) {
    		return text.substring(0, size);
    	}
    	else {
    		return text;
    	}

    	return result.toString();
    }

    private static OutputStream getFileLogOutputStream() {
    	OutputStream out = null;

    	String fileName;

    	SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
    	fileName = fmt.format(new Date(System.currentTimeMillis())) + ".log";

    	File path = new File(Installer.getInstallPath("log"));

    	if (!access.exists(path))
    		access.mkdirs(path);

    	out = access.getFileOutputStream(path.getAbsolutePath() +
    			File.separator + fileName, true);

    	return out;
    }

    private void logToConsole(String out) {
    	System.out.println(out);
    }

}
