/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: JavaScriptObject.java 9990 2006-11-01 19:21:46Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.scripting;

import java.applet.Applet;
import java.lang.reflect.Method;

import org.exoplatform.applet.logging.Logger;
import org.exoplatform.applet.logging.file.FileLoggerFactory;

/**
 * @version $Rev: 9990 $ $Date: 2006-11-02 02:21:46 +0700 (Thu, 02 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class JavaScriptObject implements ScriptingObject {

	private static final String JSOBJECT_CLASS_NAME = "netscape.javascript.JSObject";

	private static boolean initialized = false;

	private static Class clazz;
	private static Method getWindowMethod;
	private static Method callMethod;
	private static Method evalMethod;
	private static Method getMemberMethod;
	private static Method setMemberMethod;

	private static Logger logger = FileLoggerFactory.getInstance().getLogger(JavaScriptObject.class.getName());

	private Object instance;

    public JavaScriptObject(Object jsObject) {
	    super();
	    this.instance = jsObject;
	    initialize();
    }

	private static void initialize() {
		if (initialized)
			return;

		try {
	        clazz = Class.forName(JSOBJECT_CLASS_NAME);

	        Method ms[] = clazz.getMethods();
	        for (int i=0; i<ms.length; i++) {

	        	String methodName = ms[i].getName();
	        	
	        	if ("getWindow".equals(methodName))
	        		getWindowMethod = ms[i];
	        	else if ("call".equals(methodName))
	        		callMethod = ms[i];
	        	else if ("eval".equals(methodName))
	        		evalMethod = ms[i];
	        	else if ("getMember".equals(methodName))
	        		getMemberMethod = ms[i];
	        	else if ("setMember".equals(methodName))
	        		setMemberMethod = ms[i];
	        	
	        }
        } catch (ClassNotFoundException e) {
        	throw new RuntimeException(e);
        }

		initialized = true;
	}

	private static Object invoke(Method m, Object obj, Object[] args) {
		try {
	        return m.invoke(obj, args);
        } catch (Exception e) {
        	throw new RuntimeException(e);
        }
		
	}

	public static JavaScriptObject getWindow(Applet applet) {
		initialize();
		Object window = invoke(getWindowMethod, clazz, new Object[] {applet});
		logger.finest("Returning Window: " + window);
		return new JavaScriptObject(window);
	}

	/* (non-Javadoc)
     * @see org.exoplatform.applications.signer.applet.browser.ScriptingObject#call(java.lang.String, java.lang.Object[])
     */
    public Object call(String methodName, Object[] args) {
    	Object params[] = new Object[] {methodName, args};
	    return invoke(callMethod, instance, params);
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applications.signer.applet.browser.ScriptingObject#eval(java.lang.String)
     */
    public Object eval(String s) {
    	Object result = invoke(evalMethod, instance, new Object[] {s});
    	if ((result != null) && (clazz.isInstance(result))) {
			return new JavaScriptObject(result);
    	} else {
    		return result;
    	}
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applications.signer.applet.browser.ScriptingObject#getMember(java.lang.String)
     */
    public Object getMember(String name) {
    	Object member = invoke(getMemberMethod, instance, new Object[] {name});
    	if ((member != null) && (clazz.isInstance(member))) {
			return new JavaScriptObject(member);
    	} else {
    		return member;
    	}
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applications.signer.applet.browser.ScriptingObject#setMember(java.lang.String, java.lang.Object)
     */
    public void setMember(String name, Object value) {
	    invoke(setMemberMethod, instance, new Object[] {name, value});
    }
}
