package org.exoplatform.services.templates.velocity.impl ;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.RuntimeServices;
import org.apache.velocity.runtime.resource.Resource;
import org.apache.velocity.runtime.resource.ResourceManager;
import org.apache.velocity.runtime.resource.loader.ResourceLoader;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.templates.velocity.ResourceLoaderPlugin;
import org.exoplatform.services.templates.velocity.StaticResourceLoader;

public class ResourceManagerImpl implements ResourceManager {
  public static final int RESOURCE_TEMPLATE = 1;
  public static final int RESOURCE_CONTENT = 2;
  
  protected RuntimeServices rsvc = null;
  private   StaticResourceLoaderImpl staticResLoader_ ;
  private List plugins_ = new ArrayList() ;
  
  
  public ResourceManagerImpl(StaticResourceLoaderImpl floader) throws Exception {
    staticResLoader_ = floader ;
  }
  
  public void initialize( RuntimeServices rs ) throws Exception {
    rsvc = rs;
    staticResLoader_.setRuntimeServices(rs) ;
    rsvc.info("Default ResourceManager initializing. (" + this.getClass() + ")");     
    rsvc.info("Default ResourceManager initialization complete.");
  }
  
  public Resource getResource(String resourceName, int resourceType, String encoding )
  throws ResourceNotFoundException, ParseErrorException, Exception {
    for(int i= 0; i < plugins_.size(); i++) {
      ResourceLoaderPlugin plugin = (ResourceLoaderPlugin) plugins_.get(i) ;
      if(plugin.canLoadResource(resourceName))  {
        return plugin.getResource( resourceName, resourceType, encoding);
      }
    }
    return staticResLoader_.getResource( resourceName, resourceType, encoding);
  }
  
  public Resource getResource(String resourceName, int resourceType  )
  throws ResourceNotFoundException, ParseErrorException, Exception {  
    return getResource( resourceName, resourceType, RuntimeConstants.ENCODING_DEFAULT);
  }
  
  public String getLoaderNameForResource(String resourceName ) {
    ResourceLoader resourceLoader = null;
    for(int i= 0; i < plugins_.size(); i++) {
      ResourceLoaderPlugin plugin = (ResourceLoaderPlugin) plugins_.get(i) ;
      if(plugin.canLoadResource(resourceName))  {
        resourceLoader = plugin ;
      }
    }
    if(resourceLoader == null) resourceLoader = staticResLoader_ ;
    
    InputStream is = null;
    try {
      is = resourceLoader.getResourceStream( resourceName );
      if( is != null)  return resourceLoader.getClass().toString();
    } catch( ResourceNotFoundException e) {
    } finally {
      if (is != null) {
        try  {
          is.close();
        } catch( IOException ioe)  { }
      }
    }
    return null;
  }
  
  public void addPlugin(ComponentPlugin plugin) {
    ResourceLoaderPlugin loader = (ResourceLoaderPlugin) plugin ;
    loader.setRuntimeServices(rsvc);
    plugins_.add(loader) ; 
  }
  
  public StaticResourceLoader getStaticResourceLoader()  { 
    return staticResLoader_;
  }
}