/*
 * Created on Mar 14, 2005
 */
package org.exoplatform.faces.core.renderer.html;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.services.log.LogUtil;

/**
 * @author benjaminmestrallet
 */
public class UploadInputRenderer extends HtmlBasicRenderer {

  public void encodeBegin(FacesContext context, UIComponent component)  throws IOException {
    ResponseWriter w = context.getResponseWriter();
    UIUploadInput uiInput = (UIUploadInput) component;

    if(uiInput.isEdited()) {
      String msg = getMessage(context, "UploadInput.msg.has-binary-data", "Binary data exists");
      w.write("<span>"+ msg + "</span>");           
    } else {
      renderInput(w, uiInput);
    }


    if(uiInput.hasSelectors()){
      Collection selectors = uiInput.getSelectors();
      for (Iterator iter = selectors.iterator(); iter.hasNext();) {
        String button = (String) iter.next();
        w.write(button);
      }
    }    
    if (uiInput.hasError()) {
      w.write("<span style='color: red'>*</span>");
    }
  }

  private String getMessage(FacesContext context, String key, String defaultValue) {
    String msg = defaultValue;
    try {
      ResourceBundle res = getApplicationResourceBundle(context.getExternalContext());
      msg = res.getString(key);
    } catch (Exception e) {
      LogUtil.getLog(getClass()).warn("Could not find bundle key: " + key);
    }
    return msg;
  }

  private void renderInput(ResponseWriter w, UIUploadInput uiInput) throws IOException {
    w.write("<input name='");
    w.write(uiInput.getName());
    w.write("'");
    w.write(" type='file'");
    if (uiInput.getClazz() != null) {
      w.write(" class='");
      w.write(uiInput.getClazz());
      w.write("'");
    }
    if (!uiInput.isEditable() || uiInput.isReadonly()) {
      w.write(" readonly='readonly' ");
    }
    w.write("/>");
  }

}