package org.exoplatform.velocity.directive;

import java.io.IOException;
import java.io.Writer;
import java.util.ResourceBundle;
import org.apache.velocity.context.InternalContextAdapter;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.RuntimeServices;
import org.apache.velocity.runtime.directive.Directive;
import org.apache.velocity.runtime.parser.node.Node;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.model.Parameter;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since March 3, 2005
 * @version $Id$
 */
public class JSFLink extends Directive {
  private String label_ ;
  private boolean useResource_ ;
  
  public String getName()  {    return "jsfLink"; }        
  public int getType() {  return LINE; }        
  
  public void init(RuntimeServices rs, InternalContextAdapter context, Node node) throws Exception {
    super.init(rs, context, node);
    label_ = (String)node.jjtGetChild(0).value(context);
    if(label_ == null) {
      useResource_ = false ;
      return ;
    }
    
    if(label_.startsWith("@")) {
      label_ = label_.substring(1) ;
      useResource_ = true ;
    } else {
      useResource_ = false ;
    }
  }
  
  public boolean render(InternalContextAdapter context,  Writer w, Node node)
  throws IOException,  MethodInvocationException, ResourceNotFoundException, ParseErrorException {        
    Parameter[] params = (Parameter[])node.jjtGetChild(1).value(context);
    UIExoComponent component = (UIExoComponent)context.get(UIExoComponent.UICOMPONENT) ;
    w.write("<a  href='");
    w.write(component.getBaseURL());
    for (int i = 0; i < params.length; i++) {
      w.write("&amp;");
      w.write(params[i].getName());
      w.write('=');
      w.write(params[i].getValue());
    }
    if(node.jjtGetNumChildren() > 2) {
      String  beanid = node.jjtGetChild(2).value(context).toString();
      w.write("&amp;" + UIExoComponent.OBJECTID + "="); w.write(beanid);
    }
    w.write("'>");
    if(label_ == null) {
      Object label = node.jjtGetChild(0).value(context);
      if(label == null)  label =  component.getId() ;
      w.write((String)label);
    } else {
      String  label = label_ ;
      if(useResource_) {
        ResourceBundle res = (ResourceBundle)context.get("res") ;
        label = res.getString(label_) ;
      }
      w.write(label);
    }
    w.write("</a>");
    return true;
  }
}