/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.zip.ZipFile;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.portal.config.PortalConfigService;
import org.exoplatform.portal.config.impl.NodeNavigationData;
import org.exoplatform.portal.config.impl.PageData;
import org.exoplatform.portal.config.impl.PortalConfigData;
import org.exoplatform.portal.config.impl.PortalConfigServiceImpl;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserProfile;
import org.hibernate.Session;
/**
 * Created by The eXo Platform SARL
 * Author : Hoa  Pham
 *          hoapham@exoplatform.com,phamvuxuanhoa@yahoo.com
 * Nov 15, 2005
 */
public class TestImportExportService extends PortalServiceTestBase {  
  private static PortalConfigService pconfigService_ ;
  
  public TestImportExportService(String name) {
    super( name);   
  }  
  public void setUp() throws Exception {
    PortalContainer manager  = PortalContainer.getInstance();           
    backupService_ = (ImportExportService)manager.getComponentInstanceOfType(ImportExportService.class) ;
    pconfigService_ = (PortalConfigService)manager.getComponentInstanceOfType(PortalConfigService.class) ;
    hservice_ = (HibernateService)manager.getComponentInstanceOfType(HibernateService.class) ;
    orgService_ = (OrganizationService)manager.getComponentInstanceOfType(OrganizationService.class) ;
  }
  public void tearDown() throws Exception {
    orgService_ = null ;
  }
  public void testImportExportPortal() throws Exception {
    System.out.println("#######--- Begin Export all user's data------#####") ;
    prepareOrganizationData() ;
    /*------create portal and node navigation for user*/
    pconfigService_.getPortalConfig(demo, demo) ;
    pconfigService_.getNodeNavigation(demo, demo) ;
    
    pconfigService_.getPortalConfig(username1, username1) ;
    pconfigService_.getNodeNavigation(username1, username1) ;
    
    pconfigService_.getPortalConfig(username2, username2) ;
    pconfigService_.getNodeNavigation(username2, username2) ;    
    /*---------get all group and membership relation with users before export-import--*/
    Collection<Group> u1GroupBefore = orgService_.getGroupHandler().findGroupsOfUser(username1) ;        
    Collection<Membership> u1MemBefore = orgService_.getMembershipHandler().findMembershipsByUser(username1) ;
    
    /*---------------Assgin user2 to group1 with mType1------*/
    orgService_.getMembershipHandler().linkMembership( user2 ,group1,mType1,true) ;
    /*---------------Assgin user2 to group1 with mType2------*/
    orgService_.getMembershipHandler().linkMembership(user2,group2,mType1,true) ;
    /*---------Get all group and membership relation with user2 before export-import--*/
    Collection<Group> u2GroupBefore = orgService_.getGroupHandler().findGroupsOfUser(username2) ;
    Collection<Membership> u2MemBefore = orgService_.getMembershipHandler().findMembershipsByUser(username2) ;
    
    /*---------------Assgin demo{default user} to group1 with mType1------*/    
    orgService_.getMembershipHandler().linkMembership(userDefault,group1,mType1,true) ;
    /*---------Get all group and membership relation with "demo" before export-import--*/
    Collection<Group> groupBeforeOfDemo = orgService_.getGroupHandler().findGroupsOfUser("demo") ;
    Collection<Membership> memBeforeOfDemo = orgService_.getMembershipHandler().findMembershipsByUser("demo") ;
    
    backupService_.exportUserData() ;
    
    /* --Expect result: found 3 ZipFile with name:user1.zip, user2.zip,demo.zip(default user)
     * --------and in a file has 6 entries: Memberships.xml/User.xml/UserProfile.xml---
     *  ---- navigation-config.xml/pages-config.xml/portal-config.xml(/portal)-----*/
    String[] portalEntries = {"Meberships.xml", "User.xml","UserProfile","navigation-config.xml",
        "pages-config.xml","portal-config.xml"} ;
    String[] organizationEntries = {"Memberships.xml","User.xml","UserProfile.xml"} ;
    ZipFile zfile = getZipUserData(username1) ;
    assertTrue("Expect entrys are found in zipfile of user:"+username1,hasEntry(zfile,organizationEntries)) ;
    assertTrue("Expect entrys are found in zipfile of user:"+username1,hasEntry(zfile,portalEntries)) ;
    zfile = getZipUserData(username2) ;
    assertTrue("Expect entrys are found in zipfile of user :"+username2,hasEntry(zfile,organizationEntries)) ;
    assertTrue("Expect entrys are found in zipfile of user :"+username2,hasEntry(zfile,portalEntries)) ;
    zfile = getZipUserData("demo") ;
    assertTrue("Expect entrys are found in zipfile of user :demo",hasEntry(zfile,organizationEntries)) ;
    assertTrue("Expect entrys are found in zipfile of user :demo",hasEntry(zfile,portalEntries)) ;
    zfile.close() ;
    System.out.println("--------- Success export all user's data-----------") ;
    
    System.out.println("--------- Clean database before import-----------") ;
    orgService_.getUserHandler().removeUser(username1,true) ;
    orgService_.getUserHandler().removeUser(username2,true) ;
    orgService_.getUserHandler().removeUser("demo",true) ;
    System.out.println("--------- Clean database successfull-----------") ;
    
    System.out.println("--------- begin import all user's data-----------") ;
    backupService_.importUserData() ;
    /*--------------Expect all data of USER_1 before/after export/import are same-----*/
    Collection<Group> u1GroupAfter = orgService_.getGroupHandler().findGroupsOfUser( username1) ;    
    assertCollection( u1GroupBefore,u1GroupAfter, new GroupComparator()) ;
    
    Collection<Membership> u1MemAfter = orgService_.getMembershipHandler().findMembershipsByUser(username1) ;
    assertCollection(u1MemBefore,u1MemAfter,new MembershipComparator()) ;
    
    /*---Expect all data of USER_2 before/after export/import are same-----*/
    Collection<Group> u2GroupAfter = orgService_.getGroupHandler().findGroupsOfUser(username2) ;
    assertCollection(u2GroupBefore,u2GroupAfter,new GroupComparator()) ;
    
    Collection<Membership> u2MemAfter = orgService_.getMembershipHandler().findMembershipsByUser(username2) ;
    assertCollection(u2MemBefore,u2MemAfter, new MembershipComparator()) ;
    
    /*---Expect all data of DEFAULT USER(demo) before/after export/import are same-----*/
    Collection<Group> groupAfterOfDemo = orgService_.getGroupHandler().findGroupsOfUser("demo") ;
    assertCollection(groupBeforeOfDemo,groupAfterOfDemo,new GroupComparator()) ;
    
    Collection<Membership> memAfterOfDemo = orgService_.getMembershipHandler().findMembershipsByUser("demo") ;
    assertCollection(memBeforeOfDemo,memAfterOfDemo,new MembershipComparator()) ;    
    System.out.println("######--End import all user's data successful #####-----") ;    
    
    System.out.println("--------- Begin Export single user's data-----------") ;
    prepareOrganizationData() ;
    /*---------------Assgin user1 to group1 with mType1------*/
    orgService_.getMembershipHandler().linkMembership(user1,group1,mType1,true) ;
    /*---------------Assgin user1 to group1 with mType2------*/
    orgService_.getMembershipHandler().linkMembership(user1,group1,mType2,true) ;
    /*---------------Assgin user1 to group2 with mType2------*/
    orgService_.getMembershipHandler().linkMembership(user1,group2,mType2,true) ;
    /*--------- get all data relation with user before import/export------------*/
    Collection<Membership> memBefore = orgService_.getMembershipHandler().findMembershipsByUser(username1) ;    
    Collection<Group> gBefore = orgService_.getGroupHandler().findGroupsOfUser(username1) ;
    
    Session session = hservice_.openSession();    
    PortalConfigData portalBefore = (PortalConfigData) session.get( PortalConfigData.class, username1) ;     
    List l = 
      session.createQuery( PortalConfigServiceImpl.queryPageDataByOwner).setString(0, username1).list();
    
    List pagesBefore = new ArrayList(l.size() + 5)  ;
    for(int i = 0 ; i < l.size(); i++) {
      PageData pageData = (PageData) l.get(i) ;
      pagesBefore.add(pageData.getPage()) ;
    }
    NodeNavigationData navigationBefore = 
      (NodeNavigationData) session.get(NodeNavigationData.class, username1) ;    
    session.clear() ;
    
    /* --Expect result: found a ZipFile with name:user1.userName.zip
     * --------and it's entries are: Memberships.xml/User.xml/UserProfile.xml (organization)
     *  ---- navigation-config.xml/pages-config.xml/portal-config.xml(/portal)-----*/
    String[] entryNames = {"Meberships.xml", "User.xml","UserProfile","navigation-config.xml",
        "pages-config.xml","portal-config.xml"} ;
    backupService_.exportUserData(username1) ;    
    zfile = getZipUserData(username1) ;    
    assertTrue("Expect 'memberships.xml' entry is found", hasEntry(zfile,"Memberships.xml")) ;
    assertTrue("Expect 'User.xml' entry is found", hasEntry(zfile,"User.xml")) ;
    assertTrue("Expect 'UserProfile.xml' entry is found", hasEntry(zfile,"UserProfile.xml" )) ;
    assertTrue("Expect  entries is found", hasEntry(zfile,entryNames )) ;
    zfile.close() ;
    System.out.println("---------Export User data successful-----------") ;
    
    System.out.println("---------Clean database before import-----------") ;
    orgService_.getUserHandler().removeUser(username1,true) ;
    System.out.println("---------Clean database successfull-----------") ;
    
    System.out.println("------- Begin Import User data  --------------") ;    
    backupService_.importUserData(username1) ;
    User u = orgService_.getUserHandler().findUserByName(username1) ;
    assertTrue("Expect user has found:",u!=null) ;
    assertTrue("Expect user's info is correct:",u.getFirstName().equalsIgnoreCase("default")) ;
    /*-------------find User Profile and check it----------------*/
    UserProfile profile = orgService_.getUserProfileHandler().findUserProfileByName(username1) ;
    assertTrue("Expect user profile has found:", profile.getUserName().equalsIgnoreCase(username1)) ;
    assertEquals("Expect profile's info is correct: user.gender is ","male", profile.getUserInfoMap().get("user.gender"));
    /*------------get all data relate with user after export/import and compare---------
     * ----------with data that get before import/export to make sure they are same-----*/
    Collection<Group>  gAfter= orgService_.getGroupHandler().findGroupsOfUser(username1) ;    
    assertCollection(gBefore,gAfter,new GroupComparator()) ;
    
    Collection<Membership> memAfter = orgService_.getMembershipHandler().findMembershipsByUser(username1) ;
    assertCollection(memBefore,memAfter, new MembershipComparator()) ;
    
    session = hservice_.openSession();    
    PortalConfigData portalAfter = (PortalConfigData) session.get( PortalConfigData.class, username1) ;    
    l = 
      session.createQuery( PortalConfigServiceImpl.queryPageDataByOwner).setString(0, username1).list();
    
    List pagesAfter = new ArrayList(l.size() + 5)  ;
    for(int i = 0 ; i < l.size(); i++) {
      PageData pageData = (PageData) l.get(i) ;
      pagesAfter.add(pageData.getPage()) ;
    }
    NodeNavigationData navigationAfter = 
      (NodeNavigationData) session.get(NodeNavigationData.class, username1) ;    
    
    session.clear() ;    
    assertObject(portalBefore,portalAfter,new PortalConfigDataComparator()) ;
    //assertCollection(pagesBefore,pagesAfter,new PageConfigDataComparator()) ;
    assertObject(navigationBefore,navigationAfter,new NavigationConfigDataComparator()) ;
    System.out.println("----------Import single user data succesful -----------------") ;            
  }
  
  protected String getDescription() {
    return "Test Import- Export Portal" ;
  }
}
