/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.log;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List ;
import java.util.Date;
import java.util.GregorianCalendar;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.portal.log.PortalLogData;
import org.exoplatform.portal.log.Query;
import org.exoplatform.services.cache.CacheListener;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.cache.FIFOExoCache;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.hibernate.Session;

/**
 * @author Hung Nguyen (nqhungvn@users.sourceforge.net)
 * @since Sep 26, 2005
 * @version $Id$
 */
public class PortalActivitiesLogPlugin extends BaseComponentPlugin implements CacheListener {  
  private FIFOExoCache portalActivitiesLogCache_ ;
  private List<Object> logsToSave_ = new LinkedList<Object>();
  private int currentTimeIndex_ ;
  private Date currentTimeSlot_ ;
  private HibernateService hservice_ ;
  private CacheService cservice_ ;  
  private String uniqueId_ ;
  
  public PortalActivitiesLogPlugin(HibernateService hservice, CacheService cservice) throws Exception {  
    hservice_ = hservice ;
    cservice_ = cservice ;
    portalActivitiesLogCache_ = (FIFOExoCache)cservice_.getCacheInstance("PortalActivitiesLogCache") ;
    portalActivitiesLogCache_.addCacheListener(this) ;
    updateTime() ;    
  }
  
  public void  updateTime() throws Exception {
    GregorianCalendar  calendar = new  GregorianCalendar() ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    currentTimeSlot_ = calendar.getTime() ;        
    currentTimeIndex_ = calendar.get(GregorianCalendar.HOUR_OF_DAY) ;
    uniqueId_ = "/" + String.valueOf(calendar.get(GregorianCalendar.YEAR)) + 
                "-" + String.valueOf(calendar.get(GregorianCalendar.MONTH)+1) +
                "-" + String.valueOf(calendar.get(GregorianCalendar.DATE)) +
                "/" + String.valueOf(currentTimeIndex_);    
    if(!portalActivitiesLogCache_.isEmpty()){
      synchronized (logsToSave_){
        logsToSave_.addAll(portalActivitiesLogCache_.getCachedObjects()) ;
        portalActivitiesLogCache_.clearCache() ;
      }
    }     
  }
  
  public int getCurrentTimeIndex() { return currentTimeIndex_ ; }
  
  public void logPortalAccess(String portalId, String portalOwner, long exeTime, boolean error) {        
    getOrCreatePortalLogData(portalId, portalOwner, exeTime, error);    
  }
  
  private void getOrCreatePortalLogData(String portalId, String portalOwner, long exeTime, boolean error) {        
    try {
      PortalLogData impl = 
        (PortalLogData) portalActivitiesLogCache_.get(portalOwner + uniqueId_) ;
      if(impl == null) {
        impl = new PortalLogData() ;
        impl.logPortalAccess(portalOwner + uniqueId_, portalId, portalOwner, currentTimeSlot_, 
            currentTimeIndex_, exeTime, error) ;
        portalActivitiesLogCache_.put(portalOwner + uniqueId_, impl) ;
      }else {
        impl.updatePortalAccess(exeTime, error) ;
      }       
    } catch(Exception ex) {
      ex.printStackTrace() ;      
    }
  }
  
  synchronized public void savePortalLogData(PortalLogData data) throws Exception {
    Session session = hservice_.openSession() ;
    PortalLogData impl =  (PortalLogData) session.get(PortalLogData.class, data.getId()) ;
    if(impl == null) {
      impl = data ;      
      session.save(impl) ;
    } else {
      impl.merge(data) ;
      session.update(impl) ;      
    }
    session.flush() ;       
  }
  
  public PageList getPortalLogDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PortalLogData.class);
    oq.addGT("timeSlot", q.getFromTimeSlot()) ;
    oq.addLT("timeSlot", q.getToTimeSlot()) ;
    oq.addGT("accessCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("portalOwner", componentId) ;
    if(q.getOrder() != "" && q.getOrder().equals("Desc")){
      oq.setDescOrderBy("timeSlot") ;
    }else if(q.getOrder() != "" && q.getOrder().equals("Asc")){
      oq.setAscOrderBy("timeSlot") ;
    }
    return new DBObjectPageList(hservice_, oq);
  }
  
  public List getPortalLogChartDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PortalLogData.class);
    oq.addSelect("portalOwner") ;
    oq.addSUM("accessCounter") ;
    oq.addSUM("errorCounter") ;
    oq.addSUM("summaryAccessTime") ;
    if(q.getMostAccess() != "" && q.getMostAccess().equals("most24")) {
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
      int dateBefore = calendar.get(GregorianCalendar.DATE) - 1;
      calendar.set(GregorianCalendar.DATE, dateBefore) ;
      oq.addGT("timeSlot", calendar.getTime()) ;
    }else if (q.getMostAccess() != "" && q.getMostAccess().equals("overall")){
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
    }else {
      oq.addGT("timeSlot", q.getFromTimeSlot()) ;
      oq.addLT("timeSlot", q.getToTimeSlot()) ; 
    }
    oq.addGT("accessCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("portalOwner", componentId) ;
    oq.setGroupBy("portalOwner") ;    
    String groupQuery = oq.getHibernateGroupByQuery() ;
    Session session = hservice_.openSession() ;
    if(q.getMostAccess() != "")
      groupQuery += " order by sum(o.accessCounter) desc " ;
    return session.createQuery(groupQuery).list() ;
  }
  
  public void  saveMonitorData() throws Exception {
    List cacheObjectList = portalActivitiesLogCache_.removeCachedObjects() ;
    for(int i = 0; i < cacheObjectList.size(); i++) {
      PortalLogData data = (PortalLogData)cacheObjectList.get(i) ;
      savePortalLogData(data) ;
    }
  }
  
  public void savePrevTimeSlotMonitorData() throws Exception {   
    if(logsToSave_.size() == 0)  return ;
    List<Object> list = null ;
    synchronized (logsToSave_) {
      list =  new ArrayList<Object>(logsToSave_) ;
      logsToSave_.clear() ;
    }
    for(int i = 0; i < list.size(); i++){
      PortalLogData data = (PortalLogData) list.get(i) ;
      savePortalLogData(data) ;
    }    
  }
  
  public void onExpire(ExoCache cache, Serializable key, Object value) throws Exception {
    PortalLogData data = (PortalLogData) value ;
    savePortalLogData(data) ;
  }
  
  public void onRemove(ExoCache cache, Serializable key, Object value) throws Exception {
  }
  
  public void onPut(ExoCache cache, Serializable key, Object value) throws Exception {
  }
  
  public void onGet(ExoCache cache, Serializable key, Object value) throws Exception {
  }
  
  public void onClearCache(ExoCache cache) throws Exception {
  }
}
