/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.jmx.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.management.*;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.jmx.component.model.JMXServer;
import org.exoplatform.portlets.jmx.component.model.MBeanCategory;
import org.exoplatform.portlets.jmx.component.model.MBeanDescription;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UINavigator.java,v 1.2 2004/08/01 04:18:27 tuan08 Exp $
 */
public class UINavigator extends UIExoCommand {
  static final public String SELECT_MBEAN_SERVER_ACTION = "selectMBeanServer" ;
  static final public String SELECT_CATEGORY_ACTION = "selectCategory" ;
  static final public String SELECT_MBEAN_ACTION = "selectMBean" ;
  
  private static Parameter[] selectMServerParams = { new Parameter(ACTION, SELECT_MBEAN_SERVER_ACTION) };
  private static Parameter[] selectCategoryParams = { new Parameter(ACTION, SELECT_CATEGORY_ACTION) };
  private static Parameter[] selectMBeanParams = { new Parameter(ACTION, SELECT_MBEAN_ACTION) };

  List<JMXServer>  servers_ ;
   
	public UINavigator() {
		setId("UINavigator") ;
		setRendererType("VelocityRenderer") ;
    servers_ = new ArrayList<JMXServer>();
		List<MBeanServer> mbservers = MBeanServerFactory.findMBeanServer(null) ;
		boolean select = true ;
		for(MBeanServer server  : mbservers) {
			JMXServer jmxserver = new JMXServer(server) ;
			jmxserver.setSelect(select) ;
			servers_.add(jmxserver)  ;
			select = false ;
		}
    addActionListener(SelectJMXServerListener.class, SELECT_MBEAN_SERVER_ACTION) ;
    addActionListener(SelectMBeanCategoryListener.class, SELECT_CATEGORY_ACTION) ;
    addActionListener(SelectMBeanListener.class, SELECT_MBEAN_ACTION) ;
	}
  
  public Parameter[] getSelectMBeanServerParams() { return selectMServerParams ; }
  
  public Parameter[] getSelectCategoryParams() { return selectCategoryParams ; }
  
  public Parameter[] getSelectMBeanParams() { return selectMBeanParams ; }
  
  public List<JMXServer> getJMXServers() { return servers_ ; }
  
  public JMXServer getSelectJMXServer() {
    for(JMXServer server:  servers_)  if(server.isSelect()) return server ;
    return null ;
  }
  
  static public class SelectJMXServerListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINavigator uiNavigator = (UINavigator)event.getComponent() ;
      String selectId = event.getParameter(OBJECTID) ;
      for(JMXServer server : uiNavigator.servers_) {
        if(server.getId().equals(selectId)) {
          server.setSelect(true) ;
          UIDetail  uiDetail  = (UIDetail) uiNavigator.getSibling(UIDetail.class) ;
          uiDetail.setViewUIListMBean(server, server.getDefaultCategory()) ;
        } else {
          server.setSelect(false) ;
        }
        
      }
    }
  }
  
  static public class SelectMBeanCategoryListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINavigator uiNavigator = (UINavigator)event.getComponent() ;
      String category = event.getParameter(OBJECTID) ;
      JMXServer server = uiNavigator.getSelectJMXServer() ;
      Collection<MBeanCategory>  categories = server.getCategories() ;
      for(MBeanCategory mbcategory : categories) {
        if(mbcategory.getCategory().equals(category)) {
          mbcategory.setSelect(!mbcategory.isSelect()) ;
          UIDetail  uiDetail  = (UIDetail) uiNavigator.getSibling(UIDetail.class) ;
          uiDetail.setViewUIListMBean(server,mbcategory) ;
        }
      }
    }
  }
  
  static public class SelectMBeanListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINavigator uiNavigator = (UINavigator)event.getComponent() ;
      String id = event.getParameter(OBJECTID) ;
      JMXServer server = uiNavigator.getSelectJMXServer() ;
      MBeanDescription md = server.getMBeanDescription(id) ;
      if(md != null) {
        UIDetail  uiDetail  = (UIDetail) uiNavigator.getSibling(UIDetail.class) ;
        uiDetail.setViewMBean(server, md);
      }
    }
  }
}