/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.rss.component;

import java.io.InputStream;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Created by The eXo Platform SARL
 * Author : Nhu Dinh Thuan
 *          nhudinhthuan@exoplatform.com
 * Jan 19, 2007  
 */
public class ParserService {
  
  static public Channel parse(String url) throws Exception {      
    Document document = createParser().parse(url);    
    return createChannel(document);
  }
  
  static public Channel parse(InputStream input) throws Exception {      
    Document document = createParser().parse(input);    
    return createChannel(document);
  }
  
  private static Channel createChannel(Document document) throws Exception{
    Channel channel = new Channel() ;
    if(setItems(channel, document)){
      setChannelInfo(channel, document) ;
    }else{
      setFeedInfo(channel, document) ;
    }
    channel.setUpdateTime(System.currentTimeMillis()) ;
    return channel ;
  }
  
  private static DocumentBuilder createParser() throws Exception {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setIgnoringComments(true);
    factory.setCoalescing(true);
    factory.setNamespaceAware(false);
    factory.setValidating(false);
    return factory.newDocumentBuilder();    
  }
  
  static private boolean setItems(Channel channelObj, Document doc) throws Exception {
    String root = doc.getDocumentElement().getTagName();
    Item[] items = null ;
    boolean isRss = true;
    if (root.equals("rdf:RDF")) {
      NodeList list = doc.getElementsByTagName("item");
      items = getItems(list);
    } else if (root.equals("rss")) {
      NodeList list = doc.getElementsByTagName("channel");
      if (list.getLength() < 1) items =  new Item[0];
      else {
        Node channel = list.item(0);
        items = getItems(channel.getChildNodes());
      }
    } else if (root.equals("feed")) {
      isRss = false;
      NodeList list = doc.getElementsByTagName("feed");
      if (list.getLength() < 1) items =  new Item[0];
      else {
        Node channel = list.item(0);
        items = getEntrys(channel.getChildNodes());
      }
    } else if (root.equals("xml")) {
      NodeList list = doc.getElementsByTagName("item");
      items =  getItems(list);
    } else {
      items = new  Item[0];
    }
    channelObj.setItems(items) ;    
    return isRss;
  }
  
  static private Item[] getItems(NodeList items) throws Exception {
    Vector v = new Vector();
    for (int i = 0; i < items.getLength(); ++i) {
      Node node = items.item(i);
      if(!node.getNodeName().equals("item")) continue;      
      NodeList itemChildren = node.getChildNodes();
      Item item = new Item() ;
      for (int j = 0; j < itemChildren.getLength(); ++j) {
        Node child = itemChildren.item(j);
        String nodeName = child.getNodeName() ;
        if (nodeName.equals("title")) {
          item.setTitle(getFirstNodeValue(child));
        }else if (nodeName.equals("link")) {
          item.setLink(getFirstNodeValue(child));
        }else if (nodeName.equals("description")) {
          item.setDescription(getFirstNodeValue(child));
        }
      }
      v.addElement(item);
    }
    Item[] foundItems = new Item[v.size()];
    v.copyInto(foundItems);
    return foundItems;
  }
  
  static private Item[] getEntrys(NodeList items) throws Exception {
    Vector v = new Vector();
    for (int i = 0; i < items.getLength(); ++i) {
      Node node = items.item(i);
      if(!node.getNodeName().equals("entry")) continue;      
      NodeList itemChildren = node.getChildNodes();
      Item item = new Item() ;
      for (int j = 0; j < itemChildren.getLength(); ++j) {
        Node child = itemChildren.item(j);
        String nodeName = child.getNodeName() ;
        if (nodeName.equals("title")) {
          item.setTitle(getFirstNodeValue(child));
        }else if (nodeName.equals("link")) {
          NamedNodeMap maps = node.getAttributes(); 
          Node href = maps.getNamedItem("href");
          if(href == null || href.getNodeValue() == null 
             || href.getNodeValue().length()< 2) continue;
          item.setLink(getFirstNodeValue(child));
        } else if(item.getDescription() != null && item.getDescription().length() > 0){
          continue;
        }else if (nodeName.equals("summary")) {
          item.setDescription(getFirstNodeValue(child));
        }else if (nodeName.equals("content")) {
          item.setDescription(getFirstNodeValue(child));
        }        
      }
      v.addElement(item);
    }
    Item[] foundItems = new Item[v.size()];
    v.copyInto(foundItems);
    return foundItems;
  }
  
  static private void setChannelInfo(Channel channelObj, Document doc) throws Exception {
    NodeList list = doc.getElementsByTagName("channel");
    if (list.getLength() < 1)  return;
    Node channel = list.item(0);
    list = channel.getChildNodes();
    for (int i = 0; i < list.getLength(); ++i) {
      Node node = list.item(i);
      String nodeName = node.getNodeName() ; 
      if ("title".equals(nodeName)) {
        channelObj.setTitle(getFirstNodeValue(node));
      } else  if ("link".equals(nodeName)) {
        channelObj.setLink(getFirstNodeValue(node));
      } else  if ("description".equals(nodeName)) {
        channelObj.setDescription(getFirstNodeValue(node));
      }
    }
  }
  
  static private void setFeedInfo(Channel channelObj, Document doc) throws Exception {
    NodeList list = doc.getElementsByTagName("feed");
    if (list.getLength() < 1)  return;
    Node channel = list.item(0);
    list = channel.getChildNodes();
    for (int i = 0; i < list.getLength(); ++i) {
      Node node = list.item(i);
      String nodeName = node.getNodeName() ; 
      if ("title".equals(nodeName)) {
        channelObj.setTitle(getFirstNodeValue(node));
      } else  if ("link".equals(nodeName)) {
        NamedNodeMap maps = node.getAttributes(); 
        Node rel = maps.getNamedItem("rel");
        Node href = maps.getNamedItem("href");
        if(href == null || href.getNodeValue() == null || href.getNodeValue().length()< 2) continue;
        boolean setValue = (rel == null) || 
                            "alternate".equals(rel.getNodeValue()) || 
                            "rel".equals(rel.getNodeValue());
        if(!setValue) continue;
        channelObj.setLink(rel.getNodeValue());
      } else  if ("info".equals(nodeName)) {
        channelObj.setDescription(getFirstNodeValue(node));
      } else  if ("subtitle".equalsIgnoreCase(nodeName)) {
        channelObj.setDescription(getFirstNodeValue(node));
      }
    }
  }
  
  static private String getFirstNodeValue(Node node){
    Node firstChild = node.getFirstChild();
    if(firstChild == null) return "";
    String value = firstChild.getNodeValue();
    return value  == null ? "" : value;
  }
    
}
