package org.esupportail.cas.server.handlers.ldap;

import org.dom4j.Element;
import org.esupportail.cas.server.util.RedundantHandler;

/**
 * This abstract class implements an LDAP handler class, inherited by
 * FastBindLdapHandler and BindLdapHandler.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
abstract class LdapHandler extends RedundantHandler {
	/**
	 * The filter for the usernames provided by the users.
	 */
	private String filter;
	
	/**
	 * Constructor.
	 *
	 * @param handlerElement the XML element that declares the handler 
	 * in the configuration file
	 * @param configDebug debugging mode of the global configuration
	 * @throws Exception Exception
	 */
	protected LdapHandler(
			final Element handlerElement, 
			final Boolean configDebug) throws Exception {
		super(handlerElement, configDebug);
		traceBegin();

		// check that a config element is present
		checkConfigElement(true);

		filter = getConfigSubElementContent("filter", true/*needed*/);
		trace("filter = " + filter);

		traceEnd();
	}

	/**
	 * Retrieve the filter of the usernames.
	 *
	 * @return a String Object.
	 */
	final String getFilter() {
		return filter;
	}
	
}

