/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web.report;

import groovy.lang.GroovyClassLoader;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.exoplatform.test.web.TestSuites;
import org.exoplatform.test.web.WebUnitSuite;
import org.exoplatform.test.web.unit.WebUnit;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Feb 20, 2006
 */
public class HTMLReport extends Report {
  private String outputDir_ ;
  private String summaryTemplate_ ;
  private String suiteTemplate_ ;
  private String unitTemplate_ ;
  private HTMLReportGroovyResourceManager rmanager_ ;
  
  public HTMLReport(TestSuites tsuites, String outputDir, String groovyPath) {
    super(tsuites) ;
    outputDir_ = outputDir ;
    GroovyClassLoader cl = new  GroovyClassLoader(Thread.currentThread().getContextClassLoader()) ;
    rmanager_ = new HTMLReportGroovyResourceManager(cl , null) ;
    summaryTemplate_ =  "org/exoplatform/test/web/report/templates/summary.groovy" ;
    suiteTemplate_ =  "org/exoplatform/test/web/report/templates/suite.groovy" ;
    unitTemplate_ =  "org/exoplatform/test/web/report/templates/unit.groovy" ;
  }
  
  public void report() throws Exception {
    mkdir(outputDir_) ;
    reportSummary() ;
    reportSuites() ;
  }
  
  
  public void  reportSummary() throws Exception {
    List<WebUnitSuite> suites = tsuites_.getSuites() ;
    Map<String, Object> binding = new HashMap<String,Object> () ;
    binding.put("rmanager", rmanager_) ;
    binding.put("report", this ) ;
    binding.put("suites", suites) ;
    FileOutputStream os = new FileOutputStream(outputDir_ + "/index.html") ;
    PrintWriter w = new PrintWriter(os) ;
    rmanager_.merge(w, summaryTemplate_ , binding) ;
    os.close() ;
  }
  
  public void  reportSuites() throws Exception {
    List<WebUnitSuite> suites = tsuites_.getSuites() ;
    for(WebUnitSuite suite : suites) {
      String suiteDir = outputDir_ + "/" + suite.getName() ;
      mkdir(suiteDir) ;
      Map<String, Object> binding = new HashMap<String,Object> () ;
      binding.put("rmanager", rmanager_) ;
      binding.put("report", this ) ;
      binding.put("suite", suite) ;
      FileOutputStream os = new FileOutputStream(suiteDir + "/index.html") ;
      PrintWriter w = new PrintWriter(os) ;
      rmanager_.merge(w, suiteTemplate_ , binding) ;
      os.close() ;
      for(WebUnit unit : suite.getWebUnits()) {
        binding = new HashMap<String,Object> () ;
        binding.put("rmanager", rmanager_) ;
        binding.put("report", this ) ;
        binding.put("suite", suite) ;
        binding.put("unit", unit) ;
        os = new FileOutputStream(suiteDir + "/" + unit.hashCode() + ".html") ;
        w = new PrintWriter(os) ;
        rmanager_.merge(w, unitTemplate_ , binding) ;
        os.close() ;
      }
    }
  }
  
  public  void mkdir(String path) throws Exception {
    File  file  = new  File(path) ;
    if(!file.exists()) {
      file.mkdirs() ;
    }
    if(!file.isDirectory()) {
      throw new Exception("The file " +  path + " is  existed  but it is not a directory") ;
    }
  }
}
