/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.admin;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * Need to check that the changes to the rule have been taken into accoount 
 *
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * July 19, 2006
 */
public class AdminDriveSuite extends WebUnitSuite {
  static String DESC = "Go to Admin The ECM page, click on Manage Driver button and test all the basic actions" ;
  static String NAME = "AdminDriveSuite" ;  
  
  public AdminDriveSuite() {    
    super(NAME, DESC) ;   
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;      
    //Manage Drives
    addWebUnit(
      new ClickURLLinkUnit("ManageDrives", "Click on Manage Drives button").
      setURL("&uicomponent=UIECMAdminSummary&op=manageDrives").                                                         
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDriveList&op=add"))      
    ) ;    
    //-----Delete if existed drive-test
    addWebUnit(
      new ClickURLLinkUnit("DeleteExistedDrive", "Click on delele button if existed drive-test").
      setURL("&uicomponent=UIDriveList&op=delete&objectId=drive-test").              
      addCondition(new HaveLinkWithURLCondition("&uicomponent=UIDriveList&op=delete&objectId=drive-test")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDriveList&op=delete&objectId=drive-test",0))      
    ) ;      
    //-----Add New Drives
    addWebUnit(
      new ClickURLLinkUnit("AddNewDrives", "Click on Add button").
      setURL("&uicomponent=UIDriveList&op=add").        
      addValidator(new ExpectFormIdValidator("uiDriveForm"))      
    ) ;
    //------------Search Apply View
    addWebUnit(
      new SubmitFormUnit("SearchApplyView", "Click on search icon").
      setFormId("uiDriveForm").      
      setField("op","selectView").
      addValidator(new ExpectFormIdValidator("uiViewList"))                                                   
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveApplyView", "Click on save button").
      setFormId("uiViewList").
      setCheckBox("view-test",true).
      setCheckBox("mac-view",true).
      setCheckBox("simple-view",true).
      setField("op","save").
      addValidator(new ExpectTextValidator("view-test"))                                                   
    ) ;    
    //------------Search Permission
    addWebUnit(
      new SubmitFormUnit("SearchPermission", "Click on search icon").
      setFormId("uiDriveForm").      
      setField("op","searchGroup").       
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectPermission&objectId=*:/user"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectPermission", "Select permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectPermission&objectId=*:/user").
      addValidator(new ExpectTextValidator("*:/user"))                                                              
    ) ;   
    //-----Save New Drives
    addWebUnit(
      new SubmitFormUnit("SaveNewDrives", "Click on save link").
      setFormId("uiDriveForm").
      setField("name","drive-test").                  
      setField("workspace","production").
      setField("path","/").
      setField("folderType","BothTypes").
//      setField("views","view-test").      
      setField("permissions","*:/user").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDriveList&op=edit&objectId=drive-test"))                                                   
    ) ;            
    //-----Go back to Admin the ECM
    addWebUnit(
      new ClickURLLinkUnit("BackToAdmintheECM", "Click on back button").
      setURL("&uicomponent=UIDriveList&op=back"). 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageDrives"))                                                   
    ) ;
  }     
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}