/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.admin;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * Need to check that the changes to the rule have been taken into accoount 
 *
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class AdminRulesSuite extends WebUnitSuite {
  static String DESC = "Go to Admin The ECM page, click on Manage Rule button and test all the basic actions" ;
  static String NAME = "AdminRulesSuite" ;
    
  public AdminRulesSuite() {    
    super(NAME, DESC) ;   
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;      
    addWebUnit(
      new ClickURLLinkUnit("ManageRules", "Click on Manage Rules button").
      setURL("&uicomponent=UIECMAdminSummary&op=manageRules").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIRuleList&op=add"))      
    ) ;    
    //Add New Rules
    addWebUnit(
      new ClickURLLinkUnit("AddNewRules", "Click on Add button").
      setURL("&uicomponent=UIRuleList&op=add").        
      addValidator(new ExpectFormIdValidator("RuleForm"))      
    ) ;        
    addWebUnit(
      new SubmitFormUnit("SaveNewRules", "Click on save link").
      setFormId("RuleForm").
      setField("content","rule-content-test").                  
      setField("rule","rule-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIRuleList&op=edit&objectId=rule-test"))                                                    
    ) ;        
    //Edit Created Rules
    addWebUnit(
      new ClickURLLinkUnit("EditRules", "Click on edit button").
      setURL("&uicomponent=UIRuleList&op=edit&objectId=rule-test").      
      addValidator(new ExpectTextValidator("rule-content-test")).
      addValidator(new ExpectFormIdValidator("RuleForm")).
      addValidator(new ExpectTextValidator("Enable Versioning"))  
    ) ;        
    addWebUnit(
      new SubmitFormUnit("UpdateRule", "Click on save link").
      setFormId("RuleForm").
      setField("content","rule-content-test-one").                  
      setField("rule","rule-test").
      setCheckBox("enableVersioning",true).
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIRuleList&op=delete&objectId=rule-test"))           
    ) ;    
    //Reedit Created Rules
    addWebUnit(
      new ClickURLLinkUnit("ReEditRules", "Click on edit button").
      setURL("&uicomponent=UIRuleList&op=edit&objectId=rule-test").
      addValidator(new ExpectFormIdValidator("selectRuleVersionForm")).     
      addValidator(new ExpectTextValidator("1")).
      addValidator(new ExpectTextValidator("rule-content-test-one")).
      addValidator(new ExpectFormIdValidator("RuleForm")).
      addValidator(new ExpectTextValidator("Enable Versioning")) 
    ) ;    
    addWebUnit(
      new SubmitFormUnit("UpdateRule", "Click on save link").
      setFormId("RuleForm").
      setField("content","rule-content-test-two").                  
      setField("rule","rule-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIRuleList&op=delete&objectId=rule-test"))           
    ) ;   
    //Restore Rule Version
    addWebUnit(
      new ClickURLLinkUnit("ReEditRules", "Click on edit button").
      setURL("&uicomponent=UIRuleList&op=edit&objectId=rule-test").  
      addValidator(new ExpectTextValidator("2")).
      addValidator(new ExpectTextValidator("rule-content-test-two")).                        
      addValidator(new ExpectLinkWithTextValidator("restore"))
    ) ;    
    addWebUnit(
      new SubmitFormUnit("Select version 1", "select version 1 from select box").
      setFormId("selectRuleVersionForm").     
      setField("version","1").
      setField("op","changeVersion").
      addValidator(new ExpectTextValidator("1")).
      addValidator(new ExpectTextValidator("rule-content-test-one"))           
    ) ;  
    addWebUnit(
      new SubmitFormUnit("RestoreVersion", "Click on restore icon").
      setFormId("RuleForm").      
      setField("op","restoreVersion").      
      addValidator(new ExpectTextValidator("The version 1 was restored successfully.")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIRuleList&op=delete&objectId=rule-test"))           
    ) ;     
    //Delete Created Rules
    addWebUnit(
      new ClickURLLinkUnit("DeleteNewRule", "Click on Delete button").
      setURL("&uicomponent=UIRuleList&op=delete&objectId=rule-test").                  
      addValidator(new ExpectNoTextValidator("rule-test"))      
    ) ;
    //Go back to Admin the ECM
    addWebUnit(
      new ClickURLLinkUnit("BackToAdmintheECM", "Click on back button").
      setURL("uicomponent=UIRuleList&op=back"). 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageScripts"))         
    ) ;
  }     
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}