/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.temp;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * This suite does:
 *    1) Go to Manage ActionType page
 *    2) Click on Add New Action type icon
 *    3) Add a new Action Type that extends BPActionType with an "exo:variable1" 
 *        and "exo:variable2" as variables
 *    4) Add new Action Type that extends a Script Action Type
 *    5) Add new Action Type that extends a Rule Action Type
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class AdminMultiActionTypesSuite extends WebUnitSuite {
  static String DESC = "Go to Admin The ECM page, click on Create an Action Type button and test all the basic actions" ;
  static String NAME = "AdminMultiActionTypesSuite" ;
  
  public AdminMultiActionTypesSuite() {    
    super(NAME, DESC) ;
    goToPage("/Home/ecm") ;
    //Go to Manage ActionType page
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;      
    //Click on Add New Action Type icon
    addWebUnit(
      new ClickURLLinkUnit("CreateActionType", "Click on Create Action Type icon").
      setURL("&uicomponent=UIECMAdminSummary&op=createActionType").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionTypeList&op=add"))      
    ) ;   
    for (i in 1..63){
      //-----Add a new Action Type that extends BPActionType with an "exo:variable1" and "exo:variable2" as variables
      addWebUnit(
        new ClickURLLinkUnit("AddNewBPActionType", "Click on add new Action button").
        setURL("&uicomponent=UIActionTypeList&op=add"). 
        addValidator(new ExpectTextValidator("Add/Edit a Action Type"))      
      ) ;
      //---------------Add Variable text input
      addWebUnit(
        new SubmitFormUnit("AddVariableTextInput", "Add text input to add variable").
        setFormId("actionTypeForm").   
        setField("op","addValuevariables")      
      ) ;    
      //---------------Save new BP Action Type 
      addWebUnit(
        new SubmitFormUnit("SaveNewBPActionType", "Fill all info and click on save button").
        setFormId("actionTypeForm").   
        setField("name","exo:test-actionType"+i).
        setField("variables","exo:variable1").
        setField("variables[1]","exo:variable2").
        setField("op","save").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionTypeList&op=add"))
      ) ;
      
      //-----Add new Action Type that extends a Script Action Type
      addWebUnit(
        new ClickURLLinkUnit("AddNewScriptActionType", "Click on add new Action button").
        setURL("&uicomponent=UIActionTypeList&op=add").                  
        addValidator(new ExpectTextValidator("Add/Edit a Action Type"))      
      ) ;
      //-----------Select Extends Action Type is Script Action Type
      addWebUnit(
        new SubmitFormUnit("SelectExtendsActionType", "Select Extends Action Type is Script Action Type").
        setFormId("selectactionTypeForm").   
        setField("actionType","exo:scriptAction").
        setField("op","changeActionType").      
        addValidator(new ExpectTextValidator("SendMailScript.groovy"))      
      ) ;    
      //-----------Save new Script Action Type    
      addWebUnit(
        new SubmitFormUnit("SaveNewScriptActionType", "Fill all info and click on save button").
        setFormId("actionTypeForm").   
        setField("name","exo:test-actionType"+i).
        setField("variables","exo:variable1").      
        setField("op","save").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionTypeList&op=add"))      
      ) ;    
      
      //-----Add new Action Type that extends a Rule Action Type
      addWebUnit(
        new ClickURLLinkUnit("AddNewRuleActionType", "Click on add new Action button").
        setURL("&uicomponent=UIActionTypeList&op=add").                  
        addValidator(new ExpectTextValidator("Add/Edit a Action Type"))      
      ) ;
      //-----------Select Extends Action Type is Rule Action Type
      addWebUnit(
        new SubmitFormUnit("SelectExtendsActionType", "Select Extends Action Type is Rule Action Type").
        setFormId("selectactionTypeForm").   
        setField("actionType","exo:ruleAction").
        setField("op","changeActionType").      
        addValidator(new ExpectTextValidator("EnougthAdmins.java.drl"))      
      ) ;    
      //-----------Save new Rule Action Type    
      addWebUnit(
        new SubmitFormUnit("SaveNewRuleActionType", "Fill all info and click on save button").
        setFormId("actionTypeForm").   
        setField("name","exo:test-actionType"+i).
        setField("variables","exo:variable1").      
        setField("op","save").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionTypeList&op=add"))      
      ) ;
    }
    //Go back to ECM admin home
    addWebUnit(
      new ClickURLLinkUnit("BackToAdminECM", "Click on back button").      
      setURL("uicomponent=UIActionTypeList&op=back").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=createActionType"))      
    ) ;    
  }     
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}