/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * Test copy, cut and paste actions on newly created documents. This Suite
 * depends on the AddDocumentSuite one.      
 *     1) Go to production workspace
 *     2) Create /test-folder directory
 *     3) Go to  /test-folder
 *     4) Create /test-folder/test-doc
 *     5) Create /test-folder/test-folder2     
 *     6) Copy test-doc
 *     7) Go to  /test-folder/test-folder2
 *     8) Paste test-doc and check it is added
 *     9) Go back to /test-folder
 *    10) Create /test-folder/test-folder3
 *    11) Cut test-doc
 *    12) Go to  /test-folder/test-folder3
 *    13) Paste test-doc and check it is added
 *    14) Go back to /test-folder
 *    15) Check test-doc was deleted
 *    16) Save Session
 *    17) Go back to workspaces selection page
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserCopyPasteAndCutSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on test-folder and to do somethings" ;
  static String NAME = "UserCopyPasteAndCutSuite" ;
  String COMMENT  = "You must run AdminViewSuite and AdminDriveSuite before run this suite";
    
  public UserCopyPasteAndCutSuite() {    
    super(NAME, DESC) ;   
    addWebSuite(new UserAddDocumentSuite());      
    //Go to production 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").       
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ; 
    //-----Goto /home/users/#{client.name}
   addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}")
    ) ;
    //Select the test-folder
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder", "click on test-folder").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder").         
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder"))
    ) ; 
    //Create a /test-folder2 directory    
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedTest-folder2", "click delete button if test-folder2 existed").
      setURL("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder2").      
      addValidator(new ExpectTextValidator("The node /test-folder2 has been removed.")).
      addCondition(new HaveLinkWithURLCondition("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder2"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("CreateTest-folder2", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder2").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder2"))      
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }      
    //Copy test-doc
    addWebUnit(
      new ClickURLLinkUnit("CopyDocument", "click on copy button").
      setURL("&uicomponent=UIContentManager&op=copy&objectId=test-doc").         
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been copied."))         
    ) ;   
    //Go to /test-folder2
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder2", "click on test-folder2").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-folder2").        
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder")).      
      addValidator(new ExpectTextValidator("test-folder2"))
    ) ;       
    //Paste test-doc and check it is added
    addWebUnit(
      new ClickURLLinkUnit("PasteDocument", "click on paste button").
      setURL("&uicomponent=UIToolbar&op=paste").        
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been pasted at the location /home/users/#{client.name}/test-folder/test-folder2/test-doc."))      
    ) ;        
    //Go back to /test-folder
    addWebUnit(
      new ClickURLLinkUnit("BackToTest-folder", "click on test-folder link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home/users/#{client.name}/test-folder").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home"))                            
    ) ;       
    //Create /test-folder3
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedTest-folder3", "click delete button if test-folder3 existed").
      setURL("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder3").      
      addValidator(new ExpectTextValidator("The node /test-folder3 has been removed.")).
      addCondition(new HaveLinkWithURLCondition("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder3"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("CreateTest-folder3", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder3").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder3"))      
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    //Cut test-doc
    addWebUnit(
      new ClickURLLinkUnit("CutDocument", "click on cut button").
      setURL("&uicomponent=UIContentManager&op=cut&objectId=test-doc").         
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been cut."))         
    ) ;
    //Go to test-folder3
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder3", "click on test-folder3").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-folder3").      
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder")).
      addValidator(new ExpectTextValidator("test-folder3"))
    ) ;    
    //Paste test-doc and check it is added    
    addWebUnit(
      new ClickURLLinkUnit("PasteDocument", "click on paste button").
      setURL("&uicomponent=UIToolbar&op=paste").
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been pasted at the location /home/users/#{client.name}/test-folder/test-folder3/test-doc."))
    ) ;
    //Go back to test-folder and Check test-doc was deleted              
    addWebUnit(
      new ClickURLLinkUnit("BackToTest-folder", "click on test-folder link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home/users/#{client.name}/test-folder").     
      addValidator(new ExpectNoTextValidator("test-doc"))      
    ) ;    
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    //Return to drive selection
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}