/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Feb 28, 2006
 */
public class UserPublicationsSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, create a node in draft, control in workflow (by admin account), then check on prodution and backup" ;
  static String NAME = "UserPublicationsSuite" ;
  String COMMENT  = "You must run AdminViewSuite and AdminDriveSuite before run this suite";
    
  public UserPublicationsSuite() {    
    super(NAME, DESC) ;
    goToPage("/Home/ecm") ;
    //DRAFT-------------------------------------------
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDraft", "click on draft link").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=draft").            
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms"))            
    ) ;     
    //-----Goto home/users/   
    addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home").            
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))            
    ) ;   
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home/users").             
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}"))            
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCurrentUsers", "click on current users link").
        setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}").             
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Actions"))            
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickOnActionsTab", "click on Actions tab").
        setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").            
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))            
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("AddFolder", "click on add folder button").
        setURL("uicomponent=UIToolbar&op=addFolder").            
        addValidator(new ExpectFormIdValidator("UISelectViewForm")).
        addValidator(new ExpectFormIdValidator("folderForm"))
    ) ; 
    addWebUnit(
        new SubmitFormUnit("CreateNewFolder", "Fill all infomation and click on save button").
        setFormId("folderForm").   
        setField("folderType","nt:unstructured").
        setField("name","test-folder").      
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SelectCreatedFolder", "select test-folder link ").
        setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder").            
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=addDocument"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on add document button").
      setURL("&uicomponent=UIToolbar&op=addDocument").            
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;           
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is nt:file").
        setFormId("selectContentTypeForm").   
        setField("contentType","nt:file").
        setField("op","changeType").
        addValidator(new ExpectNoTextValidator("Image to upload")).
        addValidator(new ExpectTextValidator("File")).
        addValidator(new ExpectTextValidator("Content:"))
      ) ;
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("CreateNewDocument", "Fill all infomation and click on save button").
      setFormId("contentForm").   
      setField("contentName","test-doc").
      //setField("summary","content").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=editDocument&objectId=test-doc"))
    ) ;   
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }         
    //-----Login with admin account
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );     
    String[] roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ;   
    //----------Goto Workflow and to do something
    goToPage("/Home/Workflow") ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnTaskController", "click on task controller button").
      setURL("&uicomponent=null&op=selectNode&objectId=UITaskListController").  
      addValidator(new ExpectTextValidator("Manage"))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManage", "click on manage link").
      setURL("&uicomponent=TemplateRenderer&op=manageState&objectId=").        
      addValidator(new ExpectFormIdValidator("taskForm"))            
    ) ;    
    addWebUnit(
      new SubmitFormUnit("StartProcess", "Fill and submit the start process form").
      setFormId("taskForm").
      setField("startDate_dd", "1").
      setField("startDate_mm", "1").
      setField("startDate_yyyy", "2006").
      setField("startDate_hh", "1").
      setField("startDate_min", "1").        
      setField("endDate_dd", "1").
      setField("endDate_mm", "1").
      setField("endDate_yyyy", "2007").
      setField("endDate_hh", "1").
      setField("endDate_min", "1").        
      setField("op", "approve").      
      addValidator(new ExpectNoTextValidator("Approve"))
    );
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );    
    //----------Refresh session and check if exist test-doc
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on Refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").  
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly")).
      addValidator(new ExpectTextValidator("test-doc"))
    ) ; 
    //----------Select Drive
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;    
    //PRODUCTION---------------------------------------
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;  
    //-----Goto home/user    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home").            
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))            
    ) ;   
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home/users").             
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}"))            
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCurrentUsers", "click on current users link").
        setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}").             
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Actions"))            
    ) ; 
    //----------Refresh session and check if exist test-doc
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on Refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").  
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications/test-doc"))                           
    ) ; 
    //----------Select Drive
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;    
    //BACKUP--------------------------------------------
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackup", "click on Backup button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=backup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;  
    //-----Goto cms/publications  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCms", "click on cms link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms").            
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications"))            
    ) ;   
    addWebUnit(
      new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications").            
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications"))            
    ) ; 
    //----------Refresh session and check if exist test-doc
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on Refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").  
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))      
    ) ; 
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}