/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;

/**
 * Created by The eXo Platform SARL
 * Author : Hoa Nguyen Thi
 *          hoa2501@yahoo.com
 * Mar 17, 2006
 * Author : Dung Hoang
 *          jobdung@yahoo.com
 * July 6, 2006
 */
public class UserSearchSuite extends WebUnitSuite {
  static String DESC = "Go to Search of Production on ECM and test  basic actions " ;
  static String NAME = "UserSearchSuite" ;
  String COMMENT  = "You must run AdminViewSuite and AdminDriveSuite before run this suite";
  
  public UserSearchSuite() {    
    super(NAME, DESC) ;            
    addWebSuite(new UserAddFolderSuite());
    //Create a nt:file node, then select on that node 
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "Click on add document icon").
      setURL("&uicomponent=UIToolbar&op=addDocument").                  
      addValidator(new ExpectFormIdValidator("selectContentTypeForm"))      
    ) ;     
    addWebUnit(
      new SubmitFormUnit("SelectFile", "select document type is File").
      setFormId("selectContentTypeForm").   
      setField("contentType","nt:file").
      setField("op","changeType").
      addValidator(new ExpectNoTextValidator("Summary"))      
    ) ;   
    addWebUnit(
      new SubmitFormUnit("Save", "fill all info and click save button").
      setFormId("contentForm").
      setField("contentName","test-node-name").
      setField("title","test-node-title").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=editDocument&objectId=/home/users/#{client.name}/test-folder/test-node-name"))
    ) ;   
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }       
    //Simple Search
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsersLink", "click on users link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users")               
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnActionsTab", "click on Actions Tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=search"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnSearchIcon", "click on Actions Tab").
      setURL("&uicomponent=UIToolbar&op=search").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UISearch&op=selectNode&objectId=UISimpleSearch")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UISearch&op=selectNode&objectId=UIJCRAdvancedSearch")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UISearch&op=selectNode&objectId=UIQueryList")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UISearch&op=selectNode&objectId=UIResults"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SimpleSearch", "Click on  Simple Search button and search with any work").
      setURL("&uicomponent=UISearch&op=selectNode&objectId=UISimpleSearch").                 
      addValidator(new ExpectFormIdValidator("UISimpleSearch"))
    ) ; 
    addWebUnit(
      new SubmitFormUnit("SearchBlankWords", "Search with blank word properties").
      setFormId("UISimpleSearch").   
      setField("text","").
      setField("op","searchAction").
      addValidator(new ExpectTextValidator("Please type a word to search"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchWords", "Search with word is 'test-node-title'").
      setFormId("UISimpleSearch").   
      setField("text","test-node-title").     
      setField("op","searchAction").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIResults&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-node-name"))      
    ) ;    
    //Search with query    
    addWebUnit(
      new ClickURLLinkUnit("NewQuery", "Click on  New Query button to advanced search by query language  SQL or XPath").
      setURL("&uicomponent=UISearch&op=selectNode&objectId=UIJCRAdvancedSearch").                 
      addValidator(new ExpectFormIdValidator("UIJCRAdvancedSearch"))
    ) ; 
    //-----search by SQL
    addWebUnit(
      new SubmitFormUnit("SearchBySQL", "Search with Query language SQL").
      setFormId("UIJCRAdvancedSearch").   
      setField("select","sql").
      setField("query","select * from nt:base where jcr:path like '/cms/%'").                          
      setField("op","searchAction").
      addValidator(new ExpectTextValidator("/cms/publications"))      
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("NewQuery", "Click on  New Query button to advanced search by query language  SQL").
      setURL("&uicomponent=UISearch&op=selectNode&objectId=UIJCRAdvancedSearch").                 
      addValidator(new ExpectFormIdValidator("UIJCRAdvancedSearch"))
    ) ; 
    addWebUnit(
      new SubmitFormUnit("SaveSearchBySQL", "Save result search by Query language  sql").
      setFormId("UIJCRAdvancedSearch").
      setField("name","SearchBySQLTest").
      setField("select","sql").
      setField("query","select * from nt:base where jcr:path like '/cms/%'").
      setField("op","save").
      addValidator(new ExpectTextValidator("The query has been added")).  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UISearch&op=selectNode&objectId=UIQueryList"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("CheckStatusSavedSQL", "Click on  Save Queries button to check status of action save search result by Query language  sql").
      setURL("uicomponent=UISearch&op=selectNode&objectId=UIQueryList").                 
      addValidator(new ExpectTextValidator("select * from nt:base where jcr:path like '/cms/%'")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIQueryList&op=execute&objectId=/home/users/#{client.name}/queries/SearchBySQLTest"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("CheckResultSavedSQL", "Click on Execute icon to check result of action save search result by Query language  sql").
      setURL("uicomponent=UIQueryList&op=execute&objectId=/home/users/#{client.name}/queries/SearchBySQL").                     
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UISearch&op=selectNode&objectId=UIJCRAdvancedSearch")).
      addValidator(new ExpectTextValidator("/home/users/#{client.name}/test-folder/test-node-name"))
    ) ;     
    //-----search by XPath
    addWebUnit(
      new ClickURLLinkUnit("NewQuery", "Click on  New Query button to advanced search by Query language  sql").
      setURL("&uicomponent=UISearch&op=selectNode&objectId=UIJCRAdvancedSearch").                 
      addValidator(new ExpectFormIdValidator("UIJCRAdvancedSearch"))
    ) ; 
    addWebUnit(
      new SubmitFormUnit("SearchByXPathEmpty", "Search by Query language  xpath with query properties is empty").
      setFormId("UIJCRAdvancedSearch").  
      setField("select","xpath").
      setField("query","").
      setField("op","searchAction").
      addValidator(new ExpectTextValidator("A Repository Exception occured."))      
    ) ; 
    addWebUnit(
      new SubmitFormUnit("SaveSearchByXPath", "Save result search by Query language  xpath").
      setFormId("UIJCRAdvancedSearch").  
      setField("name","SearchByXPath").
      setField("select","xpath").
      setField("query","/jcr:root/cms//*").
      setField("op","save").
      addValidator(new ExpectTextValidator("The query has been added"))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("CheckStatusSavedXpath", "Click on  Save Queries button to check status of action save search result by Query language  xpath").
      setURL("uicomponent=UISearch&op=selectNode&objectId=UIQueryList").                 
      addValidator(new ExpectTextValidator("/jcr:root/cms//*")).
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIQueryList&op=execute&objectId=/home/users/#{client.name}/queries/SearchByXPath"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("CheckResultSavedXpath", "Click on  Save Queries button to check result of action save search result by Query language  xpath").
      setURL("uicomponent=UIQueryList&op=execute&objectId=/home/users/#{client.name}/queries/SearchByXPath").                 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UISearch&op=selectNode&objectId=UIJCRAdvancedSearch")).
      addValidator(new ExpectTextValidator("/cms"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("GoToSavedQueries", "Go to Saved queries to delete saved queries sql and xpath").
      setURL("uicomponent=UISearch&op=selectNode&objectId=UIQueryList").                 
      addValidator(new ExpectTextValidator("select * from nt:base where jcr:path like '/cms/%'")).
      addValidator(new ExpectTextValidator("/jcr:root/cms//*")).
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIQueryList&op=delete&objectId=/home/users/#{client.name}/queries/SearchBySQL")).
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIQueryList&op=delete&objectId=/home/users/#{client.name}/queries/SearchByXPath"))
    ) ; 
    //-----Delete saved queries
    addWebUnit(
      new ClickURLLinkUnit("DeleteSavedQuerySQL", "Delete saved query sql").
      setURL("uicomponent=UIQueryList&op=delete&objectId=/home/users/#{client.name}/queries/SearchBySQL").                 
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIQueryList&op=delete&objectId=/home/users/#{client.name}/queries/SearchBySQL",0)).
      addValidator(new ExpectNoTextValidator("SearchBySQL"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteSavedQueryXpath", "Delete saved query xpath").
      setURL("uicomponent=UIQueryList&op=delete&objectId=/home/users/#{client.name}/queries/SearchByXPath").                 
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIQueryList&op=delete&objectId=/home/users/#{client.name}/queries/SearchByXPath",0)).
      addValidator(new ExpectNoTextValidator("SearchByXPath"))
    ) ;         
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "click on select workspace icon").
      setURL("uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test"))      
    ) ;
  }                 
  public java.util.List getRequiredSuites() {    
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}