/**************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved. *
 * Please look at license.txt in info directory for more license detail.  *
 **************************************************************************/
package suites.portal;

import condition.* ;
import suites.share.* ;
import suites.share.UserLoginSuite;
import suites.share.UserLogoutSuite;
import suites.admin.*;

import org.exoplatform.test.*;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * Created by The eXo Platform SARL
 * Author : Hoa Nguyen Thi
 *          hoa2501@yahoo.com
 * Mar 13, 2006
 */
public class AdminCustomizationPortalForPortalSuite extends WebUnitSuite {  
  String COMMENT = "When you running portal and you must call PortletRegistryPortalSuite," +
      "if you running ECM you must call PortletRegistrySuite at line 29" ;
  
  public AdminCustomizationPortalForPortalSuite() {
    super("AdminCustomizationPortalForPortalSuite", "Add and edit in the portal mode") ;
    addWebSuite(new PortletRegistryPortalSuite());
    addWebUnit(CommonUnit.createClickEditPortalMode()) ;
    //Add a new container
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddContainer", "Click on add container icon of parent container").
      setURL("portal:action=addContainer").
      addValidator(new ExpectFormIdValidator("containerForm"))      
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnCancel", "Cancel add a new container").
        setFormId("containerForm").
        setField("op","PhaseId[1].cancel").
        addValidator(new ExpectNoTextValidator("Container-Test"))
    );
    addWebUnit(
        new ClickURLLinkUnit("DeleteIdExisted", "Delete if existed containerId").
        setURL("portal:componentId=ContainerTest&portal:action=delete").
        addCondition(new HaveLinkWithURLCondition("portal:componentId=ContainerTest&portal:action=delete"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnAddContainer", "Click on add container icon of parent container").
        setURL("portal:action=addContainer").
        addValidator(new ExpectFormIdValidator("containerForm"))      
    ) ;
    addWebSuite(new CreateEditContainerSuite("containerForm","ContainerTest",
        "Container-Test","default","default")) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditContainer", "Click on edit container icon of Container-Test container").
        setURL("portal:componentId=ContainerTest&portal:action=editProperty").
        addValidator(new ExpectFormIdValidator("containerForm"))      
    ) ;
    addWebSuite(new CreateEditContainerSuite("containerForm","ContainerTest",
        "Container-Test-Edited","default","default")) ;
    //-----Add a new Portlet
    addWebUnit(
        new ClickURLLinkUnit("ClickOnAddPortlet", "Click on edit container icon of Container-Test container").
        setURL("portal:componentId=ContainerTest&portal:action=addPortlet").
        addValidator(new ExpectTextValidator("Container-Test-Edited"))      
    ) ;
    addWebSuite(new AddPortletSuite("content","SearchPortlet")) ;
    //-----edit portlet
    addWebSuite(new EditPortletSuite("SearchPortlet")) ;
    //-----test view status
    addWebUnit(
        new ClickURLLinkUnit("ViewMinimizedPortlet", "View minimized SearchPortlet portlet ").
        setURL("portal:componentId=SearchPortlet&portal:windowState=minimized").
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=SearchPortlet&portal:windowState=normal")).      
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=SearchPortlet&portal:windowState=maximized"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ViewMaximizedPortlet", "View maximized SearchPortlet portlet ").
        setURL("portal:componentId=SearchPortlet&portal:windowState=maximized").
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=SearchPortlet&portal:windowState=minimized")).
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=SearchPortlet&portal:windowState=normal"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ViewNormalPortlet", "View normal SearchPortlet portlet ").
        setURL("portal:componentId=SearchPortlet&portal:windowState=normal").
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=SearchPortlet&portal:windowState=minimized")).
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=SearchPortlet&portal:windowState=maximized"))
    ) ;
    //Click on place the body on here icon of Container-Test-Edited after click on delete icon to test action of place the body on here icon
    addWebUnit(
        new ClickURLLinkUnit("PlaceBodyOnContainerTest", "Place ").
        setURL("portal:componentId=ContainerTest&portal:action=placePage").
        addValidator(new ExpectTextValidator("Container-Test-Edit"))      
    ) ;
    //Delete Container 
    addWebUnit(
        new ClickURLLinkUnit("DeleteContainerExisted", "Delete if existed containerId").
        setURL("portal:componentId=ContainerTest&portal:action=delete").
        addCondition(new HaveLinkWithURLCondition("portal:componentId=ContainerTest&portal:action=delete")).
        addValidator(new ExpectTextValidator("You cannot remove a container that contains the page"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("PlaceBodyOnContainerTest", "Place ").
        setURL("portal:action=placePage").
        addValidator(new ExpectTextValidator("Container-Test-Edit"))      
    ) ;
    //-----Delete Portlet 
    addWebUnit(
        new ClickURLLinkUnit("DeletePortletExisted", "Delete if existed portlerId is SearchPortlet").
        setURL("portal:componentId=SearchPortlet&portal:action=delete").
        addCondition(new HaveLinkWithURLCondition("portal:componentId=SearchPortlet&portal:action=delete")).
        addValidator(new ExpectNoTextValidator("portletUpdated"))
    ) ;
    //Delete Container
    addWebUnit(
        new ClickURLLinkUnit("DeleteContainerExisted", "Delete if existed containerId").
        setURL("portal:componentId=ContainerTest&portal:action=delete").
        addCondition(new HaveLinkWithURLCondition("portal:componentId=ContainerTest&portal:action=delete")).
        addValidator(new ExpectNoTextValidator("Container-Test-Edited"))      
    ) ;
  }  
  public java.util.List getRequiredSuites() {         
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;  
  } 
}